/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Nexus.clm.SettingsView', {
    extend: 'Ext.Panel',

    requires: [
      'Nexus.staging.repository.activity.EventDetail',
      'Nexus.staging.repository.SummaryView',
      'Nexus.staging.repository.RepositoriesMediator'
    ],

    requirejs: [
      'Nexus/siesta',
      'Nexus/clm'
    ],

    mixins: [
      'Nexus.LogAwareMixin'
    ],

    constructor: function (config) {
        var self = this;

        self.url = new Ext.form.TextField({
            name: 'url',
            fieldLabel: 'IQ Server URL',
            helpText: 'The address of your IQ Server',
            anchor : Sonatype.view.FIELD_OFFSET,
            allowBlank : false,
            itemCls : 'required-field',
            validator : function(v) {
              if (v === '' || v.match(/^(?:http|https):\/\//i))
              {
                return true;
              }
              else
              {
                return 'Protocol must be http:// or https://';
              }
            }
        });

        self.testConnectionStore = new Ext.data.JsonStore({
          id: 'clmTestConnection',
          fields: [{
            name: 'name',
            sortType: Ext.data.SortTypes.asUCString
          }],
          sortInfo: {
            field: 'name',
            direction: 'ASC'
          },
          data: []
        });

        self.testConnectionGrid = new Ext.grid.GridPanel({
          store: self.testConnectionStore,
          hidden: true,
          title: 'Available Applications',
          anchor: Sonatype.view.FIELD_OFFSET,
          height: 300,
          tools: [{
            type: 'close',
            handler: function(event, toolEl, panel) {
              panel.hide();
            }
          }],
          viewConfig: {
            autoFill: true,
            forceFit: true
          },
          colModel: new Ext.grid.ColumnModel({
            defaults: {
              sortable: true
            },
            columns: [{
              header: 'Application',
              dataIndex: 'name'
            }]
          })
        });
        
        self.settingsForm = new Ext.FormPanel({
          border : false,
          labelWidth : 100,
          layoutConfig : {
            labelSeparator : ''
          },
            items: [
                self.url,
                {
                  xtype: 'textfield',
                  name: 'username',
                  fieldLabel: 'Username',
                  helpText: 'User with access to the IQ Server.',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : false,
                  itemCls : 'required-field'
                },
                {
                  xtype: 'textfield',
                  name: 'password',
                  inputType: 'password',
                  fieldLabel: 'Password',
                  helpText: 'Credentials for the IQ Server user.',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : false,
                  itemCls : 'required-field'
                },
                {
                  xtype: 'numberfield',
                  name: 'timeout',
                  fieldLabel: 'Request Timeout',
                  helpText: 'Number of seconds to wait for a successful IQ Server connection before retrying. Defaults to 15 minutes.',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : true
                },
              {
                xtype: 'textarea',
                fieldLabel: 'Properties',
                name: 'properties',
                anchor : Sonatype.view.FIELD_OFFSET,
                helpText: 'Additional properties to configure for the IQ Server',
                flex: 1
              }
            ],
            buttons : [
              {
                text : 'Save',
                tooltip : 'Save settings',
                formBind : true,
                scope : self,
                handler : self.saveSettings
              },
              { text : 'Reset',
                tooltip : 'Reset changes',
                formBind : true,
                scope : this,
                handler : self.loadSettings
              },
              { text : 'Test Connection',
                tooltip : 'Test connection and permissions.',
                formBind : true,
                scope : self,
                handler : self.testConnection
              }
            ]
        });

      self.disabledWarning = NX.create('Ext.Panel', {
        border : false,
        html : '<div class="nx-clm-disabled-warning">'
            +     '<div>'
            +       'IQ Server functionality is currently disabled. To enable it, use the Capabilities UI.'
            +     '</div>'
            + '</div>',
        hidden: true
      });

      Ext.apply(self, {
          cls: 'nx-clm-SettingsView',
          border: false,
          layout: 'vbox',
          layoutConfig: {
              align: 'stretch'
          },
          items: [
              {
                  xtype: 'fieldset',
                  title: 'Settings',
                  anchor : Sonatype.view.FIELDSET_OFFSET,
                  autoHeight : true,
                  items: [
                    {
                      border : false,
                      html : '<div class="nx-clm-doco">'
                            +   '<a href="http://links.sonatype.com/products/clm/nexus/home"  target="_blank">'
                            +   'Make Nexus Staging even more powerful with Nexus IQ Server'
                            +   ' </a>'
                            + '</div>'
                    },
                    self.disabledWarning,
                    self.settingsForm,
                    self.testConnectionGrid
                  ]
              }
          ]
      });

      self.constructor.superclass.constructor.apply(self, arguments);

      // Load settings after layout, has to be done post superclass.constructor()
      self.on('afterlayout', self.loadSettings, self, {
        single: true
      });

    },

    saveSettings : function() {
      var self = this,
          form, url;

      self.logDebug('IQ Server saving settings');

      form = self.settingsForm.getForm();

      if(!form.isValid())
      {
        return;
      }

      url = form.findField('url').getValue();

      Ext.Ajax.request({
        url : Nexus.clm.resourcePath,
        method : 'POST',
        scope : self,
        suppressStatus: 400,
        jsonData : form.getFieldValues(),

        success : function() {
          var nexusPanel = Sonatype.repoServer.RepoServer.nexusPanel;
          Nexus.clm.showMessage('Configuration saved');
          form.items.each(function(item){
            item.clearInvalid();
          });
          nexusPanel.get('st-enterprise-staging').remove(nexusPanel.clmMenu);
          Nexus.clm.serverUrl = url;
          nexusPanel.clmMenu = nexusPanel.add({
            enabled: true,
            sectionId: 'st-enterprise-staging',
            title: Nexus.clm.TITLE_DASHBOARD,
            href: Nexus.clm.serverUrl
          });
          nexusPanel.doLayout();
        },

        failure: function(response, opts){
          if(response.siestaValidationError !== undefined)
          {
            Ext.each(response.siestaValidationError, function (error){
              form.findField(error.id).markInvalid(error.message);
            });
          }
        }
      });
    },

    loadSettings : function() {
      var self = this;

      self.logDebug('IQ Server loading settings');

      Ext.Ajax.request({
        url :  Nexus.clm.resourcePath,
        method : 'GET',
        scope : self,
        suppressStatus: 404,
        success : function(response) {
          var config = Ext.decode(response.responseText);
          self.settingsForm.getForm().setValues(config);
          if(config.enabled){
            self.disabledWarning.hide();
          }
          else {
            self.disabledWarning.show();
          }
          self.logDebug('IQ Server settings loaded');
        }
      });
    },

    testConnection: function() {
      var self = this, form, url;

      self.logDebug('Testing IQ Server connection.');

      form = self.settingsForm.getForm();

      if (!form.isValid()) {
        return;
      }

      Ext.Ajax.request({
        url: Nexus.clm.testConnectionPath,
        method: 'POST',
        scope: self,
        suppressStatus: 400,
        jsonData: form.getFieldValues(),
        success: function(response) {
          self.testConnectionGrid.show();
          self.testConnectionStore.loadData(JSON.parse(response.responseText));
        },
        failure: function(response, opts) {
          self.testConnectionGrid.hide();
          if (response.siestaValidationError !== undefined) {
            Ext.each(response.siestaValidationError, function(error) {
              form.findField(error.id).markInvalid(error.message);
            });
          }
        }
      });
    }
},

function () {
    var type = this;
    var key = Nexus.clm.STAGING_KEY;
    var label = Nexus.clm.STAGING_LABEL;

    // Attach SettingsView to navigation panel if current user has appropriate permissions
    Sonatype.Events.addListener('nexusNavigationInit', function (nexusPanel) {
      var p = Sonatype.lib.Permissions;
      if (p.checkPermission('nexus:settings', p.READ)) {
        nexusPanel.add({
          enabled: true,
          sectionId: 'st-nexus-config',
          title: Nexus.clm.TITLE,
          tabId: Nexus.clm.TAB_ID,
          tabTitle: Nexus.clm.TAB_TITLE,
          tabCode: type
        });
        Ext.Ajax.request({
          url :  Nexus.clm.resourcePath,
          method : 'GET',
          scope : self,
          suppressStatus: 404,
          success : function(response) {
            var clmSettings = Ext.decode(response.responseText);
            if (!Ext.isEmpty(clmSettings.url)) {
              Nexus.clm.serverUrl = clmSettings.url;
              nexusPanel.clmMenu = nexusPanel.add({
                enabled: true,
                sectionId: 'st-enterprise-staging',
                title: Nexus.clm.TITLE_DASHBOARD,
                href: Nexus.clm.serverUrl
              });
              nexusPanel.doLayout();
            }
          }
        });
      }
    });

  //Enhance StagingProfile UI model with IQ Server metadata. Works in conjunction with the staging.profile.submitted event.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.profile.loaded', function(items, properties, dataRoot) {
    var clmAppPolicyURL;
    NX.log.debug("Enhancing UI with IQ Server details.");
    clmAppPolicyURL = new Ext.form.DisplayField({
      anchor: Sonatype.view.FIELD_OFFSET,
      allowBlank: true,
      hidden: !Ext.isDefined(Nexus.clm.serverUrl),
      appId: properties[key],
      showClmAppUrl: function () {
        var appId = clmAppPolicyURL.appId,
            url = Nexus.clm.serverUrl,
            text = 'IQ Server Dashboard';

        if (!Ext.isEmpty(appId)) {
          while (url.endsWith('/')) {
            url = url.substring(0, url.length - 1);
          }
          url += '/ui/links/application/' + encodeURIComponent(appId) + '/management';
          text = 'IQ Server Application Management';
        }
        clmAppPolicyURL.setValue('<a href="' + url + '" target="_blank">' + text + '</a>');
      }
    });
    clmAppPolicyURL.showClmAppUrl();
    items.push([
      new Ext.form.ComboBox({
        fieldLabel: label,
        name: dataRoot + '.' + key,
        value: properties[key],
        width: 376,
        helpText: 'Select an IQ Server Application to enable, or (none) to disable policy enforcement for this profile.<br/>'
            + '<a href="http://links.sonatype.com/products/clm/nexus/home"  target="_blank">'
            + 'Make Nexus Staging even more powerful with Nexus IQ Server'
            + ' </a>',
        persistent: true,
        store: new Ext.data.JsonStore({
          id: 'id',
          fields: [
            { name: 'id'},
            { name: 'name', sortType: Ext.data.SortTypes.asUCString }
          ],
          sortInfo: {
            field: 'name',
            direction: 'ASC'
          },
          url: Nexus.clm.resourcePath + "/applications",
          autoLoad: true,
          listeners: {
            load: function (store) {
              store.insert(0, NX.create('Ext.data.Record', {id: '', name: '(none)'}));
            }
          }
        }),
        displayField: 'name',
        valueField: 'id',
        editable: false,
        forceSelection: true,
        mode: 'local',
        triggerAction: 'all',
        emptyText: 'Select...',
        selectOnFocus: true,
        allowBlank: true,
        listeners: {
          select: function (combo, record) {
            clmAppPolicyURL.appId = record.get('id');
            clmAppPolicyURL.showClmAppUrl();
          }
        }
      }),
      clmAppPolicyURL
    ]);
  });

  //Enhance StagingProfile UI model to serialize IQ Server metadata. Works in conjunction with the staging.profile.loaded event.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.profile.submitted', function(referenceData) {
    NX.log.debug("Enhancing StagingProfile submit data with IQ Server details.");
    referenceData[key] = null;
  });

  /**
   * @param value The encoded value with details of last IQ Server result.
   * @param showDetailedReport Whether or not a link to the full IQ Server results should be included
   * @return {*} view of the last IQ Server result.
   */
  function renderClmValues(value, showDetailedReport) {
    var values = value.split(',');
    NX.assert(values.length == 6, "Should have 6 comma separated values stored for IQ Server violations.");
    return Nexus.clm.chickletTemplate.apply({
      criticalCount : values[2],
      severeCount : values[3],
      moderateCount : values[4],
      detailedReport : showDetailedReport ? values[5] : null
    });
  }

  // enhance StagingRepository grid view with an additional column
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.repositoriesGrid.configureColumns', function(columns) {
    NX.log.debug("Enhancing Repository grid view with IQ Server details.");
    columns.push({
      id : 'violations',
      width : 250,
      header : 'IQ Policy Violations',
      renderer : function(value, metaData, record) {
        var properties = record.data.properties;
        if (properties !== undefined) {
          var clmProperty;
          Ext.each(properties, function(property) {
            if (property.name == "IQ Policy Violations") {
              clmProperty = property.value;
              return false;
            }
          });
          if(clmProperty !== undefined)
          {
            return renderClmValues(clmProperty, false);
          }
        }
        return 'No IQ Server data recorded';
      }
    });
  });

  //Register renderer for IQ Server summary in StagingRepository summary view.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.repositorySummary.configureProperties', function(properties, values){
    Ext.each(values.properties, function(property){
      if(property.name && property.value && property.name === 'IQ Policy Violations')
      {
        NX.log.debug("Enhancing Repository summary view with IQ Server details.");
        for(var i = 0; i <  properties.length; i++)
        {
          if(properties[i].name === 'Activity')
          {
            break;
          }
        }
        properties.splice(i+1, 0, { name: 'IQ Policy Violations', value: renderClmValues(property.value, true) });
        return false;
      }
    });
  });

  //Register renderers for IQ Server rules in the Staging UI
  Nexus.staging.repository.activity.EventDetail.registerBodyTemplateFactory('ruleFailed', Nexus.clm.stagingRule);
  Nexus.staging.repository.activity.EventDetail.registerBodyTemplateFactory('rulePassed', Nexus.clm.stagingRule);
});
