/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/healthcheck/HealthCheckArtifactDetailsContrib', [ 'Nexus/healthcheck/HealthCheckUtil', 'Nexus/healthcheck/HealthCheckProUtil', 'Nexus/clm/ComponentDetailView' ], function() {

  HealthCheckArtifactDetailsContrib = function() {
      function getInsight(panel) {
        if (panel && panel.items.getCount() === 2) {
          return panel.items.itemAt(1).iframe.dom.contentWindow.Insight;  
        }
      }
      
      var me = this;

      me.getFormat = function() {
        if (me.data && me.data.format) {
          if (me.data.format === 'maven2') {
            return 'maven';
          } else if (me.data.format === 'nuget') { 
            return 'nuget'; 
          }
        }
        return null;
      };
      
      me.getLoadingHtml = function() {
          return '<div style="background:#ececec; width: 100%; height: 100%; padding: 20px;font-size: 14px;font-family: Arial, Helvetica, sans-serif">Loading...</div>';
      };

      me.getFileUrl = function() {
          var url = Sonatype.HealthCheckUtil.getBase() + '/healthCheckFileDetail/' + Sonatype.utils.base64.encode(me.data.resourceURI) + '/index.html?repoId=' + me.data.repoId;
          var format = me.getFormat();
          if (format) { 
            return url + '&format=' + format; 
          }
          return url;
      };

      me.getHashUrl = function(repoId, itemPath) {
        return Sonatype.HealthCheckUtil.getBase() + '/healthCheckFileDetail/hash/' + repoId + '/' + itemPath;
      };
      
      me.getComponentIdentifierUrl = function(format, itemPath) {
        return Sonatype.HealthCheckUtil.getBase() + '/healthCheckFileDetail/componentIdentifier/' + format + '/' + itemPath;
      };

      me.toTabId = function(appId, hash, component, legacy) {
        var result = 'compDetail' + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(appId);
        result += Sonatype.view.HISTORY_DELIMITER + (hash ? encodeURIComponent(hash) : '');
        result += Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(legacy);
        result += Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(component.format);
        
        var coordinates = [];
        var key = null;
        if (component.coordinates) {
          for (key in component.coordinates) {
            if (component.coordinates.hasOwnProperty(key)) {
              coordinates.push(key);
            }
          }
        }
        coordinates.sort();
        
        for (var i = 0; i < coordinates.length; i++) {
          key = coordinates[i];
          var coordinateValue = component.coordinates[key];
          result += Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(key);
          result += Sonatype.view.HISTORY_DELIMITER + (coordinateValue ? encodeURIComponent(coordinateValue) : '');
        }
        
        return result;
      };

      me.fromTabId = function(tokens) {
        var result = {};
        result.appId = decodeURIComponent(tokens[1]);
        result.hash = decodeURIComponent(tokens[2]);
        result.legacy = decodeURIComponent(tokens[3]) === 'true';
        result.component = {};
        result.component.coordinates = {};
        result.component.format = decodeURIComponent(tokens[4]);
        for (var i = 5; i < tokens.length; i += 2) {
          result.component.coordinates[decodeURIComponent(tokens[i])] = decodeURIComponent(tokens[i + 1]);
        }
        return result;
      };

      me.isLegacyClm = function(panel) {
        if (getInsight(panel).registerCoordsViewDetailsListener || getInsight(panel).setCoordinates) { 
          return false;
        }
  
        return true;
      };
      
      me.showDetailsTab = function(appId, hash, component, legacy) {
        Sonatype.view.mainTabPanel.addOrShowTab(me.toTabId(appId, hash, component, legacy),
                Nexus.clm.ComponentDetailView, {
                  title: 'Component Details',
                  appId: appId,
                  hash: hash,
                  component: !legacy ? component : null,
                  gav: legacy ? component.coordinates : null
                });
      };
      
      me.registerLegacyClmViewDetailsListener = function(panel) {
        getInsight(panel).registerViewDetailsListener(
                function(appId, groupId, artifactId, version, classifier, extension, hash, matchState,
                        proprietary) {

                  var component = {};
                  component.format = 'maven';
                  component.coordinates = {
                    groupId: groupId,
                    artifactId: artifactId,
                    version: version,
                    classifier: classifier,
                    extension: extension
                  };

                  me.showDetailsTab(appId, hash, component, true);
                });
      };

      me.registerClmViewDetailsListener = function(panel) {
        if (!me.isLegacyClm(panel)) {
          getInsight(panel).registerCoordsViewDetailsListener(
                  function(appId, format, coordinates, hash, matchState) {

                    var component = {};
                    component.format = format;
                    component.coordinates = {};
                    if (coordinates) {
                      for (var i = 0; i < coordinates.length; i += 2) {
                        component.coordinates[coordinates[i]] = coordinates[i + 1];
                      }
                    }

                    me.showDetailsTab(appId, hash, component, false);
                  });
        } else {
          me.registerLegacyClmViewDetailsListener(panel);
        }
      };
      
      me.setAuthFailureHandler = function(panel) {
        getInsight(panel).setAuthFailureHandler(function () {
          Sonatype.repoServer.RepoServer.loginHandler();
        });
      };
      
      me.isClmLoaded = function(panel) {
        return getInsight(panel);
      };

      me.getItemPath = function() {
        if (me.data && me.data.resourceURI) {
          var segments = me.data.resourceURI.split('/');

          var index = segments.indexOf('content');
          if (index <= -1 || index + 1 >= segments.length) {
            return null;
          }

          return segments.slice(index + 1).join('/');
        }

        return null;
      };
      
      me.setCoordinates = function(itemPath, setCipCoordinates) {
        var format = me.getFormat();

        if (itemPath && me.data && format) {
          Ext.Ajax.request({
            url: me.getComponentIdentifierUrl(format, itemPath),
            success: function(result) {
              var componentIdentifier = JSON.parse(result.responseText);
              setCipCoordinates(itemPath, format, componentIdentifier.coordinates);
            },
            failure: function(result) {
              setCipCoordinates(itemPath, format, null);
            }
          });
        } else {
          setCipCoordinates(itemPath, format, null);
        }
      };

      me.setLegacyClmComponent = function(panel, format, coordinates, hash) {
        if (format !== 'maven') {
          var error = {
            errorMessage: 'Unsupported repository format, please upgrade to at least CLM 1.13.',
            errorCode: 400
          };
          getInsight(panel).setError(error);
          return;
        }

        var gav = {
          groupId: coordinates.groupId,
          artifactId: coordinates.artifactId,
          version: coordinates.version,
          extension: coordinates.extension,
          classifier: coordinates.classifier,
          hash: hash
        };
        
        getInsight(panel).setGav(gav);
      };
          
      me.setClmComponent = function(panel) {
        var setCipCoordinates = function(itemPath, format, coordinates) {
          if (coordinates && me.data && me.data.repoId && format) {
            Ext.Ajax.request({
              url: me.getHashUrl(me.data.repoId, itemPath),
              success: function(result) {
                if (me.isClmLoaded(panel)) {
                  if (!me.isLegacyClm(panel)) {
                    getInsight(panel).setCoordinates(format, coordinates, {
                      hash: result.responseText
                    });
                  } else {
                    me.setLegacyClmComponent(panel, format, coordinates, result.responseText);
                  }
                }
              },
              failure: function(result) {
                if (me.isClmLoaded(panel)) {
                  var errorMessage = 'Unable to retrieve hash for component.';
                  var error = {
                    errorMessage: (result.siestaError ? result.siestaError.message : errorMessage),
                    errorCode: result.status
                  };
                  getInsight(panel).setError(error);
                }
              }
            });
          } else {
            var errorMessage = 'Unable to retrieve component information.';
            var errorCode = 500;

            if (me.data && !format) {
              errorMessage = 'Unsupported repository format: ' + me.data.format + '.';
              errorCode = 400;
            } else if (!coordinates) {
              errorMessage = 'Unable to parse repository path "' + itemPath + '" into valid coordinates.';
            }

            var error = {
              errorMessage: errorMessage,
              errorCode: errorCode
            };
            var insightPanel = getInsight(panel);
            if (insightPanel) {
              insightPanel.setError(error);
            }
          }
        };

        me.setCoordinates(me.getItemPath(), setCipCoordinates);
      };

      me.buildLoadingPanel = function() {
          return new Ext.Panel({
              width : '100%',
              height : '100%',
              border : true,
              collapsible : false,
              collapsed : false,
              html : me.getLoadingHtml()
          });
      };

      me.buildDetailPanel = function(config) {
          var config = config || {};
          var defaultConfig = {
              width : '100%',
              height : '100%',
              border : true,
              collapsible : false,
              collapsed : false,
              layout : 'card',
              items : [ me.buildLoadingPanel() ]
          };

          Ext.applyIf(config, defaultConfig);
          return new Ext.Panel(config);
      };

      me.rebuildIframe = function(panel, url) {
          if (me.isClmLoaded(panel)) {
              me.setClmComponent(panel);
          } else {
              //item 0 is the loading panel, don't ever remove that one
              panel.getLayout().setActiveItem(0);

              var old = panel.items.get(1);
              if (old) {
                  panel.remove(old);
                  old.destroy();
              }

              panel.add({
                xtype: 'iframepanel',
                border: false,
                disableMessaging: false,
                defaultSrc: url,
                listeners: {
                  domready: function() {
                    panel.getLayout().setActiveItem(1);

                    var count = 0;
                    var functionId = setInterval(function(){
                      if (me.isClmLoaded(panel)) {
                        me.setClmComponent(panel);
                        me.registerClmViewDetailsListener(panel);
                        me.setAuthFailureHandler(panel);
                        clearInterval(functionId);
                      } else if (count > 100) {
                        //just give up, been long enough i think
                        clearInterval(functionId);
                      }
                      count++;
                    }, 100);
                  }
                }
              });

              panel.doLayout();
          }
      };

      me.viewDetails = function() {
        if (me.detailPanel && me.data && me.data.resourceURI) {
          me.rebuildIframe(me.detailPanel, me.getFileUrl());
        }
      };

      me.allowView = function() {
          if (!Sonatype.HealthCheckProUtil.hasAlertReadPerm() || !Sonatype.HealthCheckProUtil.hasPaidProLicense()) {
              return false;
          }

          return true;
      };

      me.showListener = function() {
          me.shown = true;

          if (!me.dataDisplayed && me.data) {
              me.dataDisplayed = true;
              me.viewDetails();
          }
      };

      me.hideListener = function() {
          me.shown = false;
      };

      Sonatype.Events.addListener('fileContainerInit', function(items) {
          if (me.allowView()) {
              me.filePanel = me.buildDetailPanel({
                  name : 'healthCheckFileDetailPanel',
                  tabTitle : 'Component Info',
                  preferredIndex : 80,
                  listeners : {
                      show : me.showListener,
                      hide : me.hideListener
                  }
              });
              items.push(me.filePanel);
          }
      });

      Sonatype.Events.addListener('fileContainerUpdate', function(artifactContainer, data) {
          if (me.allowView()) {
              me.detailPanel = me.filePanel;
              me.data = data;
              me.dataDisplayed = false;

              if (me.shown) {
                  me.dataDisplayed = true;
                  me.viewDetails();
              }
          }
      });

      Sonatype.Events.addListener('artifactContainerInit', function(items) {
          if (me.allowView()) {
              me.artifactPanel = me.buildDetailPanel({
                  name : 'healthCheckArtifactDetailPanel',
                  tabTitle : 'Component Info',
                  preferredIndex : 80,
                  listeners : {
                      show : me.showListener,
                      hide : me.hideListener
                  }
              });
              items.push(me.artifactPanel);
          }
      });

      Sonatype.Events.addListener('artifactContainerUpdate', function(artifactContainer, data) {
          if (me.allowView()) {
              me.detailPanel = me.artifactPanel;
              me.data = data;
              me.dataDisplayed = false;

              if (me.shown) {
                  me.dataDisplayed = true;
                  me.viewDetails();
              }
          }
      });

      Sonatype.Events.addListener('historyChanged', function(token) {
        if (token) {
          var toks = token.split(Sonatype.view.HISTORY_DELIMITER);
          if (toks[0] == 'compDetail') {
            if (Sonatype.user.curr.isLoggedIn) {
              if (me.allowView()) {
                var params = me.fromTabId(toks);
                me.showDetailsTab(params.appId, params.hash, params.component, params.legacy);
              }
            } else {
              Sonatype.view.afterLoginToken = token;
              Sonatype.repoServer.RepoServer.loginHandler();
            }
            // was handled by this event listener so return false to stop others
            return false;
          }
        }
      });
    };

  Ext.onReady(function() {
      Sonatype.HealthCheckArtifactDetailsContrib = new HealthCheckArtifactDetailsContrib();
  });
});
