/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define, Ext, Nexus, Sonatype, NX*/
NX.define('Nexus.ssl.AbstractCertificateTab', {
  extend : 'Ext.Panel',

  requires : ['Nexus.ssl.CertificateStore', 'Nexus.ssl.certificate.CertificateDetailsView'],

  requirejs : ['Nexus/config'],

  /**
   * default value, overridable
   */
  type : 'repository',

  trustKeyUrl : function() {
    return Sonatype.config.contextPath + '/service/siesta/ssl/truststore/key/' + this.type + '/' + this.data.id;
  },

  initComponent : function() {
    var self = this;

    self.certificateUrl = Sonatype.config.contextPath + '/service/siesta/ssl/certificates';
    self.trustUrl = Sonatype.config.contextPath + '/service/siesta/ssl/truststore';

    self.layout = 'fit';

    self.items = [
      {
        xtype : 'panel',
        autoScroll : true,
        layout : {
          type : 'vbox',
          align : 'stretch',
          defaultMargins : {
            top : 0,
            left : 20,
            right : 20,
            bottom :  0
          }
        },
        items : [
          {
            xtype : 'panel',
            layout : 'hbox',
            items : [
              {
                xtype : 'checkbox',
                name : 'use-truststore',
                flex : 0,
                listeners : {
                  check : function(cmp, checked) {
                    if (this.submitState) {
                      self.setUseTrustStore(checked);
                    }
                  },
                  render : function() {
                    // 'check' is also trigger when #setValue() is used on load - so we only submit if the checkbox is
                    // visible
                    this.submitState = true;
                  }
                }
              },
              {
                xtype : 'label',
                text : 'Use Nexus SSL trust store',
                style : 'padding-top: 5px;' // label needs to be centered to the checkbox
              },
              {
                xtype : 'panel',
                html : '&nbsp;',
                flex : 1
              },
              {
                xtype : 'button',
                name : 'trust-certificate',
                text : 'Add to trust store',
                width : 150,
                disabled : true,
                handler : function() {
                  if ( !self.certificate ) {
                    throw new Error('No certificate available');
                  }

                  var
                        btn = this,
                        record = Nexus.ssl.CertificateStore.getById(self.certificate.id);

                  if (record) {
                    Sonatype.MessageBox.show({
                      animEl : this.getEl(),
                      title : 'Nexus needs to be restarted',
                      msg : 'You need to restart Nexus to make sure<br/> the certificate is not used anymore.',
                      buttons : Sonatype.MessageBox.OK,
                      scope : this,
                      icon : Sonatype.MessageBox.WARNING,
                      fn : function(btnName) {
                        if (btnName === 'yes' || btnName === 'ok') {
                          Nexus.ssl.CertificateStore.remove(record);
                          btn.setAddText();
                        }
                      }
                    });
                  } else {
                    self.addCertificate(self.certificate.pem, function() {
                      btn.setRemoveText();
                      Nexus.ssl.CertificateStore.reload();
                    });
                  }

                },
                setAddText : function() {
                  this.setText('Add to trust store');
                },
                setRemoveText : function() {
                  this.setText('Remove from trust store');
                },
                listeners : {
                  show : function() {
                    if (!self.certificate) {
                      return;
                    }

                    if (Nexus.ssl.CertificateStore.getById(self.certificate.id)) {
                      this.setRemoveText();
                    }
                  }
                }

              }
            ]
          },
          {
            xtype : 'panel',
            name : 'certificate-details',
            title : 'Certificate',
            autoScroll : true,
            flex : 1,
            items : [
              {
                // full layout with empty values, to get the right size
                xtype : 'certificate-details',
                certificate : {}
              }
            ]
          }
        ]
      }
    ];

    Ext.Panel.prototype.initComponent.apply(self, arguments);

    self.on('activate', function() {
      self.loadSettings();
      self.loadCertificate();
    },  self, {single:true});
  },

  loadSettings : function() {
    var self = this;
    Ext.Ajax.request({
      url : this.trustKeyUrl(),
      success : function(response) {
        var data = Ext.decode(response.responseText),
              checkbox = self.find('name', 'use-truststore')[0];

        checkbox.submitState = false;
        checkbox.setValue(data.enabled);
        checkbox.submitState = true;
      }
    });

  },

  setUseTrustStore : function(enable) {
    var
          self = this,
          mask = new Ext.LoadMask(self.el, {
            msg : 'Saving settings...',
            removeMask : true
          });

    mask.show();

    Ext.Ajax.request({
      url : self.trustKeyUrl(),
      method : 'PUT',
      jsonData : {
        enabled : enable
      },
      callback : function() {
        mask.hide();
      }
    });
  },
  addCertificate : function(pem, callback) {
    var self = this;

    Ext.Ajax.request({
      url : self.trustUrl,
      jsonData : {
        value : pem
      },
      callback : callback
    });
  },

  loadCertificate : function() {
    var
          self = this,
          mask = { // mask is a noop if details is not displayed yet (refresh seems to trigger that)
            hide : Ext.emptyFn
          },
          button = self.find('name', 'trust-certificate')[0],
          parent = self.findParentByType('tabpanel'),
          details = self.find('name', 'certificate-details')[0];

    if (details.el) { // only use a real mask if details is already rendered
      mask = new Ext.LoadMask(details.el, {
        msg : 'Retrieving certificate...',
        removeMask : true
      });
      mask.show();
    }

    self.certificate = undefined;
    button.disable();

    Ext.Ajax.request({
      method : 'GET',
      url : self.certificateUrl,
      params : self.certRequestParams,
      callback : function() {
        mask.hide();
      },
      success : function(response) {
        details.removeAll();

        self.certificate = Ext.decode(response.responseText);

        if (self.certificate.inNexusSSLTrustStore) {
          button.setRemoveText();
        } else {
          button.setAddText();
        }
        button.enable();

        details.add(new Nexus.ssl.certificate.CertificateDetailsView({
          certificate : self.certificate
        }));

        // need to layout the tabpanel to update the content height, scrollbars were "too short" otherwise
        parent.doLayout();
      },
      failure : function(response) {
        Ext.MessageBox.hide();

        details.removeAll();
        self.certificate = undefined;
        button.disable();

        if ( response.siestaError ) {
          details.add(new Nexus.ssl.certificate.CertificateDetailsView.RETRIEVE_ERROR( response.siestaError.message ));
        } else if ( response.siestaValidationErrors ) {
          var msg = '';
          Ext.each(response.siestaValidationErrors, function(item) {
            msg += item.message + "<br/>";
          });
          details.add(new Nexus.ssl.certificate.CertificateDetailsView.RETRIEVE_ERROR( msg ));
        } else {
          details.add(Nexus.ssl.certificate.CertificateDetailsView.COULD_NOT_RETRIEVE);
        }

        parent.doLayout();
      }
    });

  },

  refreshContent : function() {
    this.loadSettings();
    this.loadCertificate();
  }
});
