/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/
/**
 * Abstract base class for PEM/fetch from server view
 */
NX.define('Nexus.ssl.certificate.FetchCertificateView', {
  extend : 'Ext.Panel',

  requires : ['Nexus.ssl.certificate.AddCertificateView'],
  requirejs : ['Sonatype/utils', 'Nexus/config'],

  buttonAlign : 'center',
  frame : true,

  asWindow : function(title) {
    var self = this, window;

    window = new Ext.Window({
      title : title,
      width : 450,
      height : self.windowHeight,
      layout : 'fit',
      onEsc : function() {
        window.close();
      },
      items : [ self ]
    });

    self.on('added', function() {
      window.close();
    });

    return window;
  },

  initComponent : function() {
    var self = this;

    self.buttons = [
      {
        text : 'Load Certificate',
        handler : self.onLoad,
        scope : self,
        disabled : true
      }
    ];

    self.items.unshift(
          {
            xtype : 'panel',
            name : 'message',
            anchor : '100%',
            html : self.messageTemplate.apply({type : 'text', icon : self.defaultMsgIcon, msg : self.defaultMsg})
          }
    );

    // FIXME some problem with calling self.constructor.... when subclass overrides this method as well (circular)
    Ext.Panel.prototype.initComponent.apply(self, arguments);
  },

  onLoad : function() {
    var self = this;

    self.loadmask = new Ext.LoadMask(self.el, {msg : "Loading certificate...", removeMask : true});
    self.loadmask.show();
    self.fetchPEM(Ext.createDelegate(self.showCertificate, self));
  },

  /**
   * Get certificate details from the PEM->Certificate REST resource.
   *
   * @param data {Object} configuration object having eiter full certificate details or a PEM.
   */
  showCertificate : function(data) {
    var
          self = this,
          details = self.find('name', 'certificate-details')[0],
          mask = new Ext.LoadMask(self.el, {
            msg : 'Fetching certificate...'
          });

    if (self.loadmask) {
      self.loadmask.hide();
    }

    if (data.id) { // we already have the certificate
      self.certificate = data;
      self.createCertificateDetailView(data).show();
      self.findParentByType('window').close();
    } else if (data.pem) { // need to get the details via REST
      mask.show();

      Ext.Ajax.request({
        url : Sonatype.config.contextPath + '/service/siesta/ssl/certificates/details',
        defaultErrorMessage : 'Could not add certificate.',
        suppressStatus : 400,
        jsonData : {
          value : data.pem
        },
        success : self.confirmCertificate(mask),
        failure : function(response) {
          mask.hide();
          var message = "Could not add certificate to Nexus<br/>";

          if (response.siestaError) {
            message += response.siestaError.message;
          } else if (response.siestaValidationError) {
            Ext.each(response.siestaValidationError, function(error) {
              // only one field to submit here, pem
              message += error.message + "<br/>";
            });
          } else {
            message += Sonatype.utils.parseHTMLErrorMessage(response.responseText);
          }

          self.showErrorMessage(message);
        }
      });
    }
  },

  messageTemplate : new Ext.XTemplate(
        '<div class="nx-ssl-fetchcert-message-icon">',
        '{[Nexus.ssl.Icons.get(values.icon).img]}',
        '</div>',
        '<div class="nx-ssl-fetchcert-message-{type}">',
        '{msg}',
        '</div>'
  ),

  showMessage : function(message, type) {
    if (!type) {
      type = 'text';
    }

    var self = this, panel = self.find('name', 'message')[0];
    // guard here, user may close the window before REST call is done
    if (panel && panel.getEl()) {
      self.messageTemplate.overwrite(panel.getEl(), {icon : 'fetch-from-server-failed', msg : message, type : type});
    }
    if (self.loadmask) {
      self.loadmask.hide();
    }

  },

  showErrorMessage : function(message) {
    this.showMessage(message, 'error');
  },

  confirmCertificate : function(mask) {
    var self = this;
    return function(response) {

      if (mask) {
        mask.hide();
      }

      self.certificate = Ext.decode(response.responseText);

      self.createCertificateDetailView(self.certificate).show();
      self.findParentByType('window').close();
    };
  },

  createCertificateDetailView : function(certificate) {
    return Ext.create({
      xtype : 'add-certificate-window',
      certificate : certificate
    });
  },

  trim : function(str) {
    return str.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
  }

});
