/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Nexus,Sonatype,Ext*/
NX.define('Nexus.ssl.smtp.UseNexusTruststoreCheckbox', {
  extend : 'Ext.form.Checkbox',

  type : 'smtp',
  smtpId : 'global',

  fieldLabel : 'Use Nexus SSL trust store',

  listeners : {
    check : function(cmp, checked) {
      if (this.submitState) {
        this.setUseTrustStore(checked);
      }
    },
    render : function() {
      // 'check' is also trigger when #setValue() is used on load - so we only submit if the checkbox is
      // visible
      this.submitState = true;
    },
    afterrender : function() {
      this.loadSettings();
    }
  },

  loadSettings : function() {
    var self = this;
    Ext.Ajax.request({
      url : this.trustKeyUrl(),
      success : function(response) {
        var data = Ext.decode(response.responseText);
        self.submitState = false;
        self.setValue(data.enabled);
        self.submitState = true;
      }
    });

  },

  setUseTrustStore : function(enable) {
    var
          self = this,
          mask = new Ext.LoadMask(self.ownerCt.el, {
            msg : 'Saving settings...',
            removeMask : true
          });

    mask.show();

    Ext.Ajax.request({
      url : self.trustKeyUrl(),
      method : 'PUT',
      jsonData : {
        enabled : enable
      },
      callback : function() {
        mask.hide();
      }
    });
  },

  trustKeyUrl : function() {
    return Sonatype.config.contextPath + '/service/siesta/ssl/truststore/key/' + this.type + '/' + this.smtpId;
  },

  setState : function(settings) {
    this.setDisabled(!settings.sslEnabled);
  }
}, function() {
  Sonatype.Events.addListener('serverConfigViewPostInit', function(panel) {
    var useTruststore = NX.create('Nexus.ssl.smtp.UseNexusTruststoreCheckbox');
    panel.getForm().on('actioncomplete', function(form, action) {
      if (action.type !== 'sonatypeLoad') {
        return;
      }
      useTruststore.setState(action.result.data.smtpSettings);
    });

    panel.getForm().findField('smtpSettings.connection').on('select', function(combo, record, index) {
      useTruststore.setDisabled(record.get('value') !== 'ssl');
    });

    panel.find('name', 'smtp-settings')[0].add(useTruststore);
  });
});