/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository content browser.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.ContentView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Sonatype.repoServer.ArtifactContainer'

        // FIXME: not a real module
        //'Sonatype.repoServer.RepositoryBrowsePanel'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.artifactPanel = new Sonatype.repoServer.ArtifactContainer({
            region: 'east',
            split: true,
            width: '50%',
            collapsible: true,
            collapsed: true,

            // hide the collapse tool so we don't have that ugly blank title bar
            hideCollapseTool: true,

            // Change the events which are fired (this is pretty shitty api); this transforms the container into a file container
            // (instead of an artifact container) which alters which set of tabs will show up.
            // This is required or tabs will show up which are not meant to be used with staging content (like custom metadata).
            initEventName: 'fileContainerInit',
            updateEventName: 'fileContainerUpdate'
        });

        // HACK: override tools, (AutoTabPanel via ArtifactContainer) wants to put a hidden refresh tool here which is an extra <div> with padding we don't want
        // HACK: doesn't seem to have affect when used in ctor cfg, so we force undefine all tools here
        self.artifactPanel.tools = undefined;

        // FIXME: Kill refresh button here, move "Path Lookup" to south?

        self.browserPanel = new Sonatype.repoServer.RepositoryBrowsePanel({
            payload: null,
            browseIndex: false,
            region: 'center',

            // Using a simple callback function instead of complex events
            nodeClickHandler: function (node) {
                self.selectArtifact(node);
            }
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-ContentView',
            title: 'Content',
            layout: 'border',
            hideMode: 'offsets',
            items: [
                self.browserPanel,
                self.artifactPanel
            ]
        });

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * An artifact has been selected, update the artifact panel.
     *
     * @param node
     *
     * @private
     */
    selectArtifact: function(node) {
        var self = this,
            panel = self.artifactPanel,
            attrs = node.attributes;

        if (panel.items.getCount() > 0) {
            if (node && node.isLeaf()) {
                self.logDebug('Selected artifact:', node);

                panel.updateArtifact({
                    text: attrs.text,
                    leaf: attrs.leaf,
                    resourceURI: attrs.resourceURI
                });
            }
            else {
                panel.collapsePanel();
            }
        }
    },

    /**
     * Update the repository record.
     *
     * @param repo
     */
    updateRecord: function (repo) {
        var self = this,
            sc = Sonatype.config,
            u = Sonatype.config.repos.urls;

        self.artifactPanel.collapsePanel();

        // updatePayload() wants a Ext.data.Record-like object
        self.browserPanel.updatePayload({
            data: {
                resourceURI: sc.host + u.repositories + '/' + repo.repositoryId,
                name: repo.repositoryId + " (u:" + repo.userId + ", a:" + repo.ipAddress + ")",
                repositoryURI: sc.host + u.repositories + '/' + repo.repositoryId,
                repoType: 'hosted',
                exposed: false,
                contentResourceURI: repo.repositoryURI,
                showCtx: true
            },

            // here to make more Ext.data.Record-like
            get: function (field) {
                return this.data[field];
            }
        });
    }
});