/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository group members.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.MembersView', {
    extend: 'Ext.tree.TreePanel',

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.store = NX.create('Ext.data.SimpleStore', {
            fields: [
                { name: 'id' },
                { name: 'name' },
                { name: 'description' }
            ]
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-MembersView',
            title: 'Members',
            layout: 'fit',
            animate: false,

            root: NX.create('Ext.tree.TreeNode', {
                text: ''
            }),
            rootVisible: false,
            enableDD: false,

            loader: NX.create('Ext.tree.SonatypeTreeLoader', {
                url: '',
                baseAttrs: {
                    expanded: true
                },
                listeners: {
                    scope: self,
                    loadexception: self.treeLoadExceptionHandler
                }
            })
        });

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * Update the repository record.
     */
    updateRecord: function (repo) {
        var self = this;

        // FIXME: Sort out how to put proper icons on to tree elements

        //noinspection JSValidateTypes
        self.setRootNode(new Ext.tree.AsyncTreeNode({
            text: repo.repositoryName,
            id: Sonatype.config.servicePath + '/staging/profile_repositories/tree/' + repo.repositoryId,
            expanded: true
        }));
    },

    /**
     * @private
     */
    treeLoadExceptionHandler: function (treeLoader, node, response) {
        var self = this,
            code = response.status,
            mb = Sonatype.MessageBox;

        if (code === 503) {
            if (mb.isVisible()) {
                mb.hide();
            }
            node.setText(node.text + ' (Out of Service)');
        }
        else if (code === 404 || code === 400) {
            if (mb.isVisible()) {
                mb.hide();
            }
            node.setText(node.text + (node.isRoot ? ' (Not Available)' : ' (Not Found)'));
        }
        else if (code === 401 || code === 403) {
            if (mb.isVisible()) {
                mb.hide();
            }
            node.setText(node.text + ' (Access Denied)');
        }
    }
});