/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * ???
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.DetailSupport', {
    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.prefs.DateFormat',
        'Nexus.staging.Icons'
    ],

    statics: {
        /**
         * Helper to render a date in a full timestamp.
         *
         * @protected
         */
        timestamp: function (date) {
            return date.format(Nexus.prefs.DateFormat.forName('datetime')['long']);
        },

        /**
         * Message icon, title and timestamp.
         *
         * @protected
         */
        headerTpl: NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-ActivityView-messagePanel-head">',
                '{icon}',
                '<div class="nx-staging-repository-ActivityView-messagePanel-title">{title}</div>',
                '<div class="nx-staging-repository-ActivityView-messagePanel-timestamp">{timestamp}</div>',
            '</div>',
            {
                compiled: true
            }
        ),

        /**
         * Message body, no additional detail provided.
         *
         * @protected
         */
        noDetailTpl: NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-ActivityView-messagePanel-body-nodetail">',
                'No additional detail provided for event.',
            '</div>',
            {
                compiled: true
            }
        ),

        /**
         * Renders properties as key -> value data.
         *
         * @protected
         */
        propertiesTpl: NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-ActivityView-messagePanel-properties">',
                '<tpl if="values.properties">',
                    '<table>',
                        '<tpl for="properties">',
                            '<tr class="{[ this.rowclass(xindex) ]}">',
                                '<td class="nx-staging-repository-ActivityView-messagePanel-properties-name">{name}</td>',
                                '<td class="nx-staging-repository-ActivityView-messagePanel-properties-value">{value}</td>',
                            '</tr>',
                        '</tpl>',
                    '</table>',
                '</tpl>',
                '<tpl if="!values.properties">',
                    '{[ Nexus.staging.repository.activity.DetailSupport.noDetailTpl.apply() ]}',
                '</tpl>',
            '</div>',
            {
                compiled: true,

                rowclass: function(xindex) {
                    var base = 'nx-staging-repository-ActivityView-messagePanel-properties-entry-';
                    if (xindex % 2 === 0) {
                        return base + 'even';
                    }
                    else {
                        return base + 'odd';
                    }
                }
            }
        )
    },

    // TODO: Add node and message panel icon/text representations, they might be slightly different.

    /**
     * @public
     */
    getIcon: function () {
        throw 'abstract';
    },

    /**
     * @public
     */
    getTitle: function () {
        throw 'abstract';
    },

    /**
     * @public
     */
    getTemplate: function () {
        throw 'abstract';
    }
});
