/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging rule-type data store.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.rule.RuleTypeStore', {
    extend: 'Ext.data.JsonStore',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    constructor: function (config) {
        var self = this,
            config = config || {};

        Ext.applyIf(config, {
            /**
             * @cfg {Boolean} True to enable filtering of rules with ids starting with 'testing.'
             */
            filterTesting: false
        });

        Ext.apply(config, {
            url: Sonatype.config.servicePath + '/staging/rule_types',
            root: 'data',
            id: 'id',

            fields: [
                { name: 'id' },
                { name: 'name' },
                { name: 'description' },
                { name: 'resourceURI' }
            ],

            sortInfo: {
                field: 'name',
                direction: 'ASC'
            }
        });

        self.constructor.superclass.constructor.call(self, config);

        if (config.filterTesting === true) {
            self.logDebug('Filtering testing.* rules enabled');

            self.on('load', function(store, records, options) {
                self.filterBy(function (record, id) {
                    // ignore any rule who's id starts with this prefix
                    return !id.startsWith('testing.');
                });
            });
        }
    },

    /**
     * @public
     */
    nameForId: function (id) {
        var record = this.getById(id);
        if (record) {
            return record.get('name');
        }
        else {
            this.logDebug('Missing rule-type for ID:', id);
            // use id as name
            return id;
        }
    }
});