/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

// TODO: Move to NX core, once we are ready to use this other places, maybe tree handling still blows so this might be premature to move

/**
 * Support for {@link Ext.tree.TreeLoader} implementations.
 *
 * Not sure this is the best way to use TreeLoader, hopefully we can move on to ExtJS 4 soon,
 * which has *much more sane* data-model for trees (same as grids, etc).
 *
 * @since staging 2.3
 */
NX.define('Nexus.util.TreeLoaderSupport', {
    extend: 'Ext.tree.TreeLoader',
    
    mixins: [
        'Nexus.LogAwareMixin'
    ],

    /**
     * This mostly duplicated from super-impl so we can pass in this#suppressStatus :-(
     *
     * @override
     * @private
     */
    requestData: function (node, callback, scope) {
        if (this.fireEvent("beforeload", this, node, callback) !== false) {
            if (this.directFn) {
                var args = this.getParams(node);
                args.push(this.processDirectResponse.createDelegate(this, [
                    {callback: callback, node: node, scope: scope}
                ], true));
                this.directFn.apply(window, args);
            }
            else {
                this.transId = Ext.Ajax.request({
                    method: this.requestMethod,
                    url: this.dataUrl || this.url,
                    success: this.handleResponse,
                    failure: this.handleFailure,
                    scope: this,
                    argument: {callback: callback, node: node, scope: scope},
                    params: this.getParams(node),

                    // HACK: Pass our custom status suppression configuration
                    suppressStatus: this.suppressStatus
                });
            }
        }
        else {
            // if the load is cancelled, make sure we notify the node that we are done
            this.runCallback(callback, scope || node, []);
        }
    },

    /**
     * This mostly duplicated from super-impl so we can call #createNodes() to process node
     * creation response data which could be in any format.
     *
     * @override
     * @private
     */
    processResponse: function (response, node, callback, scope) {
        var json = response.responseText;

        try {
            var o = response.responseData || Ext.decode(json);
            node.beginUpdate();

            // HACK: Delegate to createNodes()
            this.createNodes(node, o);

            node.endUpdate();
            this.runCallback(callback, scope || node, [node]);
        }
        catch (e) {
            this.logError('Node creation failed', e);
            this.handleFailure(response);

            // If we do not throw here failures when loading the detail will be lost!
            throw e;
        }
    },

    /**
     * Create nodes from the given response data.
     *
     * @protected
     *
     * @param node
     * @param data
     */
    createNodes: function(node, data) {
        throw new Error('abstract');
    }
});
