/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Sonatype.repoServer.ArtifactProcurementRulePickerDialog', {
  extend : 'Ext.Window',
  constructor : function(mask, node) {

    this.node = node;

    this.gavMask = mask;

    var disableDelete = !Sonatype.lib.Permissions.checkPermission('nexus:procurementresolution',
          Sonatype.lib.Permissions.DELETE);
    if (node.isRoot) {
      if (this.gavMask == Nexus.procurement.configuration.FREEFORM_GAV) {
        this.title = " a Specified GAV";
      }
      else {
        this.title = "*:*:*";
      }
    }
    else if (node.attributes.type == 'G') {
      var groupId = node.attributes.path;
      this.title = groupId.substring(1, groupId.length - 1).replace(/\//g, '.');
    }
    else if (node.attributes.type == 'A') {
      this.title = node.attributes.groupId + ':' + node.attributes.artifactId;
    }
    else if (node.attributes.type == 'V' || node.attributes.type == 'artifact') {
      this.title = node.attributes.groupId + ':' + node.attributes.artifactId + ':' + node.attributes.version;
    }

    this.title = 'Add Rule for ' + this.title;

    this.treeRoot = new Ext.tree.TreeNode({
      text : 'root'
    });
    this.deletedNodes = [];

    this.ruleTypeDataStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : ['id', 'name', 'description', 'properties'],
      url : Sonatype.config.servicePath + '/procurement/rules',
      autoLoad : false
    });

    Sonatype.repoServer.ArtifactProcurementRulePickerDialog.superclass.constructor.call(this, {
      closable : true,
      autoWidth : false,
      width : 525,
      height : 440,
      modal : true,
      constrain : true,
      resizable : true,
      items : [
        {
          xtype : 'form',
          layoutConfig : {
            labelSeparator : ''
          },
          labelWidth : 100,
          frame : true,
          defaultType : 'textfield',
          monitorValid : true,
          region : 'center',
          items : [
            {
              xtype : 'combo',
              fieldLabel : 'Available Rules',
              itemCls : 'required-field',
              helpText : 'Procurement rules that can be applied at this level.',
              name : 'rulesId',
              id : 'rulesCombo',
              store : this.ruleTypeDataStore,
              width : 350,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              listeners : {
                select : {
                  fn : this.onRuleSelect,
                  scope : this
                }
              }
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Description',
              name : 'ruleDescription',
              id : 'ruleDescriptionField',
              width : 350,
              readOnly : true
            },
            {
              xtype : 'panel',
              id : 'gavFieldPanel',
              layout : 'form',
              deferredRender : false,
              autoScroll : false,
              frame : false,
              width : 520,
              autoHeight : true,
              items : [
                {
                  xtype : 'textfield',
                  name : 'freeformGAV',
                  fieldLabel : 'Enter a GAV',
                  id : 'freeformGAVField',
                  width : 350,
                  helpText : "Enter a GAV (Group, Artifact, Version) to apply to this rule. The specified GAV should have the form \'Group:Artifact:Version\'. For example, enter org.apache.ant:ant:* for all versions of the ant artifacts.",
                  readOnly : false
                }
              ]
            }
          ]
        },
        {
          xtype : 'panel',
          id : 'rule-config-forms',
          title : 'Rule Properties',
          layout : 'card',
          region : 'center',
          activeItem : 0,
          deferredRender : false,
          autoScroll : false,
          height : 220,
          frame : false,
          items : [
            {}
          ]
        },
        {
          xtype : 'panel',
          id : 'rule-existing-value-card-panel',
          title : 'Existing Rule(s)',
          bodyStyle : 'padding:5px',
          deferredRender : false,
          autoScroll : false,
          height : 100,
          frame : false,
          items : [
            {
              xtype : 'label',
              fieldLabel : 'Value',
              id : 'existing-rule-value-label',
              text : 'none'
            }
          ]
        }
      ],
      buttons : [
        {
          text : 'Save',
          formBind : true,
          disabled : disableDelete,
          handler : this.addNewRule,
          scope : this
        },
        {
          text : 'Cancel',
          formBind : false,
          handler : function(button, e) {
            this.close();
          },
          scope : this
        }
      ]
    });
    this.formCards = this.findById('rule-config-forms');
    var startConfig = {
      xtype : 'panel',
      layout : 'form',
      id : 'rule-properties-card-panel',
      header : false,
      bodyStyle : 'padding:15px',
      deferredRender : false,
      autoScroll : false,
      height : 150,
      frame : false,
      items : [
        {}
      ]
    }
    var formPanel = new Ext.FormPanel(startConfig);
    this.formCards.add(formPanel);

    formPanel.doLayout();
    this.loadAllRules();
  },

  getValueForProp : function(id) {
    var field = this.findById('ruleProperties_' + id);
    return field.getValue() + "";
  },

  getMissingPropMsg : function() {
    var rule = this.getSelectedRule();
    var props = rule.data.properties;
    if (props != null && props.length > 0) {
      for (var i = 0; i < props.length; i++) {
        if (props[i].required) {
          var propValue = this.getValueForProp(props[i].id);
          if (propValue == null || propValue.length == 0) {
            return "A value is required for the property '" + props[i].name + "'";
          }
        }
      }
    }
    return "";
  },

  addNewRule : function() {
    var repoRec = this.node.ownerTree.payload;
    var repoName = repoRec.data.name;
    var gav = "";
    if (this.gavMask == Nexus.procurement.configuration.FREEFORM_GAV) {
      var gavValue = this.findById('freeformGAVField').getValue();

      gav = gavValue.split(':');
      if (gav.length != 3) {
        Ext.Msg.alert('Error',
              "The specified GAV should have the form \'Group:Artifact:Version\'.  For example, enter org.apache.ant:ant:* for all versions of the ant artifacts.");
        return;
      }
    }
    else {
      gav = this.gavMask.split(':');
    }
    var selectedRuleId = this.getSelectedRuleId();
    var missingPropMsg = this.getMissingPropMsg();
    if (missingPropMsg != null && missingPropMsg.length > 0) {
      Ext.Msg.alert('Missing Property', missingPropMsg);
      return;
    }
    var propz = [];

    var rule = this.getSelectedRule();
    var props = rule.data.properties;
    if (props != null && props.length > 0) {
      for (var i = 0; i < props.length; i++) {
        var propValue = this.getValueForProp(props[i].id);
        if (propValue) {
          var newElem = {
            key : props[i].id,
            value : propValue
          }
          propz.push(newElem);
        }
      }
    }
    Ext.Ajax.request({
      method : 'POST',
      jsonData : {
        data : {
          artifactCoordinate : {
            groupId : gav[0],
            artifactId : gav[1],
            version : gav[2]
          },
          properties : propz,
          ruleTypeId : selectedRuleId
        }
      },
      url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id,
      scope : this,
      callback : function(options, success, response) {
        if (success) {
          this.close();
          this.node.ownerTree.refreshNode(this.node);
        }
        else {
          Sonatype.utils.connectionError(response, 'Adding the new rule failed in ' + repoName + '.');
        }
      }
    });

  },

  updateExistingRuleText : function(txt) {
    var existingRuleLabel = this.findById('existing-rule-value-label');
    if (existingRuleLabel) {
      Ext.get(existingRuleLabel.getEl()).update(txt);
    }
  },

  getExistingRule : function() {
    this.updateExistingRuleText('Finding existing rule...');
    if (!this.node) {
      return;
    }
    var repoRec = this.node.ownerTree.payload;
    var repoName = repoRec.data.name;
    var appliedIds = this.node.attributes.appliedArtifactResolutionIds;
    if (appliedIds == null || appliedIds.length == 0) {
      this.updateExistingRuleText("There are no rules applied.");
      return;
    }
    var ids = "";
    var count = 0;
    for (var i = 0; i < appliedIds.length; i++) {
      if (appliedIds[i]) {
        for (var j = 0; j < appliedIds[i].ruleIds.length; j++) {
          if (appliedIds[i].ruleIds[j]) {
            if (count > 0) {
              ids += ',';
            }
            ids += appliedIds[i].ruleIds[j];
            count++;
          }
        }
      }
    }
    if (repoRec && repoRec.data) {
      Ext.Ajax.request({
        method : 'GET',
        url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id + '?ids=' + ids,
        scope : this,
        callback : function(options, success, response) {
          if (success) {
            var result = Ext.util.JSON.decode(response.responseText);
            var data = result.data;
            var existingRulesText = "";
            if (data) {
              for (var i = 0; i < data.length; i++) {
                if (i > 0) {
                  existingRulesText += ", ";
                }
                var ac = data[i].artifactCoordinate;
                var ruleText = this.getExistingRuleText(data[i]);
                existingRulesText += ruleText;

              }
            }
            this.updateExistingRuleText(existingRulesText);
          }
          else {
            Sonatype.utils.connectionError(response, 'Failed to find rules ' + repoName + '.');
            this.updateExistingRuleText('Failed to find rule...');
          }
        }
      });
    }
  },

  getExistingRuleText : function(rule) {
    var ruleText = rule.ruleTypeId;
    if (rule.ruleTypeId == "simple" && rule.properties && rule.properties.length == 1) {
      var key = rule.properties[0].key;
      var value = rule.properties[0].value;
      if (value == "true") {
        ruleText = "Approve for";
      }
      else {
        ruleText = "Deny for"
      }
    }
    else if (rule.ruleTypeId == "signature" && rule.properties && rule.properties.length == 1) {
      var key = rule.properties[0].key;
      var value = rule.properties[0].value;
      if (value == "true") {
        ruleText = "Signature must be present for";
      }
      else {
        ruleText = "Signature can be present/empty for";
      }
    }
    else {
      ruleText = rule.ruleTypeId
    }
    return (ruleText + " " + rule.artifactCoordinate.groupId + ":" + rule.artifactCoordinate.artifactId + ":"
          + rule.artifactCoordinate.version);
  },

  ruleTypeStoreLoadHandler : function() {},

  showFreeformField : function() {
    if (this.gavMask != Nexus.procurement.configuration.FREEFORM_GAV) {
      var freeFormGAVText = this.findById('freeformGAVField');
      if (freeFormGAVText) {
        freeFormGAVText.hideParent = true;
        freeFormGAVText.hidden = true;
        freeFormGAVText.hideLabel = true;
      }
    }
  },

  loadAllRules : function() {
    this.showFreeformField();
    this.show();
    this.ruleTypeDataStore.reload();
    this.getExistingRule();
  },

  populateRulePanel : function(rec) {
    var allItems = [];
    var strValue = 'string';
    var numValue = 'number';
    var boolValue = 'boolean';
    if (rec.data.properties == null || rec.data.properties.length == 0) {
      allItems[0] = {
        xtype : 'label',
        text : 'There are no properties for this rule.',
        style : 'text-align:center;',
        name : 'no-props-lbl',
        width : 500
      };
      return allItems;
    }
    for (var j = 0; j < rec.data.properties.length; j++) {
      var curRec = rec.data.properties[j];
      if (curRec.type == strValue || curRec.type.toLowerCase() == strValue) {
        allItems[j] = {
          xtype : 'textfield',
          fieldLabel : curRec.name,
          itemCls : curRec.required ? 'required-field' : '',
          helpText : curRec.helpText,
          id : 'ruleProperties_' + curRec.id,
          allowBlank : curRec.required ? false : true,
          labelStyle : 'width:150px;',
          width : 300
        };
      }
      else if (curRec.type == numValue || curRec.type.toLowerCase() == numValue) {
        allItems[j] = {
          xtype : 'numberfield',
          fieldLabel : curRec.name,
          itemCls : curRec.required ? 'required-field' : '',
          helpText : curRec.helpText,
          name : 'ruleProperties_' + curRec.id,
          id : 'ruleProperties_' + curRec.id,
          allowBlank : curRec.required ? false : true,
          labelStyle : 'width:150px;',
          width : 300
        };
      }
      else if (curRec.type == boolValue || curRec.type.toLowerCase() == boolValue) {
        allItems[j] = {
          xtype : 'checkbox',
          fieldLabel : curRec.name,
          helpText : curRec.helpText,
          id : 'ruleProperties_' + curRec.id,
          width : 300,
          labelStyle : 'width:150px;',
          name : 'ruleProperties_' + curRec.id
        };
      }
    }

    return allItems;
  },

  getSelectedRuleId : function() {
    var combo = this.findById('rulesCombo');
    return combo.getValue() + "";
  },

  getSelectedRule : function() {
    var selectedRuleId = this.getSelectedRuleId();
    var rule = this.ruleTypeDataStore.getAt(this.ruleTypeDataStore.findBy(function(record, id) {
      if (record.data.id == selectedRuleId) {
        return true;
      }
      return false;
    }));
    return rule;
  },

  updateDescription : function() {
    var rule = this.getSelectedRule();
    if (rule && rule.data.description) {
      var descField = this.findById('ruleDescriptionField');
      descField.setValue(rule.data.description);
    }
  },

  onRuleSelect : function(combo, rec, index) {
    var id = 'rule-properties-card-panel';
    var newItems = this.populateRulePanel(rec);
    var config = {
      xtype : 'panel',
      id : 'rule-properties-card-panel',
      header : false,
      bodyStyle : 'padding:15px',
      deferredRender : false,
      autoScroll : false,
      frame : false,
      height : 150,
      items : [
        {}
      ]
    }
    config.items = newItems;
    var formPanel = new Ext.FormPanel(config);
    this.formCards.add(formPanel);
    var layout = this.formCards.getLayout();
    this.formCards.getLayout().setActiveItem(formPanel);
    this.updateDescription();
    formPanel.doLayout();

  },

  updateExistingRule : function(rec) {
    var valueLabel = this.findById("existing-rule-value-label");
    Ext.get(valueLabel.getEl()).update(rec.name);
  }
}, function() {
  Sonatype.Events.addListener('procurementToolbarInit', function(treepanel, toolbar) {
    toolbar.push(new Ext.Button({
      text : 'Add Freeform Rule',
      icon : Sonatype.config.resourcePath + '/static/images/icons/add.png',
      cls : 'x-btn-text-icon',
      handler : function() {
        new Sonatype.repoServer.ArtifactProcurementRulePickerDialog(Nexus.procurement.configuration.FREEFORM_GAV,
              treepanel.getRootNode());
      },
      scope : treepanel
    }));
  });
});
