/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Sonatype.repoServer.ArtifactProcurementStartPopup', {
  extend : 'Ext.Window',
  constructor : function(config) {
    var config = config || {};
    var defaultConfig = {
      title : 'Start Procurement'
    };
    Ext.apply(this, config, defaultConfig);

    this.hostedRepoStore = new Ext.data.SimpleStore({
      fields : ['id', 'name'],
      data : [],
      sortInfo : {
        field : 'name',
        direction : 'ASC'
      }
    });
    this.targetRepoStore = new Ext.data.SimpleStore({
      fields : ['id', 'name', 'displayName'],
      data : [],
      sortInfo : {
        field : 'name',
        direction : 'ASC'
      }
    });

    this.groupStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : [
        {
          name : 'id'
        },
        {
          name : 'repoType'
        },
        {
          name : 'format'
        },
        {
          name : 'name',
          sortType : Ext.data.SortTypes.asUCString
        },
        {
          name : 'displayName',
          mapping : 'name',
          convert : function(s, parent) {
            return '<b>' + s + '</b>';
          }
        }
      ],
      sortInfo : {
        field : 'name',
        direction : 'asc'
      },
      url : Sonatype.config.repos.urls.groups,
      autoLoad : false,
      listeners : {
        load : {
          fn : function(store, records, options) {
            this.repoStore.insert(0, records);
            this.show();
          },
          scope : this
        },
        // need to handle this in case there are no groups
        loadexception : {
          fn : function() {
            this.show();
          },
          scope : this
        }
      }
    });

    this.repoStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : [
        {
          name : 'id'
        },
        {
          name : 'repoType'
        },
        {
          name : 'repoPolicy'
        },
        {
          name : 'format'
        },
        {
          name : 'name',
          sortType : Ext.data.SortTypes.asUCString
        },
        {
          name : 'displayName',
          mapping : 'name'
        }
      ],
      sortInfo : {
        field : 'name',
        direction : 'asc'
      },
      url : Sonatype.config.repos.urls.repositories,
      autoLoad : true,
      listeners : {
        load : {
          fn : function(store, records, options) {
            var hostedRepos = [];

            for (var i = 0; i < records.length; i++) {
              var rec = records[i];
              if (rec.data.repoType == 'hosted' && (rec.data.format == 'maven2' || rec.data.format == 'maven1')) {
                hostedRepos.push([rec.data.id, rec.data.name]);
              }
            }

            this.hostedRepoStore.loadData(hostedRepos);

            this.groupStore.load();
          },
          scope : this
        },
        loadexception : {
          fn : function() {
            this.groupStore.load();
          },
          scope : this
        }
      }
    });

    Sonatype.repoServer.ArtifactProcurementStartPopup.superclass.constructor.call(this, {
      closable : true,
      autoWidth : false,
      width : 400,
      autoHeight : true,
      modal : true,
      constrain : true,
      resizable : false,
      draggable : false,
      items : [
        {
          xtype : 'form',
          layoutConfig : {
            labelSeparator : ''
          },
          labelWidth : 100,
          frame : true,
          defaultType : 'textfield',
          monitorValid : true,
          items : [
            {
              xtype : 'combo',
              fieldLabel : 'Repository',
              itemCls : 'required-field',
              helpText : 'Repository to start procurement for.',
              name : 'repositoryId',
              anchor : Sonatype.view.FIELD_OFFSET,
              width : 200,
              store : this.hostedRepoStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              listeners : {
                select : {
                  fn : this.onRepositorySelect,
                  scope : this
                }
              }
            },
            {
              xtype : 'combo',
              fieldLabel : 'Source',
              itemCls : 'required-field',
              helpText : 'Repository or group to procure from.',
              name : 'targetId',
              anchor : Sonatype.view.FIELD_OFFSET,
              width : 200,
              store : this.targetRepoStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              tpl : '<tpl for="."><div class="x-combo-list-item">{displayName}</div></tpl>'
            }
          ],
          buttons : [
            {
              text : 'OK',
              formBind : true,
              handler : this.startProcurement,
              scope : this
            },
            {
              text : 'Cancel',
              formBind : false,
              handler : function(button, e) {
                this.close();
              },
              scope : this
            }
          ]
        }
      ],
      listeners : {
        show : {
          fn : function(p) {
            if (this.payload) {
              var repositoryCombo = p.find('name', 'repositoryId')[0];
              var payloadId = this.payload.data.id;
              repositoryCombo.setValue(payloadId);
              this.filterTargetList(payloadId);
              repositoryCombo.disable();
            }
          },
          scope : this
        }
      }
    });
  },

  onRepositorySelect : function(combo, rec, index) {
    this.filterTargetList(rec.data.id);
  },

  filterTargetList : function(repoId) {
    this.targetRepoStore.removeAll();
    var repoPolicy = this.repoStore.getById(repoId).data.repoPolicy;
    var targetRepos = [];
    var records = this.repoStore.getRange();

    for (var i = 0; i < records.length; i++) {
      var rec = records[i];
      if (rec.data.id != repoId && (rec.data.repoPolicy == null || rec.data.repoPolicy == repoPolicy)
            && (rec.data.format == 'maven2')) {
        targetRepos.push([rec.data.id, rec.data.name, rec.data.displayName]);
      }
    }

    this.targetRepoStore.loadData(targetRepos);
    this.find('name', 'targetId')[0].reset();
  },

  startProcurement : function(button, e) {
    var targetId = this.find('name', 'targetId')[0].getValue();
    var targetRec = this.repoStore.getById(targetId);

    var repositoryCombo = this.find('name', 'repositoryId')[0]
    var repositoryId = repositoryCombo.getValue();
    var repositoryName = repositoryCombo.getRawValue();

    Ext.Ajax.request({
      scope : this,
      method : 'POST',
      suppressStatus : 400,
      jsonData : {
        data : {
          repositoryId : repositoryId,
          targetClassId : targetRec.data.repoType ? 'repositories' : 'groups',
          targetId : targetId
        }
      },
      repositoryName : repositoryName,
      url : Nexus.procurement.configuration.PROCUREMENT_SERVICE_PATH,
      success : this.procurementSuccess,
      failure : this.procurementFailure,
      scope : this
    });
  },

  procurementSuccess : function(response, options) {
    this.close();
    Sonatype.MessageBox.show({
      title : 'Procurement Started',
      msg : 'Procurement is now started in ' + options.repositoryName + '.',
      buttons : Sonatype.MessageBox.OK,
      icon : Sonatype.MessageBox.INFO,
      animEl : 'mb3'
    });

    var procurementPanel = Sonatype.view.mainTabPanel.findById('artifact-procurement');
    if (procurementPanel) {
      procurementPanel.refreshHandler();
      Sonatype.Events.fireEvent('repositoryChanged');
    }
  },

  procurementFailure : function(response, options) {
    var r = Ext.decode(response.responseText);
    if (r.errors) {
      this.find('name', 'repositoryId')[0].markInvalid(r.errors[0].msg);
    }
    else {
      Sonatype.utils.connectionError(response,
            'There was a problem starting procurement in ' + options.repositoryName + '.');
    }
  }
}, function() {
  Sonatype.Events.addListener('procurementViewInit', function(cardPanel, rec) {
    if (rec.data.targetId != null && rec.data.targetId == '') {
      cardPanel.add({
        xtype : 'panel',
        name : 'configuration',
        items : [
          {
            border : false,
            html : '<div class="little-padding">Procurement is not started in this repository.</div>'
          },
          {
            cls : 'little-padding',
            xtype : 'button',
            text : 'Start Procurement...',
            hidden : !Sonatype.lib.Permissions.checkPermission('nexus:procurementrepos',
                  Sonatype.lib.Permissions.CREATE),
            handler : function(button, e) {
              new Sonatype.repoServer.ArtifactProcurementStartPopup({
                payload : rec
              });
            }
          }
        ]
      });
    }
  });

});
