/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Ext,define,Sonatype*/

NX.define('Sonatype.repoServer.ArtifactProcurementTree', {
  extend : 'Sonatype.panels.TreePanel',
  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {};
    Ext.apply(this, config, defaultConfig);

    Sonatype.repoServer.ArtifactProcurementTree.superclass.constructor.call(this, {
      toolbarInitEvent : 'procurementToolbarInit',
      nodeContextMenuEvent : 'procurementNodeMenuInit',
      url : Sonatype.config.servicePath + '/procurement/treeview/' + this.payload.data.id
            + Sonatype.config.browsePathSnippet
    });

    this.on('expandnode', this.expandNodeHandler, this);
  },
  expandNodeHandler : function(node) {
    this.updateNodeIcon(node);
  },

  refreshNode : function(node) {
    var path = node.getPath();
    while (node.attributes.type != 'G' && !node.isRoot) {
      node = node.parentNode;
    }
    node.attributes.children = null;
    node.reload(function() {
      node.ownerTree.expandPath(path);
    });
  },
  getIconName : function(resolution) {
    var rule = resolution.ruleAppliedState;
    var name = '';
    if (rule) {
      if (rule.toLowerCase() == 'no_rule') {
        name = 'undefined';
      }
      else if (rule.toLowerCase() == 'approved') {
        name = 'approved';
      }
      else if (rule.toLowerCase() == 'disapproved') {
        name = 'disapproved';
      }
      else {
        name = 'runtime';
      }
    }
    else {
      name = '';
    }
    return name;
  },
  updateNodeIcon : function(node) {
    if (node.attributes.ruleAppliedState) {
      var iconName = this.getIconName(node.attributes);
      node.ui.iconNode.className = 'x-tree-node-procurement-' + iconName + '-icon';
    }
    for (var j = 0; j < node.childNodes.length; j++) {
      this.updateNodeIcon(node.childNodes[j]);
    }
  }
}, function() {
  Sonatype.Events.addListener('procurementViewInit', function(cardPanel, rec) {
    if (rec.data.targetId != null && rec.data.targetId != '') {
      cardPanel.add(new Sonatype.repoServer.ArtifactProcurementTree({
        payload : rec
      }));
    }
  });
});




