/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Ext,Sonatype,Nexus*/
/**
 * The backup mirror configuration
 */
NX.define('Sonatype.repoServer.LdapConfigurationBackupMirror', {
  extend : 'Ext.Panel',
  constructor : function(config) {
    var config = config || {};
    var defaultConfig = {
      fieldWidth : 200
    };
    Ext.apply(this, config, defaultConfig);

    var protocolStore = new Ext.data.SimpleStore({
      fields : ['value'],
      data : [
        ['ldap'],
        ['ldaps']
      ]
    });

    Sonatype.repoServer.LdapConfigurationBackupMirror.superclass.constructor.call(this, {
      items : [
        {
          xtype : 'fieldset',
          checkboxToggle : true,
          title : 'Use Backup Mirror',
          labelWidth : 188,
          collapsed : false,
          autoHeight : true,
          allowBlank : false,
          itemCls : 'required-field',
          listeners : {
            'expand' : {
              fn : function(panel) {
                this.ldapConfigurationContainer.expandHandler(panel);

                if (this.ldapConfigurationContainer.tabPanel.getActiveTab().getId() == this.getId()) {
                  this.ldapConfigurationContainer.formPanel.buttons[0].show();
                }
              },
              scope : this
            },
            'collapse' : {
              fn : function(panel) {
                this.find('name', 'connectionInfo.backupMirrorProtocol')[0].setValue(null);
                this.find('name', 'connectionInfo.backupMirrorHost')[0].setValue(null);
                this.find('name', 'connectionInfo.backupMirrorPort')[0].setValue(0);
                this.ldapConfigurationContainer.collapseHandler(panel);

                if (this.ldapConfigurationContainer.tabPanel.getActiveTab().getId() == this.getId()) {
                  this.ldapConfigurationContainer.formPanel.buttons[0].hide();
                }
              },
              scope : this,
              delay : 100
            }
          },
          items : [
            {
              xtype : 'combo',
              fieldLabel : 'Protocol',
              helpText : 'Use plain text (ldap://) or secure (ldaps://) connection.',
              name : 'connectionInfo.backupMirrorProtocol',
              width : this.fieldWidth,
              store : protocolStore,
              valueField : 'value',
              displayField : 'value',
              editable : false,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              itemCls : 'required-field',
              listeners : {
                select : {
                  fn : function(combo, record, index) {
                    var newValue = combo.getValue();
                    var port = this.find('name', 'connectionInfo.backupMirrorPort')[0];
                    var portValue = port.getRawValue();
                    if (newValue == 'ldap' && (portValue == 636 || portValue == 0)) {
                      port.setValue(389);
                    }
                    else if (newValue == 'ldaps' && (portValue == 389 || portValue == 0)) {
                      port.setValue(636);
                    }
                  },
                  scope : this
                }
              }
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Hostname',
              helpText : 'The host name of the LDAP server.',
              name : 'connectionInfo.backupMirrorHost',
              width : this.fieldWidth,
              allowBlank : false,
              itemCls : 'required-field'
            },
            {
              xtype : 'numberfield',
              fieldLabel : 'Port',
              helpText : 'The port the LDAP server is listening on (ldap - 389, ldaps - 636).',
              name : 'connectionInfo.backupMirrorPort',
              width : this.fieldWidth,
              allowBlank : false,
              itemCls : 'required-field'
            }
          ]
        }
      ],
      listeners : {
        afterlayout : {
          fn : function() {
            this.isLoaded = true;
          },
          scope : this
        }
      }
    });
  },

  postLoad : function() {
    var port = this.find('name', 'connectionInfo.backupMirrorPort')[0];

    if (port.getValue() > 0) {
      port.ownerCt.expand();
    }
    else {
      port.setValue(0);
      port.ownerCt.collapse();
    }
  },
  // hide the check authentication and check user mappings buttons
  handleButtons : function() {
    if (this.find('name', 'connectionInfo.backupMirrorProtocol')[0].ownerCt.collapsed == false) {
      this.ldapConfigurationContainer.formPanel.buttons[0].show();
    }
    else {
      this.ldapConfigurationContainer.formPanel.buttons[0].hide();
    }
    this.ldapConfigurationContainer.formPanel.buttons[1].hide();
    this.ldapConfigurationContainer.formPanel.buttons[2].hide();
  },
  checkAuthentication : function() {
    Sonatype.MessageBox.wait('Checking LDAP Mirror Authentication...');
    Ext.Ajax.request({
      method : 'PUT',
      url : Sonatype.config.servicePath + '/ldap/test_auth',
      jsonData : {
        data : {
          protocol : this.find('name', 'connectionInfo.backupMirrorProtocol')[0].getValue(),
          host : this.find('name', 'connectionInfo.backupMirrorHost')[0].getValue(),
          port : this.find('name', 'connectionInfo.backupMirrorPort')[0].getValue(),
          searchBase : this.ldapConfigurationContainer.connectionConfigTab.find('name',
                'connectionInfo.searchBase')[0].getValue(),
          authScheme : this.ldapConfigurationContainer.connectionConfigTab.find('name',
                'connectionInfo.authScheme')[0].getValue(),
          realm : this.ldapConfigurationContainer.connectionConfigTab.find('name',
                'connectionInfo.realm')[0].getValue(),
          systemUsername : Sonatype.utils.base64.encode(this.ldapConfigurationContainer.connectionConfigTab.find('name',
                'connectionInfo.systemUsername')[0].getValue()),
          systemPassword : Sonatype.utils.base64.encode(this.ldapConfigurationContainer.connectionConfigTab.find('name',
                'connectionInfo.systemPassword')[0].getValue()),
          connectionTimeout : this.ldapConfigurationContainer.connectionConfigTab.find('name',
                'connectionInfo.connectionTimeout')[0].getValue(),
          connectionRetryDelay : this.ldapConfigurationContainer.connectionConfigTab.find('name',
                'connectionInfo.connectionRetryDelay')[0].getValue(),
          cacheTimeout : this.ldapConfigurationContainer.connectionConfigTab.find('name',
                'connectionInfo.cacheTimeout')[0].getValue()
        }
      },
      callback : function(options, success, response) {
        if (success) {
          Sonatype.MessageBox.alert('Check Mirror Authentication',
                'LDAP Mirror connection and authentication test completed successfully.').setIcon(Sonatype.MessageBox.INFO);
        }
        else {
          Sonatype.utils.connectionError(response, 'LDAP Mirror connection and authentication test failed.', false, {
            hideErrorStatus : true,
            decodeErrorResponse : true
          });
        }
      }
    });
  }
});