/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/

define('Sonatype/repoServer/ProfileEditor', function() {

    Sonatype.repoServer.ProfileEditor = function(cfg) {
      var config = cfg || {},
        defaultConfig = {
        uri : Sonatype.config.servicePath + '/staging/profiles',
        resetButton : true,
        defaultTimeoutValues : {},
        dataModifiers : {
          load : {
            finishNotifyRoles : function(arr, srcObj, fpanel) {
              fpanel.find('name', 'finishRoleManager')[0].setSelectedRoleIds(arr, true);
              return arr;
            },
            promotionNotifyRoles : function(arr, srcObj, fpanel) {
              fpanel.find('name', 'promotionRoleManager')[0].setSelectedRoleIds(arr, true);
              return arr;
            },
            dropNotifyRoles : function(arr, srcObj, fpanel) {
              fpanel.find('name', 'dropRoleManager')[0].setSelectedRoleIds(arr, true);
              return arr;
            },
            targetGroups : this.importGroupsHelper.createDelegate(this),
            closeRuleSets : this.importCloseRuleSetsHelper.createDelegate(this),
            promoteRuleSets : this.importPromoteRuleSetsHelper.createDelegate(this)
          },
          submit : {
            finishNotifyRoles : function(value, fpanel) {
              return fpanel.find('name', 'finishRoleManager')[0].getSelectedRoleIds();
            },
            promotionNotifyRoles : function(value, fpanel) {
              return fpanel.find('name', 'promotionRoleManager')[0].getSelectedRoleIds();
            },
            dropNotifyRoles : function(value, fpanel) {
              return fpanel.find('name', 'dropRoleManager')[0].getSelectedRoleIds();
            },
            targetGroups : this.exportGroupsHelper.createDelegate(this),
            closeRuleSets : this.exportCloseRuleSetsHelper.createDelegate(this),
            promoteRuleSets : this.exportPromoteRuleSetsHelper.createDelegate(this),
            autoStagingDisabled : function(value, fpanel) {
              if ( value === true || value === false ) {
                return value;
              } else {
                return 'true' === value;
              }
            },
            repositoriesSearchable : function(value, fpanel) {
              if ( value === true || value === false ) {
                return value;
              } else {
                return 'false' === value;
              }
            }
          }
        },
        validationModifiers : {
          targetGroups: function(error, fpanel) {
            fpanel.find('name', 'targetGroups')[0].markTreeInvalid(null, error.msg);
          }
        },
        referenceData : {
          id : "",
          name : "",
          repositoryTemplateId : "",
          repositoryType : "",
          repositoryTargetId : "",
          targetGroups : [],
          finishNotifyRoles : [],
          promotionNotifyRoles : [],
          dropNotifyRoles : [],
          finishNotifyEmails : "",
          promotionNotifyEmails : "",
          dropNotifyEmails : "",
          closeRuleSets : [],
          promoteRuleSets : [],
          promotionTargetRepository : "",
          dropNotifyCreator : true,
          finishNotifyCreator : true,
          promotionNotifyCreator : true,
          autoStagingDisabled : false,
          repositoriesSearchable : true,
          mode : "",
          properties: {}
        }
      };
      Ext.apply(this, config, defaultConfig);

      this.currentTemplate = undefined;

      this.selectionStrategyStore = new Ext.data.SimpleStore({
        fields : ['value', 'display'],
        data : [[false, 'Explicit or Implicit - selected by ID or Nexus will calculate match'], [true, 'Explicit Only - only selected by ID']]
      });

      this.templateStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [{
                  name : 'resourceURI'
                }, {
                  name : 'repoType'
                }, {
                  name : 'repoPolicy'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString,
                  convert : this.calculateTemplateName
                }, {
                  name : 'format'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.repoTemplates,
            autoLoad : false,
            listeners : {
              load : {
                fn : function(store, records, options) {
                  Sonatype.repoServer.ProfileEditor.prototype.repositoryTemplateFilter(store);

                  var
                        combo = this.find('name', 'repositoryTemplateId')[0],
                        nameField = this.find('name', 'name')[0];

                  combo.doQuery(combo.allQuery, true);
                  nameField.focus();
                },
                scope : this
              }
            }
          });

      this.rulesetStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [{
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.servicePath + '/staging/rule_sets',
            autoLoad : false
          });

      this.repoTypeStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'roleHint',
            fields : [{
                  name : 'description'
                }, {
                  name : 'roleHint',
                  sortType : Ext.data.SortTypes.asUCString
                }],
            sortInfo : {
              field : 'description',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.repoTypes,
            autoLoad : false
          });

      this.checkPayload();

      this.COMBO_WIDTH = 376;

      this.items = [];
      if (!this.isNew)
      {
        if (!this.groupProfile)
        {
          this.items.push({
              xtype : 'panel',
              layout : 'column',
              items : [{
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : 'Profile ID',
                    height : 21,
                    width : 205
                  }, {
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : this.payload.get('id')
                  }]
          });
          this.items.push({
              xtype : 'panel',
              layout : 'column',
              items : [{
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : 'Deploy URL',
                    height : 21,
                    width : 205
                  }, {
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : this.payload.get('deployURI')
                  }]
          });
        }
        else
        {
          this.items.push({
              xtype : 'panel',
              layout : 'column',
              items : [{
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : 'Profile ID',
                    height : 21,
                    width : 205
                  }, {
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : this.payload.get('id')
                  }]
          });
        }
      }

      this.items.push({
            xtype : 'textfield',
            fieldLabel : 'Profile Name',
            itemCls : 'required-field',
            helpText : 'Staging profile name.',
            name : 'name',
            width : this.COMBO_WIDTH,
            allowBlank : false,
            disabled: !this.isNew
          });

      if (this.groupProfile)
      {
        this.items.push({
              xtype : 'textfield',
              fieldLabel : 'Staging Mode',
              itemCls : 'required-field',
              helpText : 'The source(s) this profile will accept artifacts from, either from the staging deploy URL, or staging upload via the UI, or both.',
              name : 'modeInput',
              width : this.COMBO_WIDTH,
              allowBlank : false,
              disabled : true
            }, {
              xtype : 'hidden',
              name : 'mode'
            }, {
              xtype : 'hidden',
              value : false,
              name : 'autoStagingDisabled'
            }, {
              xtype : 'hidden',
              value : true,
              name : 'repositoriesSearchable'
            });
      }
      else
      {
        this.items.push({
          xtype : 'combo',
          fieldLabel : 'Profile Selection Strategy',
          itemCls : 'required-field',
          helpText : '\'Explicit or Implicit\' will include this profile as a possible match using user agent, IP address, username and repository target if the profile ID is not specified explicitly. \'Explicit Only\' means this profile will only be selected when identified by profile ID.',
          name : 'autoStagingDisabledComboName',
          hiddenName : 'autoStagingDisabled',
          store : this.selectionStrategyStore,
          displayField : 'display',
          valueField : 'value',
          width : this.COMBO_WIDTH,
          listWidth : this.COMBO_WIDTH,
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          allowBlank : false
        });

        this.items.push({
          xtype : 'checkbox',
          checked : true,
          fieldLabel : 'Searchable Repositories',
          itemCls : 'required-field',
          helpText : 'When enabled, repositories created using this profile are indexed and searchable. Setting applies only to new repositories.',
          name : 'repositoriesSearchable',
          width : this.COMBO_WIDTH
        });

        this.items.push({
              xtype : 'combo',
              fieldLabel : 'Staging Mode',
              itemCls : 'required-field',
              helpText : 'The source(s) this profile will accept artifacts from, either from the staging deploy URL, or staging upload via the UI, or both.',
              name : 'mode',
              store : this.modeStore,
              displayField : 'display',
              valueField : 'value',
              width : this.COMBO_WIDTH,
              listWidth : this.COMBO_WIDTH,
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false
            });
      }

      this.items.push({
            xtype : 'combo',
            fieldLabel : 'Template',
            itemCls : 'required-field',
            helpText : 'Template used to create staging repositories.',
            name : 'repositoryTemplateId',
            width : this.COMBO_WIDTH,
            listWidth : this.COMBO_WIDTH,
            store : this.templateStore,
            displayField : 'name',
            valueField : 'id',
            editable : false,
            forceSelection : true,
            mode : 'local',
            triggerAction : 'all',
            emptyText : 'Select...',
            selectOnFocus : true,
            allowBlank : false,
            listeners : {
              select : {
                fn : this.templateSelectHandler,
                scope : this
              }
            }
          });

      if (!this.groupProfile)
      {
        this.items.push({
              xtype : 'combo',
              fieldLabel : 'Repository Target',
              itemCls : 'required-field',
              helpText : 'Repository target that will cause this profile to be selected.',
              name : 'repositoryTargetId',
              store : this.targetStore,
              displayField : 'name',
              valueField : 'id',
              width : this.COMBO_WIDTH,
              listWidth : this.COMBO_WIDTH,
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              lastQuery : ''
            });
        this.items.push({
              xtype : 'combo',
              fieldLabel : 'Release Repository',
              helpText : 'A Repository that will be used to release all staged repositories to.',
              name : 'promotionTargetRepository',
              width : this.COMBO_WIDTH,
              itemCls : 'required-field',
              listWidth : this.COMBO_WIDTH,
              store : this.filteredRepoStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : false,
              mode : 'local',
              required : true,
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              lastQuery : '',
              clearFilterOnReset: false
            });
      }

      // Allow contributors to add to the form. They will also be responsible for ensuring that this additional data gets serialized into the form appropriately.
      Nexus.staging.repository.RepositoriesMediator.fireEvent('staging.profile.loaded', this.items, this.payload.get('properties'), 'properties');

      this.items.push({
            xtype : 'textfield',
            fieldLabel : 'Content Type',
            itemCls : 'required-field',
            helpText : 'The type of the generated staging repositories (e.g. "maven1", "maven2").',
            name : 'repositoryType',
            width : this.COMBO_WIDTH,
            allowBlank : false,
            disabled : true
          }, {
            xtype : 'twinpanelchooser',
            titleLeft : 'Target Groups',
            titleRight : 'Available Groups',
            name : 'targetGroups',
            valueField : 'id',
            store : this.groupStore,
            halfSize : true,
            required : true,
            validateLeftItems: true,
            validateLeftItemsText : "Invalid target group selected."
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Close Repository Notification Settings',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'textfield',
                  fieldLabel : 'Email Addresses',
                  helpText : 'Comma seperated list of email addresses to send notifications to.',
                  name : 'finishNotifyEmails',
                  width : this.COMBO_WIDTH
                }, {
                  xtype : 'rolemanager',
                  name : 'finishRoleManager',
                  height : 200,
                  width : 581,
                  usePrivileges : false,
                  doValidation : false,
                  style : 'margin-top: 10px;border: 1px solid #B5B8C8;'
                }, {
                  xtype : 'checkbox',
                  checked : true,
                  fieldLabel : 'Notify creator',
                  helpText : 'Notify the repository creator (the user who uploaded the repository content) if the repository gets closed.',
                  name : 'finishNotifyCreator',
                  width : this.COMBO_WIDTH
                }]
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Promote Repository Notification Settings',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'textfield',
                  fieldLabel : 'Email Addresses',
                  helpText : 'Comma seperated list of email addresses to send notifications to.',
                  name : 'promotionNotifyEmails',
                  width : this.COMBO_WIDTH
                }, {
                  xtype : 'rolemanager',
                  name : 'promotionRoleManager',
                  height : 200,
                  width : 581,
                  usePrivileges : false,
                  doValidation : false,
                  style : 'margin-top: 10px;border: 1px solid #B5B8C8;'
                }, {
                  xtype : 'checkbox',
                  checked : true,
                  fieldLabel : 'Notify creator',
                  helpText : 'Notify the repository creator (the user who uploaded the repository content) if the repository gets promoted.',
                  name : 'promotionNotifyCreator',
                  width : this.COMBO_WIDTH
                }]
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Drop Repository Notification Settings',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'textfield',
                  fieldLabel : 'Email Addresses',
                  helpText : 'Comma seperated list of email addresses to send notifications to.',
                  name : 'dropNotifyEmails',
                  width : this.COMBO_WIDTH
                }, {
                  xtype : 'rolemanager',
                  name : 'dropRoleManager',
                  height : 200,
                  width : 581,
                  usePrivileges : false,
                  doValidation : false,
                  style : 'margin-top: 10px;border: 1px solid #B5B8C8;'
                }, {
                  xtype : 'checkbox',
                  checked : true,
                  fieldLabel : 'Notify creator',
                  helpText : 'Notify the repository creator (the user who uploaded the repository content) if the repository gets dropped.',
                  name : 'dropNotifyCreator',
                  width : this.COMBO_WIDTH
                }]
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Close Repository Staging Rulesets',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'twinpanelchooser',
                  titleLeft : 'Rulesets',
                  titleRight : 'Available Rulesets',
                  name : 'closeRuleSets',
                  valueField : 'id',
                  store : this.rulesetStore,
                  halfSize : true
                }]
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Promote Repository Staging Rulesets',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'twinpanelchooser',
                  titleLeft : 'Rulesets',
                  titleRight : 'Available Rulesets',
                  name : 'promoteRuleSets',
                  valueField : 'id',
                  store : this.rulesetStore,
                  halfSize : true
                }]
          }, {
            xtype : 'hidden',
            name : 'id'
          });

      Sonatype.repoServer.ProfileEditor.superclass.constructor.call(this, {
            dataStores : [this.templateStore, this.rulesetStore, this.repoTypeStore],
            listeners : {
              submit : {
                fn : this.submitHandler,
                scope : this
              },
              load : {
                fn : this.loadHandler,
                scope : this
              }
            }
          });
    };

    Ext.extend(Sonatype.repoServer.ProfileEditor, Sonatype.ext.FormPanel, {
      saveHandler : function( button, event ){
        // Allow contributors to enhance the data before serialization.
        Nexus.staging.repository.RepositoriesMediator.fireEvent('staging.profile.submitted', this.referenceData.properties);

        if (this.isValid())
        {
          this.form.doAction('sonatypeSubmit', {
            method : this.getSaveMethod(),
            url : this.getActionURL(),
            waitMsg : this.isNew ? 'Creating a new record...' : 'Updating records...',
            fpanel : this,
            validationModifiers : this.validationModifiers,
            dataModifiers : this.dataModifiers ? this.dataModifiers.submit : {},
            serviceDataObj : this.referenceData,
            isNew : this.isNew
          });
        }
      },

          loadData : function() {
            if (this.isNew)
            {
              this.form.reset();
              this.fireEvent('load', this.form);
            }
            else
            {
              this.form.doAction('sonatypeLoad', {
                    url : this.getActionURL(),
                    method : 'GET',
                    fpanel : this,
                    dataModifiers : this.dataModifiers.load,
                    scope : this
                  });
            }
          },
          isValid : function() {
            return this.form.isValid() && this.find('name', 'targetGroups')[0].validate();
          },
          loadHandler : function(form, action, receivedData) {
            if (this.groupProfile)
            {
              this.find('name', 'modeInput')[0].setRawValue('Group');
              this.find('name', 'mode')[0].setRawValue('GROUP');
              this.templateStore.filterBy(function(rec, id) {
                    return rec.get('repoType') === 'group';
                  });
            }
            else
            {
              this.templateStore.filterBy(function(rec, id) {
                    return rec.get('repoType') === 'hosted' && (rec.get('format') === 'maven1' || rec.get('format') === 'maven2');
                  });

              // If there is data received (ie, not a newly added record, then do some filtering ( https://issues.sonatype.org/browse/NXCM-3852 )
              if (receivedData) {
                  var templateRec = this.templateStore.getById(receivedData.repositoryTemplateId);
                  this.filterTemplateSelectHandler(templateRec);
              }
            }

          },
          submitHandler : function(form, action, receivedData) {
            if (this.isNew)
            {
              receivedData.sTargetGroups = this.joinGroups(receivedData.targetGroups);
              receivedData.sTarget = this.getRepositoryTarget(receivedData.repositoryTargetId);
              receivedData.sPromotionTargetRepository = this.getRepoName(receivedData.promotionTargetRepository);
              receivedData.sMode = this.getModeName(receivedData.mode);
              return;
            }

            var rec = this.payload;
            rec.beginEdit();
            rec.set('name', receivedData.name);
            rec.set('repositoryTemplateId', receivedData.repositoryTemplateId);
            rec.set('repositoryType', receivedData.repositoryType);
            rec.set('targetGroups', receivedData.targetGroups);
            rec.set('sTargetGroups', this.joinGroups(receivedData.targetGroups));
            rec.set('currentRepositoryIds', receivedData.currentRepositoryIds);
            rec.set('repositoryTargetId', receivedData.repositoryTargetId);
            rec.set('sTarget', this.getRepositoryTarget(receivedData.repositoryTargetId));
            rec.set('finishNotifyRoles', receivedData.finishNotifyRoles);
            rec.set('promotionNotifyRoles', receivedData.promotionNotifyRoles);
            rec.set('dropNotifyRoles', receivedData.dropNotifyRoles);
            rec.set('closeRuleSets', receivedData.closeRuleSets);
            rec.set('promoteRuleSets', receivedData.promoteRuleSets);
            rec.set('promotionTargetRepository', receivedData.promotionTargetRepository);
            rec.set('sPromotionTargetRepository', this.getRepoName(receivedData.promotionTargetRepository));
            rec.set('mode', receivedData.mode);
            rec.set('sMode', this.getModeName(receivedData.mode));
            rec.set('deployURI', receivedData.deployURI);
            rec.set('properties', receivedData.properties)
            rec.commit();
            rec.endEdit();
          },
          exportGroupsHelper : function(value, formPanel) {
            return formPanel.find('name', 'targetGroups')[0].getValue();
          },
          importGroupsHelper : function(value, parent, formPanel) {
            formPanel.find('name', 'targetGroups')[0].setValue(value);
            return value;
          },
          importCloseRuleSetsHelper : function(value, parent, formPanel) {
            formPanel.find('name', 'closeRuleSets')[0].setValue(value);
            return value;
          },
          exportCloseRuleSetsHelper : function(value, formPanel) {
            return formPanel.find('name', 'closeRuleSets')[0].getValue();
          },
          importPromoteRuleSetsHelper : function(value, parent, formPanel) {
            formPanel.find('name', 'promoteRuleSets')[0].setValue(value);
            return value;
          },
          exportPromoteRuleSetsHelper : function(value, formPanel) {
            return formPanel.find('name', 'promoteRuleSets')[0].getValue();
          },
          filterTemplateSelectHandler : function(record) {
            this.currentTemplate = record.get('id');

            var format = record.get('format');
            if (this.payload.get('mode') !== 'GROUP')
            {
              // filter the targets
              this.targetStore.filterBy(function(rec, id) {
                    return rec.get('contentClass') === format;
                  });

              // filter the promotion repos
              this.filteredRepoStore.filterBy(function(rec, id) {
                    return rec.get('format') === format;
                  });

              // filter the target groups
              this.groupStore.filterBy(function(rec, id) {
                    return rec.get('format') === format;
                  });
            }
          },
          templateSelectHandler : function(combo, record, index) {
            var
                  reset = this.currentTemplate !== record.get('id'),
                  groupsField = this.find('name', 'targetGroups')[0],
                  format = record.get('format');

            this.filterTemplateSelectHandler(record);

            if (reset && this.payload.get('mode') !== 'GROUP')
            {
              this.find('name', 'repositoryTargetId')[0].reset();
              this.find('name', 'promotionTargetRepository')[0].reset();
            }

            if(reset) {
              groupsField.setValue([]);
              groupsField.loadStore();
              groupsField.clearInvalid();
            }

            // set the content type
            this.find('name', 'repositoryType')[0].setRawValue(format);
          },
          // Reverted NXCM-2691 (Add support for staging snapshots)
          repositoryTemplateFilter : function(store) {
            var rec, policy, i, records;
            records = store.getRange();
            for (i = records.length - 1; i >= 0; i = i - 1)
            {
              rec = records[i];
              policy = rec.get('repoPolicy');
              // NXCM-3548 only accept maven release repositories
              if (!(rec.get('format') === 'maven2' && rec.get('repoType') !== 'proxy' && (policy === 'RELEASE' || policy === ''))) {
                store.remove(rec);
              }
            }
          }
        });

    Sonatype.Events.addListener('profileViewInit', function(cardPanel, rec, gridPanel) {
          var
                sp = Sonatype.lib.Permissions,
                data = [],
                repoData,filteredRepoStore,
                i, repoRec;

          if (sp.checkPermission('nexus:stagingprofiles', sp.READ) && (sp.checkPermission('nexus:stagingprofiles', sp.CREATE) || sp.checkPermission('nexus:stagingprofiles', sp.DELETE) || sp.checkPermission('nexus:stagingprofiles', sp.EDIT)))
          {
            for (i = 0; i < gridPanel.repoStore.getCount(); i = i + 1)
            {
              repoRec = gridPanel.repoStore.getAt(i);

              // Reverted NXCM-2691 (Add support for staging snapshots)
              if (repoRec.get('repoType') === 'hosted' && repoRec.get('repoPolicy') !== 'SNAPSHOT')
              {
                repoData = [repoRec.get('id'), repoRec.get('name'), repoRec.get('format')];
                data.push(repoData);
              }
            }

            filteredRepoStore = new Ext.data.SimpleStore({
                  fields : [{
                        name : 'id'
                      }, {
                        name : 'name'
                      }, {
                        name : 'format'
                      }, {
                        name : 'repoPolicy'
                      }],
                  data : data
                });

            cardPanel.add(new Sonatype.repoServer.ProfileEditor({
                  name : 'configuration',
                  groupStore : gridPanel.groupStore,
                  targetStore : gridPanel.targetStore,
                  repoStore : gridPanel.repoStore,
                  filteredRepoStore : filteredRepoStore,
                  joinGroups : gridPanel.joinGroups,
                  modeStore : gridPanel.modeStore,
                  fullModeStore : gridPanel.fullModeStore,
                  getRepositoryTarget : gridPanel.getRepositoryTarget,
                  getRepoName : gridPanel.getRepoName,
                  getModeName : gridPanel.getModeName,
                  tabTitle : 'Configuration',
                  payload : rec,
                  groupProfile : rec.get('mode') === 'GROUP'
                }));
          }
        });

});
