/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype*/

define('Sonatype/repoServer/RulesetEditor', ['Nexus/staging/rule/RuleTypeStore'], function() {

    Sonatype.repoServer.RulesetEditor = function(cfg) {
      var
            config = cfg || {},
            defaultConfig = {
              region : 'center',
              width : '100%',
              height : '100%',
              autoScroll : true,
              border : false,
              frame : true,
              collapsible : false,
              collapsed : false,
              buttonAlign : 'center'
            },
            checkColumn = new Ext.grid.CheckColumn({
              header : 'Enabled',
              dataIndex : 'enabled',
              width : 60
            });

      Ext.apply(this, config, defaultConfig);

      this.isNew = this.payload.data.id ? false : true;

      // not using mediators store here since we want to filter it specially to hide some stuff
      this.ruleTypeDataStore = NX.create('Nexus.staging.rule.RuleTypeStore', {
          autoLoad: true,
          filterTesting: window.location.search !== '?debug' // filter out testing.* rules unless ?debug
      });

      this.ruleStore = new Ext.data.SimpleStore({
            id : 0,
            fields : [{
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }, {
                  name : 'enabled'
                }, {
                  name : 'typeId'
                }, {
                  name : 'typeName'
                }, {
                  name : 'properties'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            }
          });

      Sonatype.repoServer.RulesetEditor.superclass.constructor.call(this, {
            items : [{
                  xtype : 'hidden',
                  name : 'id'
                }, {
                  xtype : 'textfield',
                  fieldLabel : 'Name',
                  htmlDecode : true,
                  helpText : 'The name of the rule set',
                  itemCls : 'required-field',
                  name : 'name',
                  width : 250,
                  allowBlank : false
                }, {
                  xtype : 'textfield',
                  fieldLabel : 'Description',
                  htmlDecode : true,
                  helpText : 'The description of the rule set',
                  name : 'description',
                  width : 250
                }, {
                  xtype : 'fieldset',
                  title : 'Rules',
                  name : 'rulesFieldSet',
                  collapsible : false,
                  autoHeight : true,
                  items : [{
                        xtype : 'panel',
                        layout : 'table',
                        layougConfig : {
                          columns : 2
                        },
                        style : 'padding-bottom: 10px',
                        items : [{
                              xtype : 'combo',
                              fieldLabel : 'Rule Type',
                              helpText : 'The type of the rule',
                              name : 'ruleType',
                              width : 530,
                              store : this.ruleTypeDataStore,
                              clearFilterOnReset: false, // do not reset the store filter
                              displayField : 'name',
                              valueField : 'id',
                              editable : false,
                              emptyText : 'Select...',
                              selectOnFocus : true,
                              triggerAction : 'all',
                              lastQuery: '', // do not clear filter on first load
                              forceSelection : true,
                              itemCls : 'required-field',
                              mode : 'local'
                            }, {
                              xtype : 'button',
                              name : 'ruleAdd',
                              text : 'Add',
                              handler : this.ruleAddHandler,
                              scope : this,
                              style : 'padding-left: 30px'
                            }]
                      }, {
                        xtype : 'editorgrid',
                        name : 'rulesGrid',
                        title : 'Applied Rules',
                        autoExpandColumn : 'type-column',
                        width : 600,
                        height : 160,
                        frame : true,
                        store : this.ruleStore,
                        plugins : checkColumn,
                        colModel : new Ext.grid.ColumnModel({
                              defaults : {
                                sortable : true
                              },
                              columns : [checkColumn, {
                                    id : 'type-column',
                                    header : 'Type',
                                    dataIndex : 'typeName'
                                  }, {
                                    width : 30,
                                    renderer : function(value, meta, record, row, col, store) {
                                      return '<img src="' + Sonatype.config.extPath + '/resources/images/gray/tabs/tab-close.gif" height="16" width="16" alt="Delete" />';
                                    }
                                  }]
                            }),
                        selModel : new Ext.grid.RowSelectionModel({
                              singleSelect : true
                            }),
                        listeners : {
                          cellclick : {
                            fn : function(grid, rowIndex, columnIndex, event) {
                              if (columnIndex === 2)
                              {
                                var record = grid.getStore().getAt(rowIndex);
                                grid.getStore().remove(record);
                              }
                            },
                            scope : this
                          }
                        }
                      }]
                }],
            buttons : [{
                  name : 'rulesetSave',
                  text : 'Save',
                  handler : this.saveHandler,
                  scope : this
                }, {
                  name : 'rulesetCancel',
                  text : this.isNew ? 'Cancel' : 'Reset',
                  handler : this.isNew ? this.cancelHandler : this.resetHandler,
                  scope : this
                }]
          });

      this.on('afterlayout', this.initData, this);

      this.initialized = false;
    };

    Ext.extend(Sonatype.repoServer.RulesetEditor, Ext.form.FormPanel, {
          initData : function() {
            if (!this.initialized)
            {
              var
                    i, ruleRec,
                    idField = this.find('name', 'id')[0],
                    nameField = this.find('name', 'name')[0],
                    descriptionField = this.find('name', 'description')[0];

              this.ruleStore.removeAll();

              idField.setValue(this.payload.data.id);
              nameField.setValue(this.payload.data.name);
              descriptionField.setValue(this.payload.data.description);

              for (i = 0; i < this.payload.data.rules.length; i = i + 1)
              {
                ruleRec = new Ext.data.Record(this.payload.data.rules[i], this.payload.data.rules[i].name);
                this.ruleStore.addSorted(ruleRec);
              }

              if (this.isNew)
              {
                nameField.setRawValue('');
                descriptionField.setRawValue('');
              }

              this.initialized = true;
            }
          },

          ruleAddHandler : function() {
            if (!this.validateRuleFields())
            {
              return;
            }

            var
                  typeId = this.find('name', 'ruleType')[0].getValue(),
                  typeName = this.ruleTypeDataStore.getById(typeId).data.name,
                  rec = new Ext.data.Record({}, typeName);

            rec.beginEdit();
            rec.set('enabled', true);
            rec.set('name', typeName);
            rec.set('typeId', typeId);
            rec.set('typeName', typeName);
            rec.commit();
            rec.endEdit();
            this.ruleStore.addSorted(rec);

            this.resetRuleFields();

            this.clearRuleGridInvalid();
          },

          validateRuleFields : function() {
            var
                  records, i,
                  typeCombo = this.find('name', 'ruleType')[0];

            if (typeCombo.getValue() === '')
            {
              typeCombo.markInvalid('This field is required');
              return false;
            }

            records = this.ruleStore.getRange();

            for (i = 0; i < records.length; i = i+1)
            {
              if (records[i].get('typeId') === typeCombo.getValue())
              {
                typeCombo.markInvalid('This Rule is already in the ruleset');
                return false;
              }
            }

            return true;
          },

          resetRuleFields : function() {
            this.find('name', 'ruleType')[0].reset();
          },

          cancelHandler : function() {
            this.fireEvent('cancel', this);
          },
          resetHandler : function() {
            this.initialized = false;
            this.initData();
          },
          saveHandler : function() {
            if (!this.validateRuleSet())
            {
              return;
            }

            /*
              The payload is the record in the upper grid, updating it will trigger the rulesetViewInit event,
              which will in turn recreate the below ruleSetEditor. We could show the saved values before the server
              roundtrip this way, but the grid is updated in the ajax callback as well.

            this.payload.beginEdit();
            this.payload.set('name', this.find('name', 'name')[0].getValue());
            this.payload.set('description', this.find('name', 'description')[0].getValue());
            this.payload.set('rules', this.getRulesData());
            this.payload.commit();
            this.payload.endEdit();
            */

            var
                  submitUrl = this.isNew ? Sonatype.config.servicePath + '/staging/rule_sets' : Sonatype.config.servicePath + '/staging/rule_sets/' + this.payload.data.id,
                  submitData = { data : {} };

            if (this.isNew) {
              submitData.data.id = '';
            } else {
              submitData.data.id = this.payload.data.id;
            }
            submitData.data.name =  this.find('name', 'name')[0].getValue();
            submitData.data.description = this.find('name', 'description')[0].getValue();
            submitData.data.rules = this.getRulesData();

            Ext.Ajax.request({
                  scope : this,
                  method : this.isNew ? 'POST' : 'PUT',
                  url : submitUrl,
                  jsonData : Ext.util.JSON.encode(submitData),
                  callback : function(options, isSuccess, response) {
                    var errMsg, recData;

                    if (!isSuccess)
                    {
                      errMsg = this.isNew ? 'Unable to create the rule set.' : 'Unable to update the rule set.';
                      Sonatype.utils.connectionError(response, errMsg, false, options.options);
                    }
                    else
                    {
                      recData = Ext.util.JSON.decode(response.responseText).data;
                      this.updateGrid(recData);
                    }
                  }
                });

          },
          getRulesData : function() {
            var rulesData = [], i;
            for (i = 0; i < this.ruleStore.getCount(); i = i+1)
            {
              rulesData[i] = this.ruleStore.getAt(i).data;
            }
            return rulesData;
          },
          validateRuleSet : function() {
            var
                  nameField = this.find('name', 'name')[0],
                  currentIndex, existingIndex;

            if (!nameField.validate())
            {
              return false;
            }
            if (this.ruleStore.getCount() === 0)
            {
              this.markRuleGridInvalid('At least one rule should be specified!');
              return false;
            }

            currentIndex = this.payload.store.indexOf(this.payload);
            existingIndex = this.payload.store.find('name', new RegExp('^' + nameField.getValue() + '$'));
            if (this.isNew)
            {
              existingIndex = this.payload.store.find('name', new RegExp('^' + nameField.getValue() + '$'), 1);
            }
            if (existingIndex !== -1 && existingIndex !== currentIndex)
            {
              nameField.markInvalid('A rule set with the same name already exists.');
              return false;
            }
            return true;
          },

          clearRuleGridInvalid : function() {
            var
                  rulesFS = this.find('name', 'rulesFieldSet')[0],
                  rulesWarning = rulesFS.find('name', 'rulesWarning')[0];

            if (rulesWarning)
            {
              rulesFS.remove(rulesWarning);
            }
          },

          markRuleGridInvalid : function(msg) {
            this.clearRuleGridInvalid();
            var rulesFS = this.find('name', 'rulesFieldSet')[0];
            rulesFS.insert(1, {
                  xtype : 'panel',
                  name : 'rulesWarning',
                  layout : 'fit',
                  html : '<div style="padding-bottom:10px; padding-top:5px; color:red">' + 'WARNING: ' + msg + '</div>'
                });
            rulesFS.doLayout();
          },

          updateGrid : function(data) {
            var rec, pos, store = this.payload.store;

            store.remove(this.payload);
            rec = new Ext.data.Record(data, data.resourceURI);
            store.addSorted(rec);
            this.isNew = false;
            this.payload.autoCreateNewRecord = false;

            pos = store.indexOf(rec);
            this.grid.gridPanel.getSelectionModel().selectRow(pos);
          }
        });

    Sonatype.Events.addListener('rulesetViewInit', function(cardPanel, rec, gridPanel) {
          var sp = Sonatype.lib.Permissions;
          if (sp.checkPermission('nexus:stagingrulesets', sp.READ) && (sp.checkPermission('nexus:stagingrulesets', sp.CREATE) || sp.checkPermission('nexus:stagingrulesets', sp.DELETE) || sp.checkPermission('nexus:stagingrulesets', sp.EDIT)))
          {
            cardPanel.add(new Sonatype.repoServer.RulesetEditor({
                  name : 'configuration',
                  payload : rec,
                  grid : gridPanel
                }));
          }
        });
});