/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext,Sonatype,Nexus*/
define('Sonatype/repoServer/SmartProxyCertificateUtils', function() {

  Sonatype.repoServer.SmartProxyCertificateUtils = {
      showAddCertificateWindow: function (label, rec) {
          var window = new Ext.Window({
              title: label,
              closable: true,
              autoWidth: false,
              width: 600,
              autoHeight: true,
              modal: true,
              constrain: true,
              resizable: false,
              draggable: false,
              scope: this,
              items: [
                  {
                      xtype: 'form',
                      id: 'sp_publicKeyForm',
                      name: 'publicKeyForm',
                      labelWidth: 125,
                      frame: true,
                      defaultType: 'textfield',
                      monitorValid: true,
                      items: [
                          {
                              xtype: 'hidden',
                              name: 'id'
                          },
                          {
                              xtype: 'textarea',
                              fieldLabel: 'Certificate',
                              hideLabel: true,
                              name: 'certificate',
                              helpText: 'A certificate to trust (PEM format). Allow the remote Nexus instance with this certificate to access smart proxy services.',
                              height: 200,
                              itemCls: 'required-field',
                              allowBlank: false,
                              wordWrap: false,
                              anchor: Sonatype.view.FIELDSET_OFFSET
                          },
                          {
                              xtype: 'label',
                              text: 'Details',
                              hideLabel: true,
                              style: 'text-decoration: underline; color: blue; cursor: pointer; padding-left: 520px;',
                              listeners: {
                                  render: {
                                      fn: function (component) {
                                          var handler = function () {
                                              var pemField = Ext.getCmp('sp_publicKeyForm').getForm().findField('certificate');

                                              if (rec && rec.data.certificate.detail && rec.data.certificate.pem === pemField.getValue()) {
                                                  this.showCertificateDetailWindow(rec.data.certificate.detail);
                                              }
                                              else {
                                                  Ext.Ajax.request({
                                                      url: Sonatype.config.servicePath + '/smartproxy/certificate-detail',
                                                      method: 'PUT',
                                                      jsonData: {
                                                          pem: Ext.getCmp('sp_publicKeyForm').getForm().findField('certificate').getValue()
                                                      },
                                                      success: function (resp) {
                                                          this.showCertificateDetailWindow(resp.responseText);
                                                      },
                                                      failure: function (resp) {
                                                          if (resp.status === 400) {
                                                              var json = Ext.decode(resp.responseText);
                                                              if (json && json.errors && json.errors[0] && json.errors[0].id === 'certificate') {
                                                                  Sonatype.MessageBox.hide();
                                                                  Ext.getCmp('sp_publicKeyForm').getForm().findField('certificate').markInvalid(json.errors[0].msg);
                                                              }
                                                          }
                                                      },
                                                      scope: this
                                                  });
                                              }
                                          };

                                          component.getEl().on('click', handler.createDelegate(this));
                                      },
                                      scope: this
                                  }
                              }
                          },
                          {
                              fieldLabel: 'Description',
                              xtype: 'textfield',
                              name: 'description',
                              itemCls: 'required-field',
                              allowBlank: false,
                              // for some reason didn't worked value : rec ?
                              // rec.get('description') : '',
                              anchor: Sonatype.view.FIELDSET_OFFSET
                          }
                      ],
                      buttons: [
                          {
                              text: 'Ok',
                              formBind: true,
                              scope: this,
                              handler: function () {
                                  var form, id;
                                  form = window.find('name', 'publicKeyForm')[0];
                                  if (!form.getForm().isValid()) {
                                      return;
                                  }

                                  id = rec ? rec.get('id') : undefined;

                                  form.getForm().doAction('sonatypeSubmit', {
                                      method: id ? 'PUT' : 'POST',
                                      url: Sonatype.config.servicePath + '/smartproxy/trusted-keys' + (id ? '/' + id : ''),
                                      fpanel: form,
                                      serviceDataObj: {
                                          id: '',
                                          certificate: '',
                                          description: ''
                                      },
                                      scope: this,
                                      success: function () {
                                          window.close();
                                          if (this.trustedKeysStore !== undefined) {
                                              this.trustedKeysStore.reload();
                                          }
                                      }
                                  });
                              }
                          },
                          {
                              text: 'Cancel',
                              formBind: false,
                              scope: this,
                              handler: function () {
                                  window.close();
                              }
                          }
                      ]
                  }
              ]
          });

          window.show();

          if (rec) {
              // FIXME for some reason value didn't worked
              window.find('name', 'id')[0].setValue(rec.get('id'));
              window.find('name', 'certificate')[0].setValue(rec.get('certificate').pem);
              window.find('name', 'description')[0].setValue(rec.get('description'));
          }

          return window;
      },

      showCertificateDetailWindow: function (detailText) {
          var window = new Ext.Window({
              title: 'Certificate Details',
              closable: true,
              autoWidth: false,
              width: 750,
              autoHeight: true,
              modal: true,
              constrain: true,
              resizable: false,
              draggable: false,
              scope: this,
              items: [
                  {
                      xtype: 'textarea',
                      id: 'sp_cert_details',
                      height: 400,
                      width: 740,
                      readOnly: true,
                      anchor: Sonatype.view.FIELDSET_OFFSET,
                      listeners: {
                          render: function (component) {
                              component.getEl().on('click', function () {
                                  var pos = component.getEl().dom.scrollTop;
                                  component.selectText();
                                  setTimeout(function () {
                                      component.getEl().dom.scrollTop = pos;
                                  }, 50);
                              });
                          }
                      }
                  }
              ],
              buttons: [
                  {
                      text: 'Close',
                      scope: this,
                      handler: function () {
                          window.close();
                      }
                  }
              ]
          });
          window.show();
          Ext.getCmp('sp_cert_details').setValue(detailText);
      }
  };
});
