/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/
define('nexus-archive-browser-plugin-boot', [
  'Sonatype/repoServer/archiveBrowsingPanel',
  'Sonatype/repoServer/repositoryBrowserContextMenuContribution'
], function() {
        Nexus.Log.debug('Archive Browser plugin loaded');
      }
);
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Sonatype/repoServer/archiveBrowsingPanel', function() {

  Sonatype.repoServer.ArchiveBrowsingPanel = function(config) {
    var config = config || {};
    var defaultConfig = {};
    Ext.apply(this, config, defaultConfig);

    Sonatype.repoServer.ArchiveBrowsingPanel.superclass.constructor.call(this, {
          appendPathToRoot : false,
          leafClickEvent : 'fileContentClicked',
          leafClickPassthru : this,
          resetRootNodeText : false,
          nodeClass : 'leaf-node-title',
          useNodeClassParam : 'leaf',
          autoExpandRoot : false,
          root : new Ext.tree.TreeNode({
                text : ''
              })
        });

    this.refreshHandler();
  };

  Ext.extend(Sonatype.repoServer.ArchiveBrowsingPanel, Sonatype.panels.TreePanel, {
        browseArchive : function(data) {
          if (data == null)
          {
            this.setRootNode(new Ext.tree.TreeNode({
                  text : ''
                }));
          }
          else
          {
            this.setRootNode(new Ext.tree.AsyncTreeNode({
                  text : data.text + ' (Expand to browse archive)',
                  path : '/',
                  expanded : false,
                  loader : this.getLoader()
                }));

            this.getLoader().url = this.contentToArchiveUrl(data.resourceURI);
          }
        },
        contentToArchiveUrl : function(url) {
          // switch out /content/ with /archive/
          // and always use the repository url (as groups are repos now, it will
          // work for them as well)
          return url.replace('/content/', '/archive/').replace('/service/local/repo_groups/', '/service/local/repositories/') + "/!/";
        }
      });

  Sonatype.Events.addListener('fileContentClicked', function(node, panel) {
        if (node.attributes.leaf)
        {
          window.open(node.attributes.rootUrl + node.attributes.path);
        }
      });

  Sonatype.Events.addListener('fileContainerInit', function(items) {
        if (Sonatype.lib.Permissions.checkPermission('nexus:archivefiles', Sonatype.lib.Permissions.READ))
        {
          items.push(new Sonatype.repoServer.ArchiveBrowsingPanel({
                name : 'archiveBrowsingPanel',
                tabTitle : 'Archive Browser',
                preferredIndex : 40
              }));
        }
      });

  Sonatype.Events.addListener('fileContainerUpdate', function(artifactContainer, data) {
        var archiveExtensionFound = false;
        if (Sonatype.lib.Permissions.checkPermission('nexus:archivefiles', Sonatype.lib.Permissions.READ))
        {
          var panel = artifactContainer.find('name', 'archiveBrowsingPanel')[0];
          if (artifactContainer.tabPanel.getActiveTab() == panel)
          {
            artifactContainer.tabPanel.setActiveTab(0);
          }
          artifactContainer.tabPanel.hideTabStripItem(panel);

          if (data == null)
          {
            panel.browseArchive(null);
          }
          else
          {
            artifactContainer.tabPanel.unhideTabStripItem(panel);
            panel.browseArchive(data);
          }
        }
      });

  Sonatype.Events.addListener('artifactContainerInit', function(items) {
        if (Sonatype.lib.Permissions.checkPermission('nexus:archivefiles', Sonatype.lib.Permissions.READ))
        {
          items.push(new Sonatype.repoServer.ArchiveBrowsingPanel({
                name : 'archiveBrowsingPanel',
                tabTitle : 'Archive Browser',
                preferredIndex : 40
              }));
        }
      });

  Sonatype.Events.addListener('artifactContainerUpdate', function(artifactContainer, data) {
        if (Sonatype.lib.Permissions.checkPermission('nexus:archivefiles', Sonatype.lib.Permissions.READ))
        {
          var archiveExtensionFound = false;
          var panel = artifactContainer.find('name', 'archiveBrowsingPanel')[0];

          if (artifactContainer.tabPanel.getActiveTab() == panel)
          {
            artifactContainer.tabPanel.setActiveTab(0);
          }
          artifactContainer.tabPanel.hideTabStripItem(panel);

          if (data == null)
          {
            panel.browseArchive(null);
          }
          else
          {
              data.text = data.nodeName;
              artifactContainer.tabPanel.unhideTabStripItem(panel);
              panel.browseArchive(data);
          }
        }
      });
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Sonatype/repoServer/repositoryBrowserContextMenuContribution', function() {
Sonatype.Events.addListener('repositoryContentMenuInit', function(menu, repoRecord, contentRecord) {
      if (Sonatype.lib.Permissions.checkPermission('nexus:archivefiles', Sonatype.lib.Permissions.READ))
      {
        if (contentRecord.data.resourceURI == null)
        {
          contentRecord.data.resourceURI = contentRecord.data.id;
        }

        var javadocExtension = '-javadoc.jar';

        var index = contentRecord.data.resourceURI.indexOf(javadocExtension);

        if (index != -1 && contentRecord.data.resourceURI.length == index + javadocExtension.length)
        {
          this.archiveResourceURI = contentRecord.data.resourceURI.replace('/content/', '/archive/').replace('/service/local/repo_groups/', '/service/local/repositories/') + '/!/index.html';
          menu.add('-');
          menu.add({
                text : 'View Javadoc',
                scope : this,
                handler : function(node, item, event) {
                  event.stopEvent();
                  window.open(this.archiveResourceURI);
                },
                href : this.archiveResourceURI
              });
        }
      }
    });
});
