/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/
define('nexus-custom-metadata-plugin-boot', [
  'Sonatype/repoServer/ArtifactMetadataPanel',
  'Sonatype/repoServer/ArtifactMetadataSearchContribution'
], function () {
      Nexus.Log.debug('Custom Metadata plugin loaded');
    }
);
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext,Sonatype,Nexus*/
define('Sonatype/repoServer/ArtifactMetadataPanel', function() {

Sonatype.repoServer.ArtifactMetadataPanel = function(config) {
  var config = config || {};
  var defaultConfig = {};
  Ext.apply(this, config, defaultConfig);

  // constructor used to create new records for the datastore
  this.metadataRecord = Ext.data.Record.create([{
        name : 'key',
        sortType : Ext.data.SortTypes.asUCString
      }, {
        name : 'value',
        sortType : Ext.data.SortTypes.asUCString
      }, {
        name : 'readOnly'
      }, {
        name : 'namespace'
      }]);

  // definition of the store, and the fields in it
  this.metadataStore = new Ext.data.SimpleStore({
        fields : [{
              name : 'key',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'value',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'readOnly',
              sortDir : 'DESC'
            }, {
              name : 'namespace'
            }],
        sortInfo : {
          field : 'key',
          direction : 'ASC'
        }
      });

  this.filterText = new Ext.form.TextField({
        name : 'metadata-filter-field',
        emptyText : 'Enter filter term...',
        enableKeyEvents : true,
        listeners : {
          'keyup' : {
            fn : function(field, event) {
              if (!event.isNavKeyPress())
              {
                this.applyFilter(field.getValue());
              }
            },
            scope : this
          }
        }
      });

  // constructor (Panel) call
  Sonatype.repoServer.ArtifactMetadataPanel.superclass.constructor.call(this, {
        // center the buttons on bottom of panel
        buttonAlign : 'center',
        header : false,
        autoScroll : true,
        height : 300,
        store : this.metadataStore,
        enableColumnHide : false,
        enableColumnMove : false,
        // adds isValid() method, and can also start editing first invalid field
        plugins : new Ext.ux.plugins.GridValidator(),
        tbar : [
            // the label
            'Filter:',
            // the filter text field
            this.filterText,
            // the new button, default type of item in toolbar is button
            // so no need to define it
            {
              text : 'Add...',
              icon : Sonatype.config.resourcePath + '/static/images/icons/add.png',
              cls : 'x-btn-text-icon',
              handler : this.newMetadataHandler,
              style : 'padding-left: 20px',
              scope : this,
              disabled : !Sonatype.lib.Permissions.checkPermission('nexus:index', Sonatype.lib.Permissions.CREATE)
            }, {
              text : 'Delete',
              icon : Sonatype.config.resourcePath + '/static/images/icons/delete.png',
              cls : 'x-btn-text-icon',
              handler : this.deleteMetadataHandler,
              scope : this,
              disabled : true
            }],
        bbar : new Ext.ux.StatusBar({
              items : ['']
            }),
        viewConfig : {
          forceFit : true
        },
        colModel : new Ext.grid.ColumnModel({
              store : this.metadataStore,
              defaults : {
                sortable : true
              },
              isCellEditable : function(col, row) {
                var record = this.store.getAt(row);

                // no edit on readonly
                if (record.get('readOnly'))
                {
                  return false;
                }

                // must be for update, don't allow w/o proper priv
                if (!record.newRecord && !Sonatype.lib.Permissions.checkPermission('nexus:index', Sonatype.lib.Permissions.EDIT))
                {
                  return false;
                }

                return Ext.grid.ColumnModel.prototype.isCellEditable.call(this, col, row);
              },
              columns : [
                  // the key column
                  {
                header : 'Key',
                dataIndex : 'key',
                width : 120,
                sortable : true,
                editor : new Ext.form.TextField({
                      // this is so we have access to the grid in the validator
                      gridPanel : this,
                      allowBlank : false,
                      emptyText : 'Enter Key...',
                      validator : this.keyValidator
                    }),
                renderer : function(value, metadata, record, rowIndex, colIndex, store) {
                  if (record.get('readOnly'))
                  {
                    return '<span style="color: #999999">' + value + '</span>';
                  }
                  return '<span>' + value + '</span>';
                }
              },
                  // the value column
                  {
                    header : 'Value',
                    dataIndex : 'value',
                    width : 200,
                    sortable : true,
                    editor : new Ext.form.TextField({
                          allowBlank : false,
                          htmlConvert : true,
                          emptyText : 'Enter Value...'
                        }),
                    renderer : function(value, metadata, record, rowIndex, colIndex, store) {
                      if (record.get('readOnly'))
                      {
                        return '<span style="color: #999999">' + value + '</span>';
                      }
                      return '<span>' + value + '</span>';
                    }
                  },
                  // the source column
                  {
                    header : 'Namespace',
                    dataIndex : 'namespace',
                    width : 150,
                    sortable : true,
                    renderer : function(value, metadata, record, rowIndex, colIndex, store) {
                      if (record.get('readOnly'))
                      {
                        return '<span style="color: #999999">' + value + '</span>';
                      }
                      return '<span>' + value + '</span>';
                    }
                  }]
            }),
        // only allow single select
        selModel : new Ext.grid.RowSelectionModel({
              singleSelect : true,
              listeners : {
                rowselect : {
                  fn : function(selectionmodel, rowIndex, record) {
                    // enable the delete button
                    if (!record.get('readOnly') && Sonatype.lib.Permissions.checkPermission('nexus:index', Sonatype.lib.Permissions.DELETE))
                    {
                      this.enableDeleteButton();
                    }
                    else
                    {
                      this.disableDeleteButton();
                    }
                  },
                  scope : this
                },
                rowdeselect : {
                  fn : function(selectionmodel, rowIndex, record) {
                    this.disableDeleteButton();
                  },
                  scope : this
                }
              }
            }),
        listeners : {
          render : {
            fn : this.renderHandler,
            scope : this
          }
        },
        buttons : [{
              text : 'Save',
              handler : this.saveHandler,
              scope : this,
              disabled : !(Sonatype.lib.Permissions.checkPermission('nexus:index', Sonatype.lib.Permissions.CREATE) || Sonatype.lib.Permissions.checkPermission('nexus:index', Sonatype.lib.Permissions.EDIT))
            }, {
              text : 'Reset',
              handler : this.resetHandler,
              scope : this
            }]
      });
};

Ext.extend(Sonatype.repoServer.ArtifactMetadataPanel, Ext.grid.EditorGridPanel, {
      renderHandler : function() {
        this.tip = new Ext.ToolTip({
              view : this.getView(),
              target : this.getView().mainBody,
              delegate : '.x-grid3-row',
              trackMouse : true,
              renderTo : document.body,
              autoHide : true,
              dismissDelay : 0,
              hideDelay : 0,
              listeners : {
                beforeshow : {
                  fn : function(tip) {
                    var rec = this.store.getAt(tip.view.findRowIndex(tip.triggerElement));
                    if (rec)
                    {
                      if (rec.get('readOnly'))
                      {
                        tip.body.dom.innerHTML = 'This is a read only record that cannot be modified.';
                        return;
                      }
                      else if ((!rec.newRecord) && rec.modified)
                      {
                        tip.body.dom.innerHTML = 'Record modified from key:&quot;';
                        tip.body.dom.innerHTML += rec.modified['key'] ? rec.modified['key'] : rec.get('key');
                        tip.body.dom.innerHTML += '&quot; and value:&quot;';
                        tip.body.dom.innerHTML += rec.modified['value'] ? rec.modified['value'] : rec.get('value');
                        tip.body.dom.innerHTML += '&quot;';
                        return;
                      }
                    }

                    return false;
                  },
                  scope : this
                }
              }
            });
      },
      keyValidator : function(value) {
        var bbar = this.gridPanel.getBottomToolbar();
        bbar.clearStatus();
        if (value.indexOf(' ') > -1)
        {
          var errorString = 'Key cannot contain spaces';
          bbar.setStatus({
                text : errorString,
                iconCls : 'x-statusbar-warning'
              });
          return errorString;
        }

        var regex = new RegExp('[ |-|`|\\^|_|-|,|;|:|!|?|\\.|\'|\\(|\\)|\[|\\]|\\{|\\}|@|*|/|\\\\|&|#|%|+|<|=|>|\\||~|$]');
        if (regex.test(value))
        {
          var errorString = 'Key contains invalid characters';
          bbar.setStatus({
                text : errorString,
                iconCls : 'x-statusbar-warning'
              });
          return errorString;
        }

        // this is only check while actively editing
        if (this.gridPanel.activeEditor)
        {
          // need to look at all other records and make sure the key/namespace
          // combo is unique
          var records = this.gridPanel.metadataStore.getRange();
          var currentRecord = this.gridPanel.activeEditor.record;

          if (currentRecord)
          {
            for (var i = 0; i < records.length; i++)
            {
              if (records[i].id != currentRecord.id)
              {
                if (records[i].get('namespace') == currentRecord.get('namespace') && value == records[i].get('key'))
                {
                  var errorString = 'Key must be unique in each namespace';
                  bbar.setStatus({
                        text : errorString,
                        iconCls : 'x-statusbar-warning'
                      });
                  return errorString;
                }
              }
            }
          }
        }

        return true;
      },
      disableDeleteButton : function() {
        this.getTopToolbar().items.items[3].disable();
      },
      enableDeleteButton : function() {
        this.getTopToolbar().items.items[3].enable();
      },
      // save changes
      saveHandler : function() {
        var bbar = this.getBottomToolbar();
        bbar.clearStatus();

        if (this.isValid(true) && this.data)
        {
          Ext.Ajax.request({
                scope : this,
                method : 'POST',
                options : {
                  dontForceLogout : true
                },
                hideErrorStatus: true,
                success : function() {
                  Sonatype.MessageBox.show({
                    title: 'Metadata Stored',
                    msg: 'Custom metadata stored for artifact.',
                    buttons: Sonatype.MessageBox.OK,
                    icon: Sonatype.MessageBox.INFO
                  });

                  this.resetHandler();
                },
                jsonData : this.buildJsonRequest(),
                url : this.buildUrl()
              });
        }
        else
        {
          bbar.setStatus({
                text : 'One or more fields have validation errors, please correct before saving...',
                iconCls : 'x-statusbar-warning'
              });
        }
      },

      // reset to original data
      resetHandler : function() {
        // clear filter
        this.filterText.setValue("");
        // clear what currently exists
        this.metadataStore.removeAll();
        // clear status text
        this.getBottomToolbar().clearStatus();

        if (this.data && this.data.leaf)
        {
          // populate
          this.retrieveMetadata();
        }
      },

      // filter the data store
      applyFilter : function(text) {
        this.metadataStore.filterBy(function(record, id) {
              var regex = new RegExp('^' + text, 'i');
              if (regex.test(record.get('key')) || regex.test(record.get('value')))
              {
                return true;
              }

              return false;
            }, this);

        this.getView().scrollToTop();
      },
      // add a new empty record to the data store
      newMetadataHandler : function(button, event) {
        var newMetadata = new this.metadataRecord({
              key : '',
              value : '',
              readOnly : false,
              namespace : 'urn:nexus/user#'
            });

        newMetadata.newRecord = true;

        this.metadataStore.insert(0, newMetadata);

        this.getSelectionModel().selectFirstRow();

        this.startEditing(0, 0);
      },
      // delete selected record from the grid
      deleteMetadataHandler : function(button, event) {
        var record = this.getSelectionModel().getSelected();

        if (record)
        {
          this.getStore().remove(record);
        }

        this.getSelectionModel().clearSelections();

        this.disableDeleteButton();
      },
      // update the records in the data store, based upon provided data
      // now is just test data, but will ultimately call into rest api to
      // retrieve metadata
      updateArtifactData : function(data) {
        // save the data as we need it for reset functionality
        this.data = data;
        // reset stuff
        this.resetHandler();
      },

      // retrieve metadata from the server
      retrieveMetadata : function() {
        if (this.data)
        {
          var remoteMetadataStore = new Ext.data.JsonStore({
                root : 'data',
                url : this.buildUrl(),
                autoLoad : true,
                fields : [{
                      name : 'key'
                    }, {
                      name : 'value'
                    }, {
                      name : 'readOnly'
                    }, {
                      name : 'namespace'
                    }],
                sortInfo : {
                  field : 'key',
                  direction : 'ASC'
                },
                listeners : {
                  // when load is complete, we update the grid
                  load : {
                    fn : function(store, records, options) {
                      for (var i = 0; i < records.length; i++)
                      {
                        var newMetadata = new this.metadataRecord({
                              key : records[i].get('key'),
                              value : records[i].get('value'),
                              readOnly : records[i].get('readOnly'),
                              namespace : records[i].get('namespace')
                            }, records[i].get('namespace') + records[i].get('key'));

                        this.metadataStore.add(newMetadata);
                      }

                      this.metadataStore.sortByFields(['readOnly', 'key']);

                      this.getView().scrollToTop();
                    },
                    scope : this
                  }
                }
              });
        }
      },

      // build the json request sent with POST
      buildJsonRequest : function() {
        var json = {
          data : []
        };

        this.metadataStore.each(function(record) {
              // there is no need to POST readOnly data, as we can't change it
              // and the server will just ignore it
              if (record.get('readOnly') == false)
              {
                json.data.push({
                      key : record.get('key'),
                      // Text displayed in the grid is html markup, so we have to decode html entities here
                      // to have them escaped again server-side
                      value : Ext.util.Format.htmlDecode(record.get('value'))
                    });
              }
            }, this);

        return json;
      },

      // build the POST/GET url
      buildUrl : function() {
        if (this.data)
        {
          return url = Sonatype.config.servicePath + '/index/custom_metadata/' + this.data.repoId + '/' + this.buildEncodedUrn();
        }

        return null;
      },

      // build the encoded URN, combines the gav and base64
      buildEncodedUrn : function() {
        if (this.data)
        {
          var urn = 'urn:maven/artifact#' + this.data.groupId + ':' + this.data.artifactId + ':' + this.data.version + ':' + (this.data.classifier ? this.data.classifier : '') + ':' + this.data.extension;

          return Sonatype.utils.base64.encode(urn);
        }

        return null;
      }

    });

// Add the panel to the artifact container
Sonatype.Events.addListener('artifactContainerInit', function(items) {
      // no read privilege, no panel
      if (Sonatype.lib.Permissions.checkPermission('nexus:index', Sonatype.lib.Permissions.READ))
      {
        items.push(new Sonatype.repoServer.ArtifactMetadataPanel({
              name : 'artifactMetadataPanel',
              tabTitle : 'Artifact Metadata',
              preferredIndex : 30
            }));
      }
    });

// Update data in the panel w/ new artifact information
Sonatype.Events.addListener('artifactContainerUpdate', function(artifactContainer, data) {
      var panel = artifactContainer.find('name', 'artifactMetadataPanel')[0];

      // if we don't find the panel, it wasn't loaded, probably because of perms
      if (panel)
      {
        panel.updateArtifactData(data);
      }
    });

});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext,Sonatype,Nexus*/
define('Sonatype/repoServer/ArtifactMetadataSearchContribution', function() {

// Add the custom metadata search
Sonatype.Events.addListener('searchTypeInit', function(searchTypes, panel) {
      var enterHandler = function(f, e) {
        if (e.getKey() == e.ENTER)
        {
          this.startSearch(this, true);
        }
      };

      var typeList = {
        equal : 'Equals',
        matches : 'Matches',
        bound : 'Key Defined',
        notEqual : 'Not Equal'
      };

      var metadataPopulator = function(panel, data) {
        // key
        if (data.length > 1)
        {
          panel.getTopToolbar().items.itemAt(2).setRawValue(data[1]);
        }
        // type
        if (data.length > 2)
        {
          panel.getTopToolbar().items.itemAt(4).value = data[2];
          panel.getTopToolbar().items.itemAt(4).setText(typeList[data[2]]);
        }
        // value
        if (data.length > 3)
        {
          panel.getTopToolbar().items.itemAt(6).setRawValue(data[3]);
        }
      };

      var m2FreeformStore = new Sonatype.SearchStore({
            grid : panel.grid,
            searchUrl : Sonatype.config.servicePath + '/search/m2/ngfreeform'
          });

      searchTypes.push({
            value : 'metadata',
            text : 'Metadata Search',
            scope : panel,
            store : m2FreeformStore,
            handler : panel.switchSearchType,
            quickSearchCheckHandler : function(panel, value) {
              return value.indexOf('=') > -1;
            },
            quickSearchHandler : function(panel, value) {
              var parts = value.split('=');
              var data = ['metadata'];

              if (parts.length == 2)
              {
                data.push(parts[0]);
                data.push('equal');
                data.push(parts[1]);
              }
              metadataPopulator(panel, data);
            },
            searchHandler : function(panel) {
              this.grid.store.baseParams = {};

              // key
              var v = panel.getTopToolbar().items.itemAt(2).getRawValue();
              if (v)
              {
                panel.grid.store.baseParams['p'] = v;
              }
              // type
              var v = panel.getTopToolbar().items.itemAt(4).value;
              if (v)
              {
                panel.grid.store.baseParams['t'] = v;
              }
              // value
              v = panel.getTopToolbar().items.itemAt(6).getRawValue();
              if (v)
              {
                panel.grid.store.baseParams['v'] = v;
              }

              if (panel.grid.store.baseParams['p'] == null)
              {
                panel.setWarningLabel('Key is required to run a search.');
                return;
              }

              panel.clearWarningLabel();

              panel.fetchRecords(panel);
            },
            applyBookmarkHandler : function(panel, data) {
              metadataPopulator(panel, data);
              panel.startSearch(this, false);
            },
            getBookmarkHandler : function(panel) {
              var result = panel.searchTypeButton.value;
              // key
              result += '~';
              var v = panel.getTopToolbar().items.itemAt(2).getRawValue();
              if (v)
              {
                result += v;
              }
              // type
              result += '~';
              v = panel.getTopToolbar().items.itemAt(4).value;
              if (v)
              {
                result += v;
              }
              // value
              result += '~';
              var v = panel.getTopToolbar().items.itemAt(6).getRawValue();
              if (v)
              {
                result += v;
              }
              return result;
            },
            panelItems : ['Key:', {
                  xtype : 'textfield',
                  id : 'metadatasearch-key',
                  size : 80,
                  listeners : {
                    'specialkey' : {
                      fn : enterHandler,
                      scope : panel
                    }
                  }
                }, {
                  xtype : 'tbspacer'
                }, {
                  id : 'metadatasearch-type',
                  text : 'Equals',
                  value : 'equal',
                  tooltip : 'Click for more search options',
                  scope : panel,
                  menu : {
                    items : [{
                          text : typeList.equal,
                          value : 'equal',
                          scope : panel,
                          handler : function() {
                            this.getTopToolbar().items.itemAt(4).value = 'equal';
                            this.getTopToolbar().items.itemAt(4).setText(typeList.equal);
                          }
                        }, {
                          text : typeList.matches,
                          value : 'matches',
                          scope : panel,
                          handler : function() {
                            this.getTopToolbar().items.itemAt(4).value = 'matches';
                            this.getTopToolbar().items.itemAt(4).setText(typeList.matches);
                          }
                        }, {
                          text : typeList.bound,
                          value : 'bound',
                          scope : panel,
                          handler : function() {
                            this.getTopToolbar().items.itemAt(4).value = 'bound';
                            this.getTopToolbar().items.itemAt(4).setText(typeList.bound);
                          }
                        }, {
                          text : typeList.notEqual,
                          value : 'notEqual',
                          scope : panel,
                          handler : function() {
                            this.getTopToolbar().items.itemAt(4).value = 'notEqual';
                            this.getTopToolbar().items.itemAt(4).setText(typeList.notEqual);
                          }
                        }]
                  }
                }, 'Value:', {
                  xtype : 'textfield',
                  id : 'metadatasearch-value',
                  size : 80,
                  listeners : {
                    'specialkey' : {
                      fn : enterHandler,
                      scope : panel
                    }
                  }
                }, {
                  xtype : 'tbspacer'
                }, {
                  icon : Sonatype.config.resourcePath + '/static/images/icons/search.gif',
                  cls : 'x-btn-icon',
                  scope : panel,
                  handler : function() {
                    this.startSearch(this, true);
                  }
                }]
          });
    });

});
