/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Sonatype, Nexus, Ext*/

// FIXME: Re-write as NX.define singleton=true

define('Nexus/clm', [
  'Nexus/config',
  'Nexus/messages',
  'Nexus/siesta'
], function () {

  Ext.namespace('Nexus.clm');

  /**
   * Base identifier for all plugins bits.
   *
   * @type {String}
   * @static
   */
  Nexus.clm.ID = 'nx-clm';

  /**
   * Title for various bits.
   *
   * @type {String}
   * @static
   */
  Nexus.clm.TITLE = 'IQ Server Connection';

  /**
   * Title for dashboard.
   *
   * @type {String}
   * @static
   */
  Nexus.clm.TITLE_DASHBOARD = 'IQ Server Dashboard';

  /**
   * Title for the tab.
   * @type {string}
   */
  Nexus.clm.TAB_TITLE = "IQ Server Connection";

  /**
   * ID for the tab.
   * @type {string}
   */
  Nexus.clm.TAB_ID = "clm";

  /**
   * The key for data collected in Staging form.
   * @type {string}
   */
  Nexus.clm.STAGING_KEY = 'CLMApplicationID';

  /**
   * The label to show in Staging form.
   * @type {string}
   */
  Nexus.clm.STAGING_LABEL = 'IQ Server Application';

  /**
   * Show a user-token transient message.
   *
   * @param message
   * @static
   */
  Nexus.clm.showMessage = function (message) {
    Nexus.messages.show(Nexus.clm.TITLE, message);
  };

  /**
   * Returns the path to a clm REST resource.
   *
   * @param path
   * @return {String}
   * @static
   */
  Nexus.clm.resourcePath = Nexus.siesta.basePath + '/clm';
  
  Nexus.clm.testConnectionPath = Nexus.clm.resourcePath + '/testConnection';
  
  Nexus.clm.componentDetailContentResourcePath = Nexus.siesta.basePath + '/healthcheck/clm/assets/version-graph/rm/nexus';

  Nexus.clm.firewallResourcePath = Nexus.clm.resourcePath + '/firewall';

  Nexus.clm.firewallSummaryResourcePath = Nexus.clm.firewallResourcePath + '/summary';

  Nexus.clm.ruleTemplate = NX.create('Ext.XTemplate',
        '<div class="nx-clm-status {messageClass}">{message}</div>',
        '<div class="nx-staging-repository-ActivityView-messagePanel-head">',
          '<tpl if="violationCount &gt; 0">Violations were found in <b>{violationCount}</b> components.</tpl>',
          '<tpl if="violationCount == 0">No violations found.</tpl>',
        '</div>',
        '<div class="nx-clm-alerts">',
          '<tpl if="criticalCount &gt; 0">',
            '<div class="nx-clm-alert_box">',
              '<div class="nx-clm-chiclet critical">{criticalCount}</div>',
              '<div class="nx-clm-alert_critical_text">CRITICAL</div>',
            '</div>',
          '</tpl>',
          '<tpl if="severeCount &gt; 0">',
            '<div class="nx-clm-alert_box">',
             '<div class="nx-clm-chiclet severe">{severeCount}</div>',
              '<div class="nx-clm-alert_severe_text">SEVERE</div>',
            '</div>',
          '</tpl>',
          '<tpl if="moderateCount &gt; 0">',
            '<div class="nx-clm-alert_box">',
              '<div class="nx-clm-chiclet moderate">{moderateCount}</div>',
              '<div class="nx-clm-alert_moderate_text">MODERATE</div>',
            '</div>',
          '</tpl>',
          '<a href="{detailedReport}" target="_blank" class="view-report-button">View Full Report</a>',
        '</div>',
        '<br style="clear:both;"/>',
        {
          compiled : true
        }
  );

  Nexus.clm.chickletTemplate = NX.create('Ext.XTemplate',
        '<tpl if="criticalCount &gt; 0">',
          '<div class="nx-clm-chiclet-small critical">{criticalCount}</div>',
        '</tpl>',
        '<tpl if="severeCount &gt; 0">',
          '<div class="nx-clm-chiclet-small severe">{severeCount}</div>',
        '</tpl>',
        '<tpl if="moderateCount &gt; 0">',
          '<div class="nx-clm-chiclet-small moderate">{moderateCount}</div>',
        '</tpl>',
        '<tpl if="(criticalCount + severeCount + moderateCount) == 0">No violations</tpl>',
        '<tpl if="detailedReport">',
          '<span class="view-report-link-span">',
            '<a href="{detailedReport}" target="_blank" class="view-report-link">View Full Report</a>',
          '</span>',
        '</tpl>',
        {
          compiled : true
        }
  );

  Nexus.clm.stagingRule = function(event, detail) {
    if (detail.getPropertyValue('typeId') === 'iq-policy' || detail.getPropertyValue('typeId') === 'clm') {
      var message, messageClass;

      var status = detail.getPropertyValue('x.appScan.status');
      if(status === undefined)
      {
        NX.log.error("Unable to determine status for IQ Server result:" + detail);
        return undefined;
      }

      switch (status) {
        case 'PASS':
          message = 'Validation was successful!';
          messageClass = 'nx-clm-status_success';
          break;
        case 'WARN':
          message = 'There are warnings.';
          messageClass = 'nx-clm-status_warn';
          break;
        case 'FAIL':
          message = 'Validation failed.';
          messageClass = 'nx-clm-status_fail';
          break;
        default :
          Nexus.clm.showMessage("Unrecognized IQ Server status: "+ details.getPropertyValue('x.appScan.status'));
      }
      var violationCount = detail.getPropertyValue('x.appScan.violationCount') || 0;
      var criticalCount = detail.getPropertyValue('x.appScan.threatLevelCount.CRITICAL') || 0;
      var severeCount = detail.getPropertyValue('x.appScan.threatLevelCount.SEVERE') || 0;
      var moderateCount = detail.getPropertyValue('x.appScan.threatLevelCount.MODERATE') || 0;

      return NX.create('Ext.XTemplate',
            '{[ this.render() ]}',
            {
              compiled : true,
              render : function() {
                return Nexus.clm.ruleTemplate.apply({
                  message: message,
                  messageClass: messageClass,
                  detailedReport : detail.getPropertyValue('x.appScan.url'),
                  violationCount : violationCount,
                  criticalCount : criticalCount,
                  severeCount : severeCount,
                  moderateCount : moderateCount
                });
              }
            });
    }

    return undefined;
  };
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/firewall/FirewallRepositoryContrib', ['Nexus/siesta'], function() {

  FirewallRepositoryContrib = function(config) {
    var me = this;
    me.reset = function(config) {
      var config = config || {};
      var defaultConfig = {
        grid: null
      };
      me.summaryData = {};
      me.summaryDataError = {};

      Ext.apply(me, config, defaultConfig);

      if (me.grid) {
        function getRepoSummary(record) {
          delete me.summaryData[record.data.id];
          delete me.summaryDataError[record.data.id];
          if (!me.isProxyRepository(record)) {
            me.summaryDataError[record.data.id] = '';
            me.grid.getView().refreshRow(record);
          }
          else {
            Ext.Ajax.request({
              url: Nexus.clm.firewallSummaryResourcePath + '/' + encodeURIComponent(record.data.id),
              method: 'GET',
              scope: self,
              suppressStatus: [400, 404, 409],
              success: function(response) {
                me.summaryData[record.data.id] = Ext.decode(response.responseText);
                me.grid.getView().refreshRow(record);
              },
              failure: function(response, opts) {
                var errorText = 'Unable to retrieve summary data';
                //400 means firewall isn't configured, so just leave column empty
                if (response.status === 400) {
                  errorText = '';
                }
                else if (response.status === 409) {
                  me.summaryData[record.data.id] = response.responseText;
                  errorText = '';
                }
                else if (response.siestaError) {
                  errorText = response.siestaError.message;
                }
                me.summaryDataError[record.data.id] = errorText;
                me.grid.getView().refreshRow(record);
              }
            });
          }
        }

        me.grid.store.addListener('load', function(store, records, options) {
          Ext.each(records, function(record) {
            getRepoSummary(record);
          });
        });

        me.grid.store.addListener('add', function(store, records, index) {
          Ext.each(records, function(record) {
            getRepoSummary(record);
          });
        });
      }
    };

    me.reset(config);

    me.isProxyRepository = function(record) {
      return record.get('repoType') == 'proxy';
    };

    me.updateColumns = function(columns) {
      columns.splice(8, 0, {
        header: 'IQ Policy Violations',
        id: 'firewallColumn',
        width: 175,
        renderer: function(value, metadata, record, rowIndex, colIndex, store) {
          if (me.summaryDataError[record.data.id]) {
            me.updateTooltip();
            return '<img class="nx-firewall-error-icon" src="' + Sonatype.config.resourcePath +
                '/static/images/exclamation.gif"><span class="nx-firewall-error">' +
                me.summaryDataError[record.data.id] + '</span>';
          }
          var summaryData = me.summaryData[record.data.id];

          if (summaryData) {
            if (typeof summaryData === 'string') {
              return Ext.util.Format.htmlEncode(summaryData);
            }

            function getChiclet(cssClass, count) {
              return '<div class="nx-firewall-chiclet ' + cssClass + '">' + count + '</div>';
            }

            var html = '<div class="nx-firewall-container">';
            if (summaryData.criticalComponentCount) {
              html += getChiclet('critical', summaryData.criticalComponentCount);
            }
            if (summaryData.severeComponentCount) {
              html += getChiclet('severe', summaryData.severeComponentCount);
            }
            if (summaryData.moderateComponentCount) {
              html += getChiclet('moderate', summaryData.moderateComponentCount);
            }
            if (!summaryData.criticalComponentCount && !summaryData.severeComponentCount && !summaryData.moderateComponentCount) {
              html += '<div style="float:left;">No violations</div>';
            }
            if (summaryData.quarantinedComponentCount) {
              html += '<div class="nx-firewall-quarantine-icon"></div><div class="nx-firewall-quarantine-count">' + summaryData.quarantinedComponentCount + '</div>';
            }
            html += '<div class="nx-external-link"></div>';
            if (summaryData.errorMessage) {
              me.updateTooltip();
              html += '<div class="nx-firewall-iq-error-icon"></div>';
              html += '<div class="nx-firewall-error">' + Ext.util.Format.htmlEncode(summaryData.errorMessage) + '</div>';
            }
            html += '</div>';

            return html;
          }
          return '';
        },
        clickHandler: function(grid, rowIndex, eventObject) {
          var record = me.grid.getStore().getAt(rowIndex);
          var summaryData = me.summaryData[record.data.id];
          if (summaryData && typeof summaryData === 'object') {
            window.open(summaryData.reportUrl, '_blank');
          }
        }
      });

      me.updateTooltip = function() {
        var view = me.grid.getView();
        if (me.grid.tip) {
          me.grid.tip.destroy();
        }
        me.grid.tip = new Ext.ToolTip({
          target : view.mainBody,
          delegate : '.x-grid3-cell',
          maxWidth : 500,
          mouseOffset : [ 0, 0 ],
          renderTo : document.body,
          listeners : {
            beforeshow : {
              fn : me.tooltipBeforeShowListener
            }
          }
        });
      };

      me.tooltipBeforeShowListener = function(tip) {
        tip.hideDelay = 200;
        var view = me.grid.getView(),
            rowIndex = view.findRowIndex(tip.triggerElement),
            cellIndex = view.findCellIndex(tip.triggerElement),
            record = me.grid.store.getAt(rowIndex);

        if (!record || cellIndex != me.grid.getColumnModel().getIndexById('firewallColumn')) {
          return false;
        }

        var tooltipText = me.summaryDataError[record.data.id];
        if (!tooltipText && me.summaryData[record.data.id]) {
          tooltipText = me.summaryData[record.data.id].errorMessage;
        }

        if (Ext.isEmpty(tooltipText)) {
          return false;
        }

        tip.body.dom.innerHTML = Ext.util.Format.htmlEncode(tooltipText);
      };
    };

    Sonatype.Events.addListener('repositoryPanelInit', function(repoPanel) {
      me.reset({
        grid: repoPanel.gridPanel
      });
      return true;
    });

    Sonatype.Events.addListener('repositoryGridInit', function(repoPanel, columns) {
      me.updateColumns(columns);
      return true;
    });
  };

  Sonatype.FirewallRepositoryContrib = new FirewallRepositoryContrib();
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/healthcheck/HealthCheckProRepositoryContrib', ['Nexus/healthcheck/HealthCheckUtil'], function() {

  Sonatype.HealthCheckDetailWindow = function(config) {
      var config = config || {};
      var defaultConfig = {
          layout : 'fit',
          items : [ {
              xtype : 'iframepanel',
              loadMask : {
                  msg : 'Loading Health Check Detail...'
              },
              autoShow : true,
              fitToParent : true,
              defaultSrc : function() {
                  return config.url;
              }
          } ]
      };
      Ext.apply(this, config, defaultConfig);
      Sonatype.HealthCheckDetailWindow.superclass.constructor.call(this, {});
  };

  Ext.extend(Sonatype.HealthCheckDetailWindow, Ext.Panel, {});

  HealthCheckProRepositoryContrib = function() {
      var me = this;
      Sonatype.Events.addListener('historyChanged', function(token) {
          if (token) {
              var toks = token.split(Sonatype.view.HISTORY_DELIMITER);
              if (toks[0] == 'healthCheckPro') {
                  if (Sonatype.user.curr.isLoggedIn) {
                      if (me.hasHealthCheckDetailReadPerm()) {
                          var name = decodeURIComponent(toks[1]);
                          var repoId = decodeURIComponent(toks[2]);
                          var path = decodeURIComponent(toks[3]);

                          var id = me.toTabId(name, repoId, path);
                          var url = Sonatype.HealthCheckUtil.getBase() + '/healthCheckDetail/' + repoId + '/current/' + path;

                          Sonatype.view.mainTabPanel.addOrShowTab(id, Sonatype.HealthCheckDetailWindow, {
                              title : 'RHC-' + name,
                              url : url
                          });
                      }
                  } else {
                      Sonatype.view.afterLoginToken = token;
                      Sonatype.repoServer.RepoServer.loginHandler();
                  }
                  //was handled by this event listener so return false to stop others
                  return false;
              }
          }
      });

      Sonatype.Events.addListener('healthCheckWindowInit', function(repoContrib, toolbar, record, healthcheckdata) {
          // require user to be logged in before adding the button, also require that license supports details
          if (healthcheckdata.detailedReportSupported && me.hasHealthCheckDetailReadPerm() && Sonatype.user.curr.isLoggedIn) {
              if (Ext.isEmpty(healthcheckdata.healthCheckDetailUrl)) {
                  toolbar.push('->', 'The next analysis will include the detailed report.');
              } else {
                  toolbar.push('->', {
                      text : 'View Detailed Report',
                      handler : function(button) {
                          var url = healthcheckdata.healthCheckDetailUrl;
                          var repoId = healthcheckdata.id;
                          var name = record.get('name');
                          var versionBegin = url.indexOf('/' + healthcheckdata.id + '/') + ('/' + healthcheckdata.id + '/').length;
                          var versionEnd = url.indexOf('/', versionBegin);
                          var version = url.substring(versionBegin, versionEnd);
                          var path = url.substring(versionEnd + 1);

                          var id = me.toTabId(name, repoId, path);

                          Sonatype.view.mainTabPanel.addOrShowTab(id, Sonatype.HealthCheckDetailWindow, {
                              title : 'RHC-' + name,
                              url : url
                          });

                          repoContrib.closeHealthCheckWindow();
                      },
                      record : record
                  });
              }
          }
      });

      me.hasHealthCheckDetailReadPerm = function() {
          return Sonatype.lib.Permissions.checkPermission('nexus:healthcheckdetail', Sonatype.lib.Permissions.READ);
      };

      me.toTabId = function(name, repoId, path) {
          return 'healthCheckPro' + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(name) + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(repoId)
                                  + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(path);
      };
  };

  Sonatype.HealthCheckProRepositoryContrib = new HealthCheckProRepositoryContrib();
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/
define('Nexus/healthcheck/HealthCheckProUtil', function() {

  HealthCheckProUtil = function() {
      var me = this;

      me.hasPaidProLicense = function() {
          var suffix = Sonatype.utils.editionShort.substr(Sonatype.utils.editionShort.lastIndexOf('-') + 1);
          return 'EVAL' != suffix && 'OSS' != suffix;
      };

      me.hasNonFreeProLicense = function() {
          var suffix = Sonatype.utils.editionShort.substr(Sonatype.utils.editionShort.lastIndexOf('-') + 1);
          return 'OSS' != suffix;
      };

      me.hasAlertReadPerm = function() {
          return Sonatype.lib.Permissions.checkPermission('nexus:healthcheckalerts', Sonatype.lib.Permissions.READ) && Sonatype.user.curr.isLoggedIn;
      };

      me.hasAlertUpdatePerm = function() {
          return Sonatype.lib.Permissions.checkPermission('nexus:healthcheckalerts', Sonatype.lib.Permissions.EDIT) && Sonatype.user.curr.isLoggedIn;
      };
  };

  Ext.onReady(function() {
      Sonatype.HealthCheckProUtil = new HealthCheckProUtil();
  });
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/healthcheck/HealthCheckArtifactDetailsContrib', [ 'Nexus/healthcheck/HealthCheckUtil', 'Nexus/healthcheck/HealthCheckProUtil', 'Nexus/clm/ComponentDetailView' ], function() {

  HealthCheckArtifactDetailsContrib = function() {
      function getInsight(panel) {
        if (panel && panel.items.getCount() === 2) {
          return panel.items.itemAt(1).iframe.dom.contentWindow.Insight;  
        }
      }
      
      var me = this;

      me.getFormat = function() {
        if (me.data && me.data.format) {
          if (me.data.format === 'maven2') {
            return 'maven';
          } else if (me.data.format === 'nuget') { 
            return 'nuget'; 
          }
        }
        return null;
      };
      
      me.getLoadingHtml = function() {
          return '<div style="background:#ececec; width: 100%; height: 100%; padding: 20px;font-size: 14px;font-family: Arial, Helvetica, sans-serif">Loading...</div>';
      };

      me.getFileUrl = function() {
          var url = Sonatype.HealthCheckUtil.getBase() + '/healthCheckFileDetail/' + Sonatype.utils.base64.encode(me.data.resourceURI) + '/index.html?repoId=' + me.data.repoId;
          var format = me.getFormat();
          if (format) { 
            return url + '&format=' + format; 
          }
          return url;
      };

      me.getHashUrl = function(repoId, itemPath) {
        return Sonatype.HealthCheckUtil.getBase() + '/healthCheckFileDetail/hash/' + repoId + '/' + itemPath;
      };
      
      me.getComponentIdentifierUrl = function(format, itemPath) {
        return Sonatype.HealthCheckUtil.getBase() + '/healthCheckFileDetail/componentIdentifier/' + format + '/' + itemPath;
      };

      me.toTabId = function(appId, hash, component, legacy) {
        var result = 'compDetail' + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(appId);
        result += Sonatype.view.HISTORY_DELIMITER + (hash ? encodeURIComponent(hash) : '');
        result += Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(legacy);
        result += Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(component.format);
        
        var coordinates = [];
        var key = null;
        if (component.coordinates) {
          for (key in component.coordinates) {
            if (component.coordinates.hasOwnProperty(key)) {
              coordinates.push(key);
            }
          }
        }
        coordinates.sort();
        
        for (var i = 0; i < coordinates.length; i++) {
          key = coordinates[i];
          var coordinateValue = component.coordinates[key];
          result += Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(key);
          result += Sonatype.view.HISTORY_DELIMITER + (coordinateValue ? encodeURIComponent(coordinateValue) : '');
        }
        
        return result;
      };

      me.fromTabId = function(tokens) {
        var result = {};
        result.appId = decodeURIComponent(tokens[1]);
        result.hash = decodeURIComponent(tokens[2]);
        result.legacy = decodeURIComponent(tokens[3]) === 'true';
        result.component = {};
        result.component.coordinates = {};
        result.component.format = decodeURIComponent(tokens[4]);
        for (var i = 5; i < tokens.length; i += 2) {
          result.component.coordinates[decodeURIComponent(tokens[i])] = decodeURIComponent(tokens[i + 1]);
        }
        return result;
      };

      me.isLegacyClm = function(panel) {
        if (getInsight(panel).registerCoordsViewDetailsListener || getInsight(panel).setCoordinates) { 
          return false;
        }
  
        return true;
      };
      
      me.showDetailsTab = function(appId, hash, component, legacy) {
        Sonatype.view.mainTabPanel.addOrShowTab(me.toTabId(appId, hash, component, legacy),
                Nexus.clm.ComponentDetailView, {
                  title: 'Component Details',
                  appId: appId,
                  hash: hash,
                  component: !legacy ? component : null,
                  gav: legacy ? component.coordinates : null
                });
      };
      
      me.registerLegacyClmViewDetailsListener = function(panel) {
        getInsight(panel).registerViewDetailsListener(
                function(appId, groupId, artifactId, version, classifier, extension, hash, matchState,
                        proprietary) {

                  var component = {};
                  component.format = 'maven';
                  component.coordinates = {
                    groupId: groupId,
                    artifactId: artifactId,
                    version: version,
                    classifier: classifier,
                    extension: extension
                  };

                  me.showDetailsTab(appId, hash, component, true);
                });
      };

      me.registerClmViewDetailsListener = function(panel) {
        if (!me.isLegacyClm(panel)) {
          getInsight(panel).registerCoordsViewDetailsListener(
                  function(appId, format, coordinates, hash, matchState) {

                    var component = {};
                    component.format = format;
                    component.coordinates = {};
                    if (coordinates) {
                      for (var i = 0; i < coordinates.length; i += 2) {
                        component.coordinates[coordinates[i]] = coordinates[i + 1];
                      }
                    }

                    me.showDetailsTab(appId, hash, component, false);
                  });
        } else {
          me.registerLegacyClmViewDetailsListener(panel);
        }
      };
      
      me.setAuthFailureHandler = function(panel) {
        getInsight(panel).setAuthFailureHandler(function () {
          Sonatype.repoServer.RepoServer.loginHandler();
        });
      };
      
      me.isClmLoaded = function(panel) {
        return getInsight(panel);
      };

      me.getItemPath = function() {
        if (me.data && me.data.resourceURI) {
          var segments = me.data.resourceURI.split('/');

          var index = segments.indexOf('content');
          if (index <= -1 || index + 1 >= segments.length) {
            return null;
          }

          return segments.slice(index + 1).join('/');
        }

        return null;
      };
      
      me.setCoordinates = function(itemPath, setCipCoordinates) {
        var format = me.getFormat();

        if (itemPath && me.data && format) {
          Ext.Ajax.request({
            url: me.getComponentIdentifierUrl(format, itemPath),
            success: function(result) {
              var componentIdentifier = JSON.parse(result.responseText);
              setCipCoordinates(itemPath, format, componentIdentifier.coordinates);
            },
            failure: function(result) {
              setCipCoordinates(itemPath, format, null);
            }
          });
        } else {
          setCipCoordinates(itemPath, format, null);
        }
      };

      me.setLegacyClmComponent = function(panel, format, coordinates, hash) {
        if (format !== 'maven') {
          var error = {
            errorMessage: 'Unsupported repository format, please upgrade to at least CLM 1.13.',
            errorCode: 400
          };
          getInsight(panel).setError(error);
          return;
        }

        var gav = {
          groupId: coordinates.groupId,
          artifactId: coordinates.artifactId,
          version: coordinates.version,
          extension: coordinates.extension,
          classifier: coordinates.classifier,
          hash: hash
        };
        
        getInsight(panel).setGav(gav);
      };
          
      me.setClmComponent = function(panel) {
        var setCipCoordinates = function(itemPath, format, coordinates) {
          if (coordinates && me.data && me.data.repoId && format) {
            Ext.Ajax.request({
              url: me.getHashUrl(me.data.repoId, itemPath),
              success: function(result) {
                if (me.isClmLoaded(panel)) {
                  if (!me.isLegacyClm(panel)) {
                    getInsight(panel).setCoordinates(format, coordinates, {
                      hash: result.responseText
                    });
                  } else {
                    me.setLegacyClmComponent(panel, format, coordinates, result.responseText);
                  }
                }
              },
              failure: function(result) {
                if (me.isClmLoaded(panel)) {
                  var errorMessage = 'Unable to retrieve hash for component.';
                  var error = {
                    errorMessage: (result.siestaError ? result.siestaError.message : errorMessage),
                    errorCode: result.status
                  };
                  getInsight(panel).setError(error);
                }
              }
            });
          } else {
            var errorMessage = 'Unable to retrieve component information.';
            var errorCode = 500;

            if (me.data && !format) {
              errorMessage = 'Unsupported repository format: ' + me.data.format + '.';
              errorCode = 400;
            } else if (!coordinates) {
              errorMessage = 'Unable to parse repository path "' + itemPath + '" into valid coordinates.';
            }

            var error = {
              errorMessage: errorMessage,
              errorCode: errorCode
            };
            var insightPanel = getInsight(panel);
            if (insightPanel) {
              insightPanel.setError(error);
            }
          }
        };

        me.setCoordinates(me.getItemPath(), setCipCoordinates);
      };

      me.buildLoadingPanel = function() {
          return new Ext.Panel({
              width : '100%',
              height : '100%',
              border : true,
              collapsible : false,
              collapsed : false,
              html : me.getLoadingHtml()
          });
      };

      me.buildDetailPanel = function(config) {
          var config = config || {};
          var defaultConfig = {
              width : '100%',
              height : '100%',
              border : true,
              collapsible : false,
              collapsed : false,
              layout : 'card',
              items : [ me.buildLoadingPanel() ]
          };

          Ext.applyIf(config, defaultConfig);
          return new Ext.Panel(config);
      };

      me.rebuildIframe = function(panel, url) {
          if (me.isClmLoaded(panel)) {
              me.setClmComponent(panel);
          } else {
              //item 0 is the loading panel, don't ever remove that one
              panel.getLayout().setActiveItem(0);

              var old = panel.items.get(1);
              if (old) {
                  panel.remove(old);
                  old.destroy();
              }

              panel.add({
                xtype: 'iframepanel',
                border: false,
                disableMessaging: false,
                defaultSrc: url,
                listeners: {
                  domready: function() {
                    panel.getLayout().setActiveItem(1);

                    var count = 0;
                    var functionId = setInterval(function(){
                      if (me.isClmLoaded(panel)) {
                        me.setClmComponent(panel);
                        me.registerClmViewDetailsListener(panel);
                        me.setAuthFailureHandler(panel);
                        clearInterval(functionId);
                      } else if (count > 100) {
                        //just give up, been long enough i think
                        clearInterval(functionId);
                      }
                      count++;
                    }, 100);
                  }
                }
              });

              panel.doLayout();
          }
      };

      me.viewDetails = function() {
        if (me.detailPanel && me.data && me.data.resourceURI) {
          me.rebuildIframe(me.detailPanel, me.getFileUrl());
        }
      };

      me.allowView = function() {
          if (!Sonatype.HealthCheckProUtil.hasAlertReadPerm() || !Sonatype.HealthCheckProUtil.hasPaidProLicense()) {
              return false;
          }

          return true;
      };

      me.showListener = function() {
          me.shown = true;

          if (!me.dataDisplayed && me.data) {
              me.dataDisplayed = true;
              me.viewDetails();
          }
      };

      me.hideListener = function() {
          me.shown = false;
      };

      Sonatype.Events.addListener('fileContainerInit', function(items) {
          if (me.allowView()) {
              me.filePanel = me.buildDetailPanel({
                  name : 'healthCheckFileDetailPanel',
                  tabTitle : 'Component Info',
                  preferredIndex : 80,
                  listeners : {
                      show : me.showListener,
                      hide : me.hideListener
                  }
              });
              items.push(me.filePanel);
          }
      });

      Sonatype.Events.addListener('fileContainerUpdate', function(artifactContainer, data) {
          if (me.allowView()) {
              me.detailPanel = me.filePanel;
              me.data = data;
              me.dataDisplayed = false;

              if (me.shown) {
                  me.dataDisplayed = true;
                  me.viewDetails();
              }
          }
      });

      Sonatype.Events.addListener('artifactContainerInit', function(items) {
          if (me.allowView()) {
              me.artifactPanel = me.buildDetailPanel({
                  name : 'healthCheckArtifactDetailPanel',
                  tabTitle : 'Component Info',
                  preferredIndex : 80,
                  listeners : {
                      show : me.showListener,
                      hide : me.hideListener
                  }
              });
              items.push(me.artifactPanel);
          }
      });

      Sonatype.Events.addListener('artifactContainerUpdate', function(artifactContainer, data) {
          if (me.allowView()) {
              me.detailPanel = me.artifactPanel;
              me.data = data;
              me.dataDisplayed = false;

              if (me.shown) {
                  me.dataDisplayed = true;
                  me.viewDetails();
              }
          }
      });

      Sonatype.Events.addListener('historyChanged', function(token) {
        if (token) {
          var toks = token.split(Sonatype.view.HISTORY_DELIMITER);
          if (toks[0] == 'compDetail') {
            if (Sonatype.user.curr.isLoggedIn) {
              if (me.allowView()) {
                var params = me.fromTabId(toks);
                me.showDetailsTab(params.appId, params.hash, params.component, params.legacy);
              }
            } else {
              Sonatype.view.afterLoginToken = token;
              Sonatype.repoServer.RepoServer.loginHandler();
            }
            // was handled by this event listener so return false to stop others
            return false;
          }
        }
      });
    };

  Ext.onReady(function() {
      Sonatype.HealthCheckArtifactDetailsContrib = new HealthCheckArtifactDetailsContrib();
  });
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Nexus.clm.SettingsView', {
    extend: 'Ext.Panel',

    requires: [
      'Nexus.staging.repository.activity.EventDetail',
      'Nexus.staging.repository.SummaryView',
      'Nexus.staging.repository.RepositoriesMediator'
    ],

    requirejs: [
      'Nexus/siesta',
      'Nexus/clm'
    ],

    mixins: [
      'Nexus.LogAwareMixin'
    ],

    constructor: function (config) {
        var self = this;

        self.url = new Ext.form.TextField({
            name: 'url',
            fieldLabel: 'IQ Server URL',
            helpText: 'The address of your IQ Server',
            anchor : Sonatype.view.FIELD_OFFSET,
            allowBlank : false,
            itemCls : 'required-field',
            validator : function(v) {
              if (v === '' || v.match(/^(?:http|https):\/\//i))
              {
                return true;
              }
              else
              {
                return 'Protocol must be http:// or https://';
              }
            }
        });

        self.testConnectionStore = new Ext.data.JsonStore({
          id: 'clmTestConnection',
          fields: [{
            name: 'name',
            sortType: Ext.data.SortTypes.asUCString
          }],
          sortInfo: {
            field: 'name',
            direction: 'ASC'
          },
          data: []
        });

        self.testConnectionGrid = new Ext.grid.GridPanel({
          store: self.testConnectionStore,
          hidden: true,
          title: 'Available Applications',
          anchor: Sonatype.view.FIELD_OFFSET,
          height: 300,
          tools: [{
            type: 'close',
            handler: function(event, toolEl, panel) {
              panel.hide();
            }
          }],
          viewConfig: {
            autoFill: true,
            forceFit: true
          },
          colModel: new Ext.grid.ColumnModel({
            defaults: {
              sortable: true
            },
            columns: [{
              header: 'Application',
              dataIndex: 'name'
            }]
          })
        });
        
        self.settingsForm = new Ext.FormPanel({
          border : false,
          labelWidth : 100,
          layoutConfig : {
            labelSeparator : ''
          },
            items: [
                self.url,
                {
                  xtype: 'textfield',
                  name: 'username',
                  fieldLabel: 'Username',
                  helpText: 'User with access to the IQ Server.',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : false,
                  itemCls : 'required-field'
                },
                {
                  xtype: 'textfield',
                  name: 'password',
                  inputType: 'password',
                  fieldLabel: 'Password',
                  helpText: 'Credentials for the IQ Server user.',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : false,
                  itemCls : 'required-field'
                },
                {
                  xtype: 'numberfield',
                  name: 'timeout',
                  fieldLabel: 'Request Timeout',
                  helpText: 'Number of seconds to wait for a successful IQ Server connection before retrying. Defaults to 15 minutes.',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : true
                },
              {
                xtype: 'textarea',
                fieldLabel: 'Properties',
                name: 'properties',
                anchor : Sonatype.view.FIELD_OFFSET,
                helpText: 'Additional properties to configure for the IQ Server',
                flex: 1
              }
            ],
            buttons : [
              {
                text : 'Save',
                tooltip : 'Save settings',
                formBind : true,
                scope : self,
                handler : self.saveSettings
              },
              { text : 'Reset',
                tooltip : 'Reset changes',
                formBind : true,
                scope : this,
                handler : self.loadSettings
              },
              { text : 'Test Connection',
                tooltip : 'Test connection and permissions.',
                formBind : true,
                scope : self,
                handler : self.testConnection
              }
            ]
        });

      self.disabledWarning = NX.create('Ext.Panel', {
        border : false,
        html : '<div class="nx-clm-disabled-warning">'
            +     '<div>'
            +       'IQ Server functionality is currently disabled. To enable it, use the Capabilities UI.'
            +     '</div>'
            + '</div>',
        hidden: true
      });

      Ext.apply(self, {
          cls: 'nx-clm-SettingsView',
          border: false,
          layout: 'vbox',
          layoutConfig: {
              align: 'stretch'
          },
          items: [
              {
                  xtype: 'fieldset',
                  title: 'Settings',
                  anchor : Sonatype.view.FIELDSET_OFFSET,
                  autoHeight : true,
                  items: [
                    {
                      border : false,
                      html : '<div class="nx-clm-doco">'
                            +   '<a href="http://links.sonatype.com/products/clm/nexus/home"  target="_blank">'
                            +   'Make Nexus Staging even more powerful with Nexus IQ Server'
                            +   ' </a>'
                            + '</div>'
                    },
                    self.disabledWarning,
                    self.settingsForm,
                    self.testConnectionGrid
                  ]
              }
          ]
      });

      self.constructor.superclass.constructor.apply(self, arguments);

      // Load settings after layout, has to be done post superclass.constructor()
      self.on('afterlayout', self.loadSettings, self, {
        single: true
      });

    },

    saveSettings : function() {
      var self = this,
          form, url;

      self.logDebug('IQ Server saving settings');

      form = self.settingsForm.getForm();

      if(!form.isValid())
      {
        return;
      }

      url = form.findField('url').getValue();

      Ext.Ajax.request({
        url : Nexus.clm.resourcePath,
        method : 'POST',
        scope : self,
        suppressStatus: 400,
        jsonData : form.getFieldValues(),

        success : function() {
          var nexusPanel = Sonatype.repoServer.RepoServer.nexusPanel;
          Nexus.clm.showMessage('Configuration saved');
          form.items.each(function(item){
            item.clearInvalid();
          });
          nexusPanel.get('st-enterprise-staging').remove(nexusPanel.clmMenu);
          Nexus.clm.serverUrl = url;
          nexusPanel.clmMenu = nexusPanel.add({
            enabled: true,
            sectionId: 'st-enterprise-staging',
            title: Nexus.clm.TITLE_DASHBOARD,
            href: Nexus.clm.serverUrl
          });
          nexusPanel.doLayout();
        },

        failure: function(response, opts){
          if(response.siestaValidationError !== undefined)
          {
            Ext.each(response.siestaValidationError, function (error){
              form.findField(error.id).markInvalid(error.message);
            });
          }
        }
      });
    },

    loadSettings : function() {
      var self = this;

      self.logDebug('IQ Server loading settings');

      Ext.Ajax.request({
        url :  Nexus.clm.resourcePath,
        method : 'GET',
        scope : self,
        suppressStatus: 404,
        success : function(response) {
          var config = Ext.decode(response.responseText);
          self.settingsForm.getForm().setValues(config);
          if(config.enabled){
            self.disabledWarning.hide();
          }
          else {
            self.disabledWarning.show();
          }
          self.logDebug('IQ Server settings loaded');
        }
      });
    },

    testConnection: function() {
      var self = this, form, url;

      self.logDebug('Testing IQ Server connection.');

      form = self.settingsForm.getForm();

      if (!form.isValid()) {
        return;
      }

      Ext.Ajax.request({
        url: Nexus.clm.testConnectionPath,
        method: 'POST',
        scope: self,
        suppressStatus: 400,
        jsonData: form.getFieldValues(),
        success: function(response) {
          self.testConnectionGrid.show();
          self.testConnectionStore.loadData(JSON.parse(response.responseText));
        },
        failure: function(response, opts) {
          self.testConnectionGrid.hide();
          if (response.siestaValidationError !== undefined) {
            Ext.each(response.siestaValidationError, function(error) {
              form.findField(error.id).markInvalid(error.message);
            });
          }
        }
      });
    }
},

function () {
    var type = this;
    var key = Nexus.clm.STAGING_KEY;
    var label = Nexus.clm.STAGING_LABEL;

    // Attach SettingsView to navigation panel if current user has appropriate permissions
    Sonatype.Events.addListener('nexusNavigationInit', function (nexusPanel) {
      var p = Sonatype.lib.Permissions;
      if (p.checkPermission('nexus:settings', p.READ)) {
        nexusPanel.add({
          enabled: true,
          sectionId: 'st-nexus-config',
          title: Nexus.clm.TITLE,
          tabId: Nexus.clm.TAB_ID,
          tabTitle: Nexus.clm.TAB_TITLE,
          tabCode: type
        });
        Ext.Ajax.request({
          url :  Nexus.clm.resourcePath,
          method : 'GET',
          scope : self,
          suppressStatus: 404,
          success : function(response) {
            var clmSettings = Ext.decode(response.responseText);
            if (!Ext.isEmpty(clmSettings.url)) {
              Nexus.clm.serverUrl = clmSettings.url;
              nexusPanel.clmMenu = nexusPanel.add({
                enabled: true,
                sectionId: 'st-enterprise-staging',
                title: Nexus.clm.TITLE_DASHBOARD,
                href: Nexus.clm.serverUrl
              });
              nexusPanel.doLayout();
            }
          }
        });
      }
    });

  //Enhance StagingProfile UI model with IQ Server metadata. Works in conjunction with the staging.profile.submitted event.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.profile.loaded', function(items, properties, dataRoot) {
    var clmAppPolicyURL;
    NX.log.debug("Enhancing UI with IQ Server details.");
    clmAppPolicyURL = new Ext.form.DisplayField({
      anchor: Sonatype.view.FIELD_OFFSET,
      allowBlank: true,
      hidden: !Ext.isDefined(Nexus.clm.serverUrl),
      appId: properties[key],
      showClmAppUrl: function () {
        var appId = clmAppPolicyURL.appId,
            url = Nexus.clm.serverUrl,
            text = 'IQ Server Dashboard';

        if (!Ext.isEmpty(appId)) {
          while (url.endsWith('/')) {
            url = url.substring(0, url.length - 1);
          }
          url += '/ui/links/application/' + encodeURIComponent(appId) + '/management';
          text = 'IQ Server Application Management';
        }
        clmAppPolicyURL.setValue('<a href="' + url + '" target="_blank">' + text + '</a>');
      }
    });
    clmAppPolicyURL.showClmAppUrl();
    items.push([
      new Ext.form.ComboBox({
        fieldLabel: label,
        name: dataRoot + '.' + key,
        value: properties[key],
        width: 376,
        helpText: 'Select an IQ Server Application to enable, or (none) to disable policy enforcement for this profile.<br/>'
            + '<a href="http://links.sonatype.com/products/clm/nexus/home"  target="_blank">'
            + 'Make Nexus Staging even more powerful with Nexus IQ Server'
            + ' </a>',
        persistent: true,
        store: new Ext.data.JsonStore({
          id: 'id',
          fields: [
            { name: 'id'},
            { name: 'name', sortType: Ext.data.SortTypes.asUCString }
          ],
          sortInfo: {
            field: 'name',
            direction: 'ASC'
          },
          url: Nexus.clm.resourcePath + "/applications",
          autoLoad: true,
          listeners: {
            load: function (store) {
              store.insert(0, NX.create('Ext.data.Record', {id: '', name: '(none)'}));
            }
          }
        }),
        displayField: 'name',
        valueField: 'id',
        editable: false,
        forceSelection: true,
        mode: 'local',
        triggerAction: 'all',
        emptyText: 'Select...',
        selectOnFocus: true,
        allowBlank: true,
        listeners: {
          select: function (combo, record) {
            clmAppPolicyURL.appId = record.get('id');
            clmAppPolicyURL.showClmAppUrl();
          }
        }
      }),
      clmAppPolicyURL
    ]);
  });

  //Enhance StagingProfile UI model to serialize IQ Server metadata. Works in conjunction with the staging.profile.loaded event.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.profile.submitted', function(referenceData) {
    NX.log.debug("Enhancing StagingProfile submit data with IQ Server details.");
    referenceData[key] = null;
  });

  /**
   * @param value The encoded value with details of last IQ Server result.
   * @param showDetailedReport Whether or not a link to the full IQ Server results should be included
   * @return {*} view of the last IQ Server result.
   */
  function renderClmValues(value, showDetailedReport) {
    var values = value.split(',');
    NX.assert(values.length == 6, "Should have 6 comma separated values stored for IQ Server violations.");
    return Nexus.clm.chickletTemplate.apply({
      criticalCount : values[2],
      severeCount : values[3],
      moderateCount : values[4],
      detailedReport : showDetailedReport ? values[5] : null
    });
  }

  // enhance StagingRepository grid view with an additional column
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.repositoriesGrid.configureColumns', function(columns) {
    NX.log.debug("Enhancing Repository grid view with IQ Server details.");
    columns.push({
      id : 'violations',
      width : 250,
      header : 'IQ Policy Violations',
      renderer : function(value, metaData, record) {
        var properties = record.data.properties;
        if (properties !== undefined) {
          var clmProperty;
          Ext.each(properties, function(property) {
            if (property.name == "IQ Policy Violations") {
              clmProperty = property.value;
              return false;
            }
          });
          if(clmProperty !== undefined)
          {
            return renderClmValues(clmProperty, false);
          }
        }
        return 'No IQ Server data recorded';
      }
    });
  });

  //Register renderer for IQ Server summary in StagingRepository summary view.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.repositorySummary.configureProperties', function(properties, values){
    Ext.each(values.properties, function(property){
      if(property.name && property.value && property.name === 'IQ Policy Violations')
      {
        NX.log.debug("Enhancing Repository summary view with IQ Server details.");
        for(var i = 0; i <  properties.length; i++)
        {
          if(properties[i].name === 'Activity')
          {
            break;
          }
        }
        properties.splice(i+1, 0, { name: 'IQ Policy Violations', value: renderClmValues(property.value, true) });
        return false;
      }
    });
  });

  //Register renderers for IQ Server rules in the Staging UI
  Nexus.staging.repository.activity.EventDetail.registerBodyTemplateFactory('ruleFailed', Nexus.clm.stagingRule);
  Nexus.staging.repository.activity.EventDetail.registerBodyTemplateFactory('rulePassed', Nexus.clm.stagingRule);
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Nexus.clm.ComponentDetailView', {
  extend: 'Ext.Panel',
  requirejs: ['Nexus/clm', 'Nexus/healthcheck/HealthCheckUtil'],
  initComponent : function() {
    var self = this;

    var encodeQueryParameters = function() {
      var result = '?appId=' + encodeURIComponent(self.appId);
      if (self.hash) {
        result += '&hash=' + encodeURIComponent(self.hash);
      }
      if (self.component) {
        result += '&componentIdentifier=' + encodeURIComponent(JSON.stringify(self.component));
      } else if (self.gav) {
        result += '&groupId=' + encodeURIComponent(self.gav.groupId);
        result += '&artifactId=' + encodeURIComponent(self.gav.artifactId);
        result += '&version=' + encodeURIComponent(self.gav.version);
        if (self.gav.classifier) {
          result += '&classifier=' + encodeURIComponent(self.gav.classifier);
        }
        if (self.gav.extension) {
          result += '&extension=' + encodeURIComponent(self.gav.extension);
        }
      }
      return result;
    };
    
    Ext.apply(self, {
      cls: 'nx-clm-ComponentDetailView',
      border: false,
      layout: 'fit',
      items: [{
        xtype: 'iframepanel',
        loadMask: {
          msg: 'Loading Component Detail...'
        },
        fitToParent: true,
        defaultSrc: Nexus.clm.componentDetailContentResourcePath + '/viewdetails.html' + encodeQueryParameters()
      }]
    });
    
    self.constructor.superclass.initComponent.apply(self, arguments);
  }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/
define('nexus-iq-repository-plugin-boot', ['Nexus/clm/SettingsView', 'Nexus/clm/ComponentDetailView', 'Nexus/healthcheck/HealthCheckArtifactDetailsContrib',
    'Nexus/healthcheck/HealthCheckProRepositoryContrib', 'Nexus/firewall/FirewallRepositoryContrib'], function() {
  Nexus.Log.debug('IQ Server plugin loaded');
});

