/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Nexus, Sonatype, define*/
define('nexus-procurement-plugin-boot', [
  'Sonatype/repoServer/ArtifactProcurementPanel',
  'Sonatype/repoServer/ArtifactResolutionMenu',
  'Sonatype/repoServer/ArtifactProcurementTree'
], function() {
  // FIXME move config to proper location
  Ext.namespace('Nexus.procurement.configuration');
  Nexus.procurement.configuration.FREEFORM_GAV = 'specified by user';
  Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH = Sonatype.config.servicePath + '/procurement/resolutions/';
  Nexus.procurement.configuration.PROCUREMENT_SERVICE_PATH = Sonatype.config.servicePath + '/procurement/manage';

  Nexus.Log.debug('Initialized procurement UI');
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Sonatype.repoServer.ArtifactProcurementRulePickerDialog', {
  extend : 'Ext.Window',
  constructor : function(mask, node) {

    this.node = node;

    this.gavMask = mask;

    var disableDelete = !Sonatype.lib.Permissions.checkPermission('nexus:procurementresolution',
          Sonatype.lib.Permissions.DELETE);
    if (node.isRoot) {
      if (this.gavMask == Nexus.procurement.configuration.FREEFORM_GAV) {
        this.title = " a Specified GAV";
      }
      else {
        this.title = "*:*:*";
      }
    }
    else if (node.attributes.type == 'G') {
      var groupId = node.attributes.path;
      this.title = groupId.substring(1, groupId.length - 1).replace(/\//g, '.');
    }
    else if (node.attributes.type == 'A') {
      this.title = node.attributes.groupId + ':' + node.attributes.artifactId;
    }
    else if (node.attributes.type == 'V' || node.attributes.type == 'artifact') {
      this.title = node.attributes.groupId + ':' + node.attributes.artifactId + ':' + node.attributes.version;
    }

    this.title = 'Add Rule for ' + this.title;

    this.treeRoot = new Ext.tree.TreeNode({
      text : 'root'
    });
    this.deletedNodes = [];

    this.ruleTypeDataStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : ['id', 'name', 'description', 'properties'],
      url : Sonatype.config.servicePath + '/procurement/rules',
      autoLoad : false
    });

    Sonatype.repoServer.ArtifactProcurementRulePickerDialog.superclass.constructor.call(this, {
      closable : true,
      autoWidth : false,
      width : 525,
      height : 440,
      modal : true,
      constrain : true,
      resizable : true,
      items : [
        {
          xtype : 'form',
          layoutConfig : {
            labelSeparator : ''
          },
          labelWidth : 100,
          frame : true,
          defaultType : 'textfield',
          monitorValid : true,
          region : 'center',
          items : [
            {
              xtype : 'combo',
              fieldLabel : 'Available Rules',
              itemCls : 'required-field',
              helpText : 'Procurement rules that can be applied at this level.',
              name : 'rulesId',
              id : 'rulesCombo',
              store : this.ruleTypeDataStore,
              width : 350,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              listeners : {
                select : {
                  fn : this.onRuleSelect,
                  scope : this
                }
              }
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Description',
              name : 'ruleDescription',
              id : 'ruleDescriptionField',
              width : 350,
              readOnly : true
            },
            {
              xtype : 'panel',
              id : 'gavFieldPanel',
              layout : 'form',
              deferredRender : false,
              autoScroll : false,
              frame : false,
              width : 520,
              autoHeight : true,
              items : [
                {
                  xtype : 'textfield',
                  name : 'freeformGAV',
                  fieldLabel : 'Enter a GAV',
                  id : 'freeformGAVField',
                  width : 350,
                  helpText : "Enter a GAV (Group, Artifact, Version) to apply to this rule. The specified GAV should have the form \'Group:Artifact:Version\'. For example, enter org.apache.ant:ant:* for all versions of the ant artifacts.",
                  readOnly : false
                }
              ]
            }
          ]
        },
        {
          xtype : 'panel',
          id : 'rule-config-forms',
          title : 'Rule Properties',
          layout : 'card',
          region : 'center',
          activeItem : 0,
          deferredRender : false,
          autoScroll : false,
          height : 220,
          frame : false,
          items : [
            {}
          ]
        },
        {
          xtype : 'panel',
          id : 'rule-existing-value-card-panel',
          title : 'Existing Rule(s)',
          bodyStyle : 'padding:5px',
          deferredRender : false,
          autoScroll : false,
          height : 100,
          frame : false,
          items : [
            {
              xtype : 'label',
              fieldLabel : 'Value',
              id : 'existing-rule-value-label',
              text : 'none'
            }
          ]
        }
      ],
      buttons : [
        {
          text : 'Save',
          formBind : true,
          disabled : disableDelete,
          handler : this.addNewRule,
          scope : this
        },
        {
          text : 'Cancel',
          formBind : false,
          handler : function(button, e) {
            this.close();
          },
          scope : this
        }
      ]
    });
    this.formCards = this.findById('rule-config-forms');
    var startConfig = {
      xtype : 'panel',
      layout : 'form',
      id : 'rule-properties-card-panel',
      header : false,
      bodyStyle : 'padding:15px',
      deferredRender : false,
      autoScroll : false,
      height : 150,
      frame : false,
      items : [
        {}
      ]
    }
    var formPanel = new Ext.FormPanel(startConfig);
    this.formCards.add(formPanel);

    formPanel.doLayout();
    this.loadAllRules();
  },

  getValueForProp : function(id) {
    var field = this.findById('ruleProperties_' + id);
    return field.getValue() + "";
  },

  getMissingPropMsg : function() {
    var rule = this.getSelectedRule();
    var props = rule.data.properties;
    if (props != null && props.length > 0) {
      for (var i = 0; i < props.length; i++) {
        if (props[i].required) {
          var propValue = this.getValueForProp(props[i].id);
          if (propValue == null || propValue.length == 0) {
            return "A value is required for the property '" + props[i].name + "'";
          }
        }
      }
    }
    return "";
  },

  addNewRule : function() {
    var repoRec = this.node.ownerTree.payload;
    var repoName = repoRec.data.name;
    var gav = "";
    if (this.gavMask == Nexus.procurement.configuration.FREEFORM_GAV) {
      var gavValue = this.findById('freeformGAVField').getValue();

      gav = gavValue.split(':');
      if (gav.length != 3) {
        Ext.Msg.alert('Error',
              "The specified GAV should have the form \'Group:Artifact:Version\'.  For example, enter org.apache.ant:ant:* for all versions of the ant artifacts.");
        return;
      }
    }
    else {
      gav = this.gavMask.split(':');
    }
    var selectedRuleId = this.getSelectedRuleId();
    var missingPropMsg = this.getMissingPropMsg();
    if (missingPropMsg != null && missingPropMsg.length > 0) {
      Ext.Msg.alert('Missing Property', missingPropMsg);
      return;
    }
    var propz = [];

    var rule = this.getSelectedRule();
    var props = rule.data.properties;
    if (props != null && props.length > 0) {
      for (var i = 0; i < props.length; i++) {
        var propValue = this.getValueForProp(props[i].id);
        if (propValue) {
          var newElem = {
            key : props[i].id,
            value : propValue
          }
          propz.push(newElem);
        }
      }
    }
    Ext.Ajax.request({
      method : 'POST',
      jsonData : {
        data : {
          artifactCoordinate : {
            groupId : gav[0],
            artifactId : gav[1],
            version : gav[2]
          },
          properties : propz,
          ruleTypeId : selectedRuleId
        }
      },
      url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id,
      scope : this,
      callback : function(options, success, response) {
        if (success) {
          this.close();
          this.node.ownerTree.refreshNode(this.node);
        }
        else {
          Sonatype.utils.connectionError(response, 'Adding the new rule failed in ' + repoName + '.');
        }
      }
    });

  },

  updateExistingRuleText : function(txt) {
    var existingRuleLabel = this.findById('existing-rule-value-label');
    if (existingRuleLabel) {
      Ext.get(existingRuleLabel.getEl()).update(txt);
    }
  },

  getExistingRule : function() {
    this.updateExistingRuleText('Finding existing rule...');
    if (!this.node) {
      return;
    }
    var repoRec = this.node.ownerTree.payload;
    var repoName = repoRec.data.name;
    var appliedIds = this.node.attributes.appliedArtifactResolutionIds;
    if (appliedIds == null || appliedIds.length == 0) {
      this.updateExistingRuleText("There are no rules applied.");
      return;
    }
    var ids = "";
    var count = 0;
    for (var i = 0; i < appliedIds.length; i++) {
      if (appliedIds[i]) {
        for (var j = 0; j < appliedIds[i].ruleIds.length; j++) {
          if (appliedIds[i].ruleIds[j]) {
            if (count > 0) {
              ids += ',';
            }
            ids += appliedIds[i].ruleIds[j];
            count++;
          }
        }
      }
    }
    if (repoRec && repoRec.data) {
      Ext.Ajax.request({
        method : 'GET',
        url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id + '?ids=' + ids,
        scope : this,
        callback : function(options, success, response) {
          if (success) {
            var result = Ext.util.JSON.decode(response.responseText);
            var data = result.data;
            var existingRulesText = "";
            if (data) {
              for (var i = 0; i < data.length; i++) {
                if (i > 0) {
                  existingRulesText += ", ";
                }
                var ac = data[i].artifactCoordinate;
                var ruleText = this.getExistingRuleText(data[i]);
                existingRulesText += ruleText;

              }
            }
            this.updateExistingRuleText(existingRulesText);
          }
          else {
            Sonatype.utils.connectionError(response, 'Failed to find rules ' + repoName + '.');
            this.updateExistingRuleText('Failed to find rule...');
          }
        }
      });
    }
  },

  getExistingRuleText : function(rule) {
    var ruleText = rule.ruleTypeId;
    if (rule.ruleTypeId == "simple" && rule.properties && rule.properties.length == 1) {
      var key = rule.properties[0].key;
      var value = rule.properties[0].value;
      if (value == "true") {
        ruleText = "Approve for";
      }
      else {
        ruleText = "Deny for"
      }
    }
    else if (rule.ruleTypeId == "signature" && rule.properties && rule.properties.length == 1) {
      var key = rule.properties[0].key;
      var value = rule.properties[0].value;
      if (value == "true") {
        ruleText = "Signature must be present for";
      }
      else {
        ruleText = "Signature can be present/empty for";
      }
    }
    else {
      ruleText = rule.ruleTypeId
    }
    return (ruleText + " " + rule.artifactCoordinate.groupId + ":" + rule.artifactCoordinate.artifactId + ":"
          + rule.artifactCoordinate.version);
  },

  ruleTypeStoreLoadHandler : function() {},

  showFreeformField : function() {
    if (this.gavMask != Nexus.procurement.configuration.FREEFORM_GAV) {
      var freeFormGAVText = this.findById('freeformGAVField');
      if (freeFormGAVText) {
        freeFormGAVText.hideParent = true;
        freeFormGAVText.hidden = true;
        freeFormGAVText.hideLabel = true;
      }
    }
  },

  loadAllRules : function() {
    this.showFreeformField();
    this.show();
    this.ruleTypeDataStore.reload();
    this.getExistingRule();
  },

  populateRulePanel : function(rec) {
    var allItems = [];
    var strValue = 'string';
    var numValue = 'number';
    var boolValue = 'boolean';
    if (rec.data.properties == null || rec.data.properties.length == 0) {
      allItems[0] = {
        xtype : 'label',
        text : 'There are no properties for this rule.',
        style : 'text-align:center;',
        name : 'no-props-lbl',
        width : 500
      };
      return allItems;
    }
    for (var j = 0; j < rec.data.properties.length; j++) {
      var curRec = rec.data.properties[j];
      if (curRec.type == strValue || curRec.type.toLowerCase() == strValue) {
        allItems[j] = {
          xtype : 'textfield',
          fieldLabel : curRec.name,
          itemCls : curRec.required ? 'required-field' : '',
          helpText : curRec.helpText,
          id : 'ruleProperties_' + curRec.id,
          allowBlank : curRec.required ? false : true,
          labelStyle : 'width:150px;',
          width : 300
        };
      }
      else if (curRec.type == numValue || curRec.type.toLowerCase() == numValue) {
        allItems[j] = {
          xtype : 'numberfield',
          fieldLabel : curRec.name,
          itemCls : curRec.required ? 'required-field' : '',
          helpText : curRec.helpText,
          name : 'ruleProperties_' + curRec.id,
          id : 'ruleProperties_' + curRec.id,
          allowBlank : curRec.required ? false : true,
          labelStyle : 'width:150px;',
          width : 300
        };
      }
      else if (curRec.type == boolValue || curRec.type.toLowerCase() == boolValue) {
        allItems[j] = {
          xtype : 'checkbox',
          fieldLabel : curRec.name,
          helpText : curRec.helpText,
          id : 'ruleProperties_' + curRec.id,
          width : 300,
          labelStyle : 'width:150px;',
          name : 'ruleProperties_' + curRec.id
        };
      }
    }

    return allItems;
  },

  getSelectedRuleId : function() {
    var combo = this.findById('rulesCombo');
    return combo.getValue() + "";
  },

  getSelectedRule : function() {
    var selectedRuleId = this.getSelectedRuleId();
    var rule = this.ruleTypeDataStore.getAt(this.ruleTypeDataStore.findBy(function(record, id) {
      if (record.data.id == selectedRuleId) {
        return true;
      }
      return false;
    }));
    return rule;
  },

  updateDescription : function() {
    var rule = this.getSelectedRule();
    if (rule && rule.data.description) {
      var descField = this.findById('ruleDescriptionField');
      descField.setValue(rule.data.description);
    }
  },

  onRuleSelect : function(combo, rec, index) {
    var id = 'rule-properties-card-panel';
    var newItems = this.populateRulePanel(rec);
    var config = {
      xtype : 'panel',
      id : 'rule-properties-card-panel',
      header : false,
      bodyStyle : 'padding:15px',
      deferredRender : false,
      autoScroll : false,
      frame : false,
      height : 150,
      items : [
        {}
      ]
    }
    config.items = newItems;
    var formPanel = new Ext.FormPanel(config);
    this.formCards.add(formPanel);
    var layout = this.formCards.getLayout();
    this.formCards.getLayout().setActiveItem(formPanel);
    this.updateDescription();
    formPanel.doLayout();

  },

  updateExistingRule : function(rec) {
    var valueLabel = this.findById("existing-rule-value-label");
    Ext.get(valueLabel.getEl()).update(rec.name);
  }
}, function() {
  Sonatype.Events.addListener('procurementToolbarInit', function(treepanel, toolbar) {
    toolbar.push(new Ext.Button({
      text : 'Add Freeform Rule',
      icon : Sonatype.config.resourcePath + '/static/images/icons/add.png',
      cls : 'x-btn-text-icon',
      handler : function() {
        new Sonatype.repoServer.ArtifactProcurementRulePickerDialog(Nexus.procurement.configuration.FREEFORM_GAV,
              treepanel.getRootNode());
      },
      scope : treepanel
    }));
  });
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Sonatype.repoServer.ArtifactProcurementStartPopup', {
  extend : 'Ext.Window',
  constructor : function(config) {
    var config = config || {};
    var defaultConfig = {
      title : 'Start Procurement'
    };
    Ext.apply(this, config, defaultConfig);

    this.hostedRepoStore = new Ext.data.SimpleStore({
      fields : ['id', 'name'],
      data : [],
      sortInfo : {
        field : 'name',
        direction : 'ASC'
      }
    });
    this.targetRepoStore = new Ext.data.SimpleStore({
      fields : ['id', 'name', 'displayName'],
      data : [],
      sortInfo : {
        field : 'name',
        direction : 'ASC'
      }
    });

    this.groupStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : [
        {
          name : 'id'
        },
        {
          name : 'repoType'
        },
        {
          name : 'format'
        },
        {
          name : 'name',
          sortType : Ext.data.SortTypes.asUCString
        },
        {
          name : 'displayName',
          mapping : 'name',
          convert : function(s, parent) {
            return '<b>' + s + '</b>';
          }
        }
      ],
      sortInfo : {
        field : 'name',
        direction : 'asc'
      },
      url : Sonatype.config.repos.urls.groups,
      autoLoad : false,
      listeners : {
        load : {
          fn : function(store, records, options) {
            this.repoStore.insert(0, records);
            this.show();
          },
          scope : this
        },
        // need to handle this in case there are no groups
        loadexception : {
          fn : function() {
            this.show();
          },
          scope : this
        }
      }
    });

    this.repoStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : [
        {
          name : 'id'
        },
        {
          name : 'repoType'
        },
        {
          name : 'repoPolicy'
        },
        {
          name : 'format'
        },
        {
          name : 'name',
          sortType : Ext.data.SortTypes.asUCString
        },
        {
          name : 'displayName',
          mapping : 'name'
        }
      ],
      sortInfo : {
        field : 'name',
        direction : 'asc'
      },
      url : Sonatype.config.repos.urls.repositories,
      autoLoad : true,
      listeners : {
        load : {
          fn : function(store, records, options) {
            var hostedRepos = [];

            for (var i = 0; i < records.length; i++) {
              var rec = records[i];
              if (rec.data.repoType == 'hosted' && (rec.data.format == 'maven2' || rec.data.format == 'maven1')) {
                hostedRepos.push([rec.data.id, rec.data.name]);
              }
            }

            this.hostedRepoStore.loadData(hostedRepos);

            this.groupStore.load();
          },
          scope : this
        },
        loadexception : {
          fn : function() {
            this.groupStore.load();
          },
          scope : this
        }
      }
    });

    Sonatype.repoServer.ArtifactProcurementStartPopup.superclass.constructor.call(this, {
      closable : true,
      autoWidth : false,
      width : 400,
      autoHeight : true,
      modal : true,
      constrain : true,
      resizable : false,
      draggable : false,
      items : [
        {
          xtype : 'form',
          layoutConfig : {
            labelSeparator : ''
          },
          labelWidth : 100,
          frame : true,
          defaultType : 'textfield',
          monitorValid : true,
          items : [
            {
              xtype : 'combo',
              fieldLabel : 'Repository',
              itemCls : 'required-field',
              helpText : 'Repository to start procurement for.',
              name : 'repositoryId',
              anchor : Sonatype.view.FIELD_OFFSET,
              width : 200,
              store : this.hostedRepoStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              listeners : {
                select : {
                  fn : this.onRepositorySelect,
                  scope : this
                }
              }
            },
            {
              xtype : 'combo',
              fieldLabel : 'Source',
              itemCls : 'required-field',
              helpText : 'Repository or group to procure from.',
              name : 'targetId',
              anchor : Sonatype.view.FIELD_OFFSET,
              width : 200,
              store : this.targetRepoStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              tpl : '<tpl for="."><div class="x-combo-list-item">{displayName}</div></tpl>'
            }
          ],
          buttons : [
            {
              text : 'OK',
              formBind : true,
              handler : this.startProcurement,
              scope : this
            },
            {
              text : 'Cancel',
              formBind : false,
              handler : function(button, e) {
                this.close();
              },
              scope : this
            }
          ]
        }
      ],
      listeners : {
        show : {
          fn : function(p) {
            if (this.payload) {
              var repositoryCombo = p.find('name', 'repositoryId')[0];
              var payloadId = this.payload.data.id;
              repositoryCombo.setValue(payloadId);
              this.filterTargetList(payloadId);
              repositoryCombo.disable();
            }
          },
          scope : this
        }
      }
    });
  },

  onRepositorySelect : function(combo, rec, index) {
    this.filterTargetList(rec.data.id);
  },

  filterTargetList : function(repoId) {
    this.targetRepoStore.removeAll();
    var repoPolicy = this.repoStore.getById(repoId).data.repoPolicy;
    var targetRepos = [];
    var records = this.repoStore.getRange();

    for (var i = 0; i < records.length; i++) {
      var rec = records[i];
      if (rec.data.id != repoId && (rec.data.repoPolicy == null || rec.data.repoPolicy == repoPolicy)
            && (rec.data.format == 'maven2')) {
        targetRepos.push([rec.data.id, rec.data.name, rec.data.displayName]);
      }
    }

    this.targetRepoStore.loadData(targetRepos);
    this.find('name', 'targetId')[0].reset();
  },

  startProcurement : function(button, e) {
    var targetId = this.find('name', 'targetId')[0].getValue();
    var targetRec = this.repoStore.getById(targetId);

    var repositoryCombo = this.find('name', 'repositoryId')[0]
    var repositoryId = repositoryCombo.getValue();
    var repositoryName = repositoryCombo.getRawValue();

    Ext.Ajax.request({
      scope : this,
      method : 'POST',
      suppressStatus : 400,
      jsonData : {
        data : {
          repositoryId : repositoryId,
          targetClassId : targetRec.data.repoType ? 'repositories' : 'groups',
          targetId : targetId
        }
      },
      repositoryName : repositoryName,
      url : Nexus.procurement.configuration.PROCUREMENT_SERVICE_PATH,
      success : this.procurementSuccess,
      failure : this.procurementFailure,
      scope : this
    });
  },

  procurementSuccess : function(response, options) {
    this.close();
    Sonatype.MessageBox.show({
      title : 'Procurement Started',
      msg : 'Procurement is now started in ' + options.repositoryName + '.',
      buttons : Sonatype.MessageBox.OK,
      icon : Sonatype.MessageBox.INFO,
      animEl : 'mb3'
    });

    var procurementPanel = Sonatype.view.mainTabPanel.findById('artifact-procurement');
    if (procurementPanel) {
      procurementPanel.refreshHandler();
      Sonatype.Events.fireEvent('repositoryChanged');
    }
  },

  procurementFailure : function(response, options) {
    var r = Ext.decode(response.responseText);
    if (r.errors) {
      this.find('name', 'repositoryId')[0].markInvalid(r.errors[0].msg);
    }
    else {
      Sonatype.utils.connectionError(response,
            'There was a problem starting procurement in ' + options.repositoryName + '.');
    }
  }
}, function() {
  Sonatype.Events.addListener('procurementViewInit', function(cardPanel, rec) {
    if (rec.data.targetId != null && rec.data.targetId == '') {
      cardPanel.add({
        xtype : 'panel',
        name : 'configuration',
        items : [
          {
            border : false,
            html : '<div class="little-padding">Procurement is not started in this repository.</div>'
          },
          {
            cls : 'little-padding',
            xtype : 'button',
            text : 'Start Procurement...',
            hidden : !Sonatype.lib.Permissions.checkPermission('nexus:procurementrepos',
                  Sonatype.lib.Permissions.CREATE),
            handler : function(button, e) {
              new Sonatype.repoServer.ArtifactProcurementStartPopup({
                payload : rec
              });
            }
          }
        ]
      });
    }
  });

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Ext,define,Sonatype*/

NX.define('Sonatype.repoServer.ArtifactProcurementTree', {
  extend : 'Sonatype.panels.TreePanel',
  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {};
    Ext.apply(this, config, defaultConfig);

    Sonatype.repoServer.ArtifactProcurementTree.superclass.constructor.call(this, {
      toolbarInitEvent : 'procurementToolbarInit',
      nodeContextMenuEvent : 'procurementNodeMenuInit',
      url : Sonatype.config.servicePath + '/procurement/treeview/' + this.payload.data.id
            + Sonatype.config.browsePathSnippet
    });

    this.on('expandnode', this.expandNodeHandler, this);
  },
  expandNodeHandler : function(node) {
    this.updateNodeIcon(node);
  },

  refreshNode : function(node) {
    var path = node.getPath();
    while (node.attributes.type != 'G' && !node.isRoot) {
      node = node.parentNode;
    }
    node.attributes.children = null;
    node.reload(function() {
      node.ownerTree.expandPath(path);
    });
  },
  getIconName : function(resolution) {
    var rule = resolution.ruleAppliedState;
    var name = '';
    if (rule) {
      if (rule.toLowerCase() == 'no_rule') {
        name = 'undefined';
      }
      else if (rule.toLowerCase() == 'approved') {
        name = 'approved';
      }
      else if (rule.toLowerCase() == 'disapproved') {
        name = 'disapproved';
      }
      else {
        name = 'runtime';
      }
    }
    else {
      name = '';
    }
    return name;
  },
  updateNodeIcon : function(node) {
    if (node.attributes.ruleAppliedState) {
      var iconName = this.getIconName(node.attributes);
      node.ui.iconNode.className = 'x-tree-node-procurement-' + iconName + '-icon';
    }
    for (var j = 0; j < node.childNodes.length; j++) {
      this.updateNodeIcon(node.childNodes[j]);
    }
  }
}, function() {
  Sonatype.Events.addListener('procurementViewInit', function(cardPanel, rec) {
    if (rec.data.targetId != null && rec.data.targetId != '') {
      cardPanel.add(new Sonatype.repoServer.ArtifactProcurementTree({
        payload : rec
      }));
    }
  });
});





/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Nexus,Ext,define,Sonatype*/
NX.define('Sonatype.repoServer.ArtifactProcurementPanel', {
  extend : 'Sonatype.panels.GridViewer',
  requires : ['Sonatype.repoServer.ArtifactProcurementStartPopup'],
  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {
      title : 'Repositories'
    };
    Ext.apply(this, config, defaultConfig);

    this.groupStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : [
        {
          name : 'id'
        },
        {
          name : 'name'
        }
      ],
      url : Sonatype.config.repos.urls.groups,
      autoLoad : true,
      listeners : {
        load : {
          fn : this.groupStoreLoadHandler,
          scope : this
        },
        loadexception : {
          fn : this.groupStoreLoadHandler,
          scope : this
        }
      }
    });

    this.repositoryStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : [
        {
          name : 'id'
        },
        {
          name : 'name'
        },
        {
          name : 'repoPolicy'
        },
        {
          name : 'resourceURI'
        }
      ],
      url : Sonatype.config.repos.urls.allRepositories,
      autoLoad : false,
      listeners : {
        load : {
          fn : this.repositoryStoreLoadHandler,
          scope : this
        }
      }
    });

    Sonatype.repoServer.ArtifactProcurementPanel.superclass.constructor.call(this, {
      emptyText: 'No repositories defined',
      emptyTextWhileFiltering: 'No repositories matched criteria: {criteria}',

      rowClickEvent : 'procurementViewInit',
      rowContextClickEvent : 'repositoryMenuInit',
      url : Nexus.procurement.configuration.PROCUREMENT_SERVICE_PATH,
      dataAutoLoad : false,
      dataId : 'repositoryId',
      columns : [
        {
          name : 'repositoryId'
        },
        {
          name : 'resourceURI'
        },
        {
          name : 'targetClassId'
        },
        {
          name : 'targetId'
        },
        {
          name : 'id',
          mapping : 'repositoryId'
        },
        {
          name : 'name',
          sortType : Ext.data.SortTypes.asUCString,
          header : 'Repository',
          width : 175
        },
        {
          name : 'repoPolicy',
          header : 'Policy',
          width : 100
        },
        {
          name : 'procurementTargetName',
          header : 'Procured From',
          width : 150
        },
        {
          name : 'contentUri',
          header : 'Repository Path',
          autoExpand : true,
          renderer : function(s) {
            return '<a href="' + s + ((s != null && (s.charAt(s.length)) == '/') ? '' : '/') + '" target="_blank">'
                  + s + '</a>';
          }
        }
      ],
      tbar : [
        {
          text : 'Add Procured Repository',
          icon : Sonatype.config.resourcePath + '/static/images/icons/add.png',
          cls : 'x-btn-text-icon',
          scope : this,
          handler : this.addProcuredRepositoryHandler
        }
      ]
    });

    this.procurementStore = this.gridPanel.store;
    this.procurementStore.on('load', this.procurementStoreLoadHandler, this);
    this.procurementStore.on('loadexception', this.procurementStoreLoadHandler, this);
  },
  addProcuredRepositoryHandler : function(button, e) {
    new Sonatype.repoServer.ArtifactProcurementStartPopup();
  },

  groupStoreLoadHandler : function(store, records, options) {
    this.repositoryStore.reload();
  },

  procurementStoreLoadHandler : function(store, records, options) {
    if (records.length == undefined || records.length == 0) {
      this.dataStore.removeAll();
      return;
    }

    for (var i = records.length - 1; i >= 0; i--) {
      var rec = records[i];

      var groupTarget = rec.data.targetClassId == 'groups';
      var targetRec = (groupTarget ? this.groupStore : this.repositoryStore).getById(rec.data.targetId);
      var repositoryRec = this.repositoryStore.getById(rec.data.repositoryId);

      if (targetRec && repositoryRec) {
        rec.beginEdit();
        rec.set('name', repositoryRec.data.name);
        rec.set('repoPolicy', Sonatype.utils.upperFirstCharLowerRest(repositoryRec.data.repoPolicy));
        rec.set('procurementTargetName', targetRec.data.name);
        rec.set('resourceURI', repositoryRec.data.resourceURI);
        rec.set('contentUri', repositoryRec.data.resourceURI.replace(Sonatype.config.repos.urls.repositories,
              Sonatype.config.content.repositories));
        rec.commit();
        rec.endEdit();
      }
      else {
        store.remove(rec);
      }
    }
  },

  refreshHandler : function(button, e) {
    this.clearCards();
    this.groupStore.reload();
  },

  repositoryStoreLoadHandler : function(store, records, options) {
    this.procurementStore.reload();
  }
}, function() {
  Sonatype.Events.addListener('nexusNavigationInit', function(nexusPanel) {
    nexusPanel.add({
      enabled : Sonatype.lib.Permissions.checkPermission('nexus:procurementrepos', Sonatype.lib.Permissions.READ),
      sectionId : 'st-nexus-enterprise',
      title : 'Artifact Procurement',
      tabTitle : 'Procurement',
      tabId : 'artifact-procurement',
      tabCode : Sonatype.repoServer.ArtifactProcurementPanel
    });
  });

  Sonatype.Events.addListener('repositoryMenuInit', function(menu, rec) {
    if (rec.data.targetId != null && Sonatype.lib.Permissions.checkPermission('nexus:procurementrepos',
          Sonatype.lib.Permissions.DELETE)) {
      menu.add({
        text : 'Stop Procurement',
        handler : function(button, e) {
          Sonatype.MessageBox.show({
            animEl : Sonatype.view.mainTabPanel,
            title : 'Stop Procurement',
            msg : 'Stop procurement in ' + rec.data.name + '?',
            buttons : Sonatype.MessageBox.YESNO,
            scope : this,
            icon : Sonatype.MessageBox.QUESTION,
            fn : function(btnName) {
              if (btnName == 'yes' || btnName == 'ok') {
                Ext.Ajax.request({
                  method : 'DELETE',
                  url : Nexus.procurement.configuration.PROCUREMENT_SERVICE_PATH + '/' + rec.data.id,
                  callback : function(options, success, response) {
                    if (success) {
                      rec.store.remove(rec);
                      Sonatype.Events.fireEvent('repositoryChanged');

                      Sonatype.MessageBox.show({
                        title : 'Procurement Stopped',
                        msg : 'Procurement is now stopped in ' + rec.data.name + '.',
                        buttons : Sonatype.MessageBox.OK,
                        icon : Sonatype.MessageBox.INFO,
                        animEl : 'mb3'
                      });
                    }
                    else {
                      Sonatype.utils.connectionError(response,
                            'There was a problem stopping procurement in ' + rec.data.name + '.');
                    }
                  }
                });
              }
            }
          });
        }
      });
    }
  });

  Sonatype.repoServer.DefaultRepoHandler.repoActions.downloadRemoteItem.addCondition(
        function(repo) {
          return repo.targetId != null;
        }
  );
});



/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Sonatype.repoServer.ArtifactProcurementRulePopup', {
  extend : 'Ext.Window',
  constructor : function(node) {

    this.node = node;
    var disableDelete = !Sonatype.lib.Permissions.checkPermission('nexus:procurementresolution',
          Sonatype.lib.Permissions.DELETE);

    if (node.isRoot) {
      this.title = "everything (*:*:*)";
    }
    else if (node.attributes.type == 'G') {
      var groupId = node.attributes.path;
      this.title = groupId.substring(1, groupId.length - 1).replace(/\//g, '.');
    }
    else if (node.attributes.type == 'A') {
      this.title = node.attributes.groupId + ':' + node.attributes.artifactId;
    }
    else if (node.attributes.type == 'V' || node.attributes.type == 'artifact') {
      this.title = node.attributes.groupId + ':' + node.attributes.artifactId + ':' + node.attributes.version;
    }

    this.title = 'Rules for ' + this.title;

    this.treeRoot = new Ext.tree.TreeNode({
      text : 'root'
    });
    this.deletedNodes = [];

    Sonatype.repoServer.ArtifactProcurementRulePopup.superclass.constructor.call(this, {
      closable : true,
      autoWidth : false,
      width : 445,
      autoHeight : true,
      modal : true,
      constrain : true,
      resizable : false,
      items : [
        {
          xtype : 'form',
          frame : true,
          items : [
            {
              xtype : 'panel',
              layout : 'column',
              autoHeight : true,
              style : 'padding-left: 5px',
              items : [
                {
                  xtype : 'treepanel',
                  title : 'Applied Rules',
                  border : true,
                  bodyBorder : true,
                  bodyStyle : 'background-color: #FFFFFF; border: 1px solid #B5B8C8',
                  width : 300,
                  height : 300,
                  animate : true,
                  lines : false,
                  autoScroll : true,
                  containerScroll : true,
                  root : this.treeRoot,
                  rootVisible : false,
                  enableDD : false
                },
                {
                  xtype : 'panel',
                  width : 112,
                  items : [
                    {
                      xtype : 'button',
                      text : 'Remove',
                      style : 'padding-left: 6px',
                      minWidth : 105,
                      disabled : disableDelete,
                      handler : this.removeResolution,
                      scope : this
                    },
                    {
                      xtype : 'button',
                      text : 'Remove All',
                      style : 'padding-left: 6px; margin-top: 5px',
                      minWidth : 105,
                      disabled : disableDelete,
                      handler : this.removeAllResolutions,
                      scope : this
                    },
                    {
                      xtype : 'button',
                      text : 'Undo',
                      style : 'padding-left: 6px; margin-top: 5px',
                      minWidth : 105,
                      disabled : disableDelete,
                      handler : this.undoResolutions,
                      scope : this
                    }
                  ]
                }
              ]
            }
          ],
          buttons : [
            {
              text : 'Save',
              formBind : true,
              disabled : disableDelete,
              handler : this.saveResolutions,
              scope : this
            },
            {
              text : 'Cancel',
              formBind : false,
              handler : function(button, e) {
                this.close();
              },
              scope : this
            }
          ]
        }
      ]
    });

    this.loadResolutions();
  },

  loadResolutions : function() {
    var repoRec = this.node.ownerTree.payload;
    var appliedIds = this.node.attributes.appliedArtifactResolutionIds;
    var ids = "";
    var count = 0;
    if (appliedIds) {
      for (var i = 0; i < appliedIds.length; i++) {
        if (appliedIds[i]) {
          for (var j = 0; j < appliedIds[i].ruleIds.length; j++) {
            if (appliedIds[i].ruleIds[j]) {
              if (count > 0) {
                ids += ',';
              }
              ids += appliedIds[i].ruleIds[j];
              count++;
            }
          }
        }
      }
    }

    Ext.Ajax.request({
      method : 'GET',
      url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id,
      params : {
        ids : ids
      },
      success : this.loadResolutionsHandler,
      scope : this
    });
  },
  getAppliedIconName : function(resolution) {
    var rule = resolution.ruleTypeId;
    var name = '';
    if (rule) {
      if (rule.toLowerCase() == 'simple') {
        if (resolution.properties && resolution.properties.length == 1) {
          var key = resolution.properties[0].key;
          var value = resolution.properties[0].value;
          if (key == "isApproved" && value == "false") {
            name = 'disapproved';
          }
          else if (key == "isApproved" && value == "true") {
            name = 'approved'
          }
          else {
            name = 'runtime';
          }
        }
        else {
          name = 'runtime'
        }
      }
      else {
        name = 'runtime';
      }
    }
    else {
      name = 'undefined';
    }
    return name;
  },

  loadResolutionsHandler : function(response, options) {
    new Ext.tree.TreeSorter(this.treeRoot.ownerTree, {
      property : 'resolutionId'
    })

    var rec = Ext.decode(response.responseText);
    if (rec.data) {
      for (var i = 0; i < rec.data.length; i++) {
        var resolution = rec.data[i];
        this.treeRoot.appendChild(new Ext.tree.TreeNode({
          text : this.getAppliedRuleText(resolution),
          resolutionId : resolution.id,
          leaf : true,
          icon : Sonatype.config.resourcePath + '/static/images/'
                + (resolution.artifactCoordinate.artifactId == '*' ? 'folder-' : 'leaf-')
                + this.getAppliedIconName(resolution) + '.png'
        }));
      }
    }
    this.show();
  },

  getAppliedRuleText : function(rule) {
    var ruleText = rule.ruleTypeId;
    if (rule.ruleTypeId == "simple" && rule.properties && rule.properties.length == 1) {
      var key = rule.properties[0].key;
      var value = rule.properties[0].value;
      if (value == "true") {
        ruleText = "Approve for";
      }
      else {
        ruleText = "Deny for"
      }
    }
    else if (rule.ruleTypeId == "signature" && rule.properties && rule.properties.length == 1) {
      var key = rule.properties[0].key;
      var value = rule.properties[0].value;
      if (value == "true") {
        ruleText = "Signature must be present for";
      }
      else {
        ruleText = "Signature can be present/empty for";
      }
    }
    else {
      ruleText = rule.ruleTypeId
    }
    return (ruleText + " " + rule.artifactCoordinate.groupId + ":" + rule.artifactCoordinate.artifactId + ":"
          + rule.artifactCoordinate.version);
  },

  removeAllResolutions : function(button, e) {
    while (this.treeRoot.firstChild) {
      this.deletedNodes.push(this.treeRoot.firstChild);
      this.treeRoot.removeChild(this.treeRoot.firstChild);
    }
  },

  removeResolution : function(button, e) {
    var selectedNode = this.treeRoot.ownerTree.getSelectionModel().getSelectedNode();
    if (selectedNode) {
      this.deletedNodes.push(selectedNode);
      this.treeRoot.removeChild(selectedNode);
    }
  },

  saveResolutions : function(button, e) {
    if (this.deletedNodes.length) {
      var repoRec = this.node.ownerTree.payload;
      var repoName = repoRec.data.name;
      var ids = '';
      for (var i = 0; i < this.deletedNodes.length; i++) {
        if (i > 0) {
          ids += ',';
        }
        ids += this.deletedNodes[i].attributes.resolutionId;
      }

      Ext.Ajax.request({
        scope : this,
        method : 'DELETE',
        url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id + '?ids=' + ids,
        callback : function(options, success, response) {
          if (success) {
            this.close();
            this.node.ownerTree.refreshNode(this.node);
          }
          else {
            Sonatype.utils.connectionError(response, 'Artifact resolution update failed in ' + repoName + '.');
          }
        }
      });
    }
    else {
      this.close();
    }
  },

  undoResolutions : function(button, e) {
    for (var i = 0; i < this.deletedNodes.length; i++) {
      this.treeRoot.appendChild(new Ext.tree.TreeNode(this.deletedNodes[i].attributes));
    }
    this.deletedNodes = [];
  }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Sonatype.repoServer.ArtifactResolutionMenu', {
  requires : [
    'Sonatype.repoServer.ArtifactProcurementRulePickerDialog',
    'Sonatype.repoServer.ArtifactProcurementRulePopup'
  ],
  constructor : function(menu, node) {
    this.menu = menu;
    this.node = node;
    this.selectedLicense = "";

    if (Sonatype.lib.Permissions.checkPermission('nexus:procurementresolution', Sonatype.lib.Permissions.CREATE)) {

      this.approveIcon = Sonatype.config.resourcePath + '/static/images/approve.png';
      this.denyIcon = Sonatype.config.resourcePath + '/static/images/disapprove.png';

      this.approveActions = [];
      this.denyActions = [];
      if (node.isRoot) {
        this.addRootActions(node);
      }
      else if (node.attributes.type == 'G') {
        var groupId = node.attributes.path;
        this.groupId = groupId.substring(1, groupId.length - 1).replace(/\//g, '.');
        this.addGroupActions(node);
      }
      else if (node.attributes.type == 'A') {
        this.groupId = node.attributes.groupId;
        this.artifactId = node.attributes.artifactId;
        this.addArtifactActions(node);
      }
      else if (node.attributes.type == 'V' || node.attributes.type == 'artifact') {
        this.groupId = node.attributes.groupId;
        this.artifactId = node.attributes.artifactId;
        this.version = node.attributes.version;
        this.addVersionActions(node);
      }
    }

    if ((node.attributes.appliedArtifactResolutionIds && node.attributes.appliedArtifactResolutionIds.length)
          || node.isRoot) {
      menu.addSeparator();
      menu.add({
        text : 'Applied Rules...',
        handler : function(button, e) {
          new Sonatype.repoServer.ArtifactProcurementRulePopup(node);
        }
      });
    }
  },

  addAction : function(text, mask, node) {
    var title = text + ' (' + mask + ')';
    this.menu.add({
      text : title,
      scope : this,
      handler : this.showRuleDialog.createDelegate(this, [mask, node])
    });
  },

  addRootActions : function(node) {
    this.addMenuHeader();
    this.addAction('All groups, artifacts and versions', '*:*:*', node);
    this.addAction('A User-Entered GAV', Nexus.procurement.configuration.FREEFORM_GAV, node);
  },

  addArtifactActions : function(node) {
    this.addGroupActions(node);
    this.addSeparator();
    this.addAction('Everything with this artifact id', '*:' + this.artifactId + ':*', node);
    this.addAction('Any artifact starting with this prefix', '*:' + this.artifactId + '*:*', node);
    this.addAction('All versions of this artifact', this.groupId + ':' + this.artifactId + ':*', node);
  },

  addMenuHeader : function() {
    var ti = new Ext.menu.TextItem();
    ti.text = 'Apply a Rule To:';
    ti.style = 'text-align:center;padding-left:27px;';
    this.menu.add(ti);
  },

  addGroupActions : function(node) {
    this.addMenuHeader();
    this.addAction('Everything in this group', this.groupId + ':*:*', node);
    this.addAction('Everything in this group and its sub-groups', this.groupId + '.*:*:*', node);
    this.addAction('Any group starting with this prefix', this.groupId + '*:*:*', node);
  },

  addVersionActions : function(node) {
    this.addArtifactActions(node);
    this.addSeparator();
    this.addAction('Exactly this artifact', this.groupId + ':' + this.artifactId + ':' + this.version, node);
  },

  addSeparator : function() {
    this.menu.addSeparator();
  },

  showRuleDialog : function(mask, node) {
    new Sonatype.repoServer.ArtifactProcurementRulePickerDialog(mask, node);
  },

  approve : function(mask) {
    this.procure('APPROVED', mask);
  },

  deny : function(mask) {
    this.procure('DISAPPROVED', mask);
  },

  procure : function(resolution, mask) {
    var repoRec = this.node.ownerTree.payload;
    var repoName = repoRec.data.name;
    var gav = mask.split(':');

    Ext.Ajax.request({
      method : 'POST',
      jsonData : {
        data : {
          artifactCoordinate : {
            groupId : gav[0],
            artifactId : gav[1],
            version : gav[2]
          },
          resolution : resolution
        }
      },
      url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id,
      scope : this,
      callback : function(options, success, response) {
        if (success) {
          this.node.ownerTree.refreshNode(this.node);
        }
        else {
          Sonatype.utils.connectionError(response, 'Artifact resolution failed in ' + repoName + '.');
        }
      }
    });
  }
}, function() {
  Sonatype.Events.addListener('procurementNodeMenuInit', function(menu, node) {
    new Sonatype.repoServer.ArtifactResolutionMenu(menu, node);
  });
});

