/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,define*/

define('nexus-staging-plugin-boot', [
    'Nexus/staging/repository/RepositoriesView',
    'Sonatype/repoServer/ProfileEditor',
    'Sonatype/repoServer/RulesetEditor',
    'Sonatype/repoServer/StagingPanel',
    'Sonatype/repoServer/StagingRuleSet',
    'Sonatype/repoServer/StagingUploadPanel'
], function() {
    NX.log.debug('Main nexus-staging-plugin modules loaded');
});

// FIXME: WARNING
// FIXME: Find a better place for this to live, still not sure what this is really doing!
// FIXME: WARNING

Sonatype.Events.addListener('privilegePanelInit', function (panel) {
    var profileStore = new Ext.data.JsonStore({
        url: Sonatype.config.servicePath + '/staging/profiles',
        root: 'data',
        id: 'id',

        fields: [
            { name: 'id' },
            { name: 'name', sortType: Ext.data.SortTypes.asUCString }
        ],

        sortInfo: {
            field: 'name',
            direction: 'asc'
        }
    });

    panel.profileStore = profileStore;
    panel.profileStore.load();

    var func = function (value, parent) {
        if (!Ext.isEmpty(value)) {
            // FIXME: What is 'this' here?
            return this.convertDataValue(value, this.profileStore, 'id', 'name');
        }
        return '';
    };

    panel.propertyTypeStore.add(new Ext.data.Record({
        'type': 'profile',
        'converter': func.createDelegate(panel)
    }));
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

// TODO: Move to NX core, once we are ready to use this other places

/**
 * Date format preferences.
 *
 * @since staging 2.3
 */
NX.define('Nexus.prefs.DateFormat', {
    singleton: true,

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    /**
     * @private
     */
    defaultPatterns: {
        date: {
            'short': 'Y-M-d',       // 2013-Mar-06
            'long': 'l, F d, Y'     // Wednesday, March 06, 2013
        },

        time: {
            'short': 'H:i:s',                   // 15:49:57
            'long': 'H:i:s T (\\G\\M\\TO)'      // 15:49:57-0700 PST (GMT-0700)
        },

        datetime: {
            'short': 'Y-M-d H:i:s',                     // 2013-Mar-06 15:49:57
            'long': 'l, F d, Y H:i:s T (\\G\\M\\TO)'    // Wednesday, March 06, 2013 15:50:19 PDT (GMT-0700)
        }
    },

    /**
     * Return the date format object for the given name.
     *
     * Date format objects currently have a 'short' and 'long' variants.
     *
     *      @example
     *      var longDatetimePattern = Nexus.prefs.DateFormat.forName('datetime')['long'];
     *      var shortDatePattern = Nexus.prefs.DateFormat.forName('date')['short'];
     *
     * @param name
     *
     * @return {*} Date format object.
     */
    forName: function (name) {
        var format = this.defaultPatterns[name];

        // if no format, complain and return the full ISO-8601 format
        if (!name) {
            this.logWarn('Missing named format:', name);
            return 'c';
        }

        // TODO: Eventually let this customizable by user, for now its hardcoded

        return format;
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging rule-set data store.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.rule.RuleSetStore', {
    extend: 'Ext.data.JsonStore',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    constructor: function (config) {
        var self = this,
            config = config || {};

        Ext.apply(config, {
            url: Sonatype.config.servicePath + '/staging/rule_sets',
            root: 'data',
            id: 'id',

            fields: [
                { name: 'id' },
                { name: 'name' },
                { name: 'description' },
                { name: 'rules' },
                { name: 'resourceURI' }
            ],

            sortInfo: {
                field: 'name',
                direction: 'ASC'
            }
        });

        self.constructor.superclass.constructor.call(self, config);
    },

    /**
     * @public
     */
    nameForId: function(id) {
        var record = this.getById(id);
        if (record) {
            return record.get('name');
        }
        else {
            this.logDebug('Missing rule-set for ID:', id);
            // use id as name
            return id;
        }
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging rule-type data store.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.rule.RuleTypeStore', {
    extend: 'Ext.data.JsonStore',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    constructor: function (config) {
        var self = this,
            config = config || {};

        Ext.applyIf(config, {
            /**
             * @cfg {Boolean} True to enable filtering of rules with ids starting with 'testing.'
             */
            filterTesting: false
        });

        Ext.apply(config, {
            url: Sonatype.config.servicePath + '/staging/rule_types',
            root: 'data',
            id: 'id',

            fields: [
                { name: 'id' },
                { name: 'name' },
                { name: 'description' },
                { name: 'resourceURI' }
            ],

            sortInfo: {
                field: 'name',
                direction: 'ASC'
            }
        });

        self.constructor.superclass.constructor.call(self, config);

        if (config.filterTesting === true) {
            self.logDebug('Filtering testing.* rules enabled');

            self.on('load', function(store, records, options) {
                self.filterBy(function (record, id) {
                    // ignore any rule who's id starts with this prefix
                    return !id.startsWith('testing.');
                });
            });
        }
    },

    /**
     * @public
     */
    nameForId: function (id) {
        var record = this.getById(id);
        if (record) {
            return record.get('name');
        }
        else {
            this.logDebug('Missing rule-type for ID:', id);
            // use id as name
            return id;
        }
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Container for icons used by staging.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.Icons', {
    extend: 'Nexus.util.IconContainer',
    singleton: true,

    /**
     * @constructor
     */
    constructor: function() {
        var self = this;

        // helper to build an icon config with variants, where variants live in directories, foo.png x16 -> x16/foo.png
        function iconConfig(fileName, variants) {
            var config = {};
            if (variants === undefined) {
                variants = [ 'x32', 'x16' ]
            }
            Ext.each(variants, function(variant) {
                config[variant] = variant + '/' + fileName;
            });
            return config;
        }

        self.constructor.superclass.constructor.call(self, {
            stylePrefix: 'nx-staging-icons-',

            icons: {
                accept:             iconConfig('accept.png'),
                bug:                iconConfig('bug.png'),
                cog:                iconConfig('cog.png'),
                cog_green:          iconConfig('cog_green.png'),
                cog_red:            iconConfig('cog_red.png'),
                cross:              iconConfig('cross.png'),
                database:           iconConfig('database.png'),
                database_back:      iconConfig('database_back.png'),
                database_delete:    iconConfig('database_delete.png'),
                database_edit:      iconConfig('database_edit.png'),
                database_go:        iconConfig('database_go.png'),
                database_link:      iconConfig('database_link.png'),
                database_save:      iconConfig('database_save.png'),
                email:              iconConfig('email.png'),
                error:              iconConfig('error.png'),
                gear:               '@cog',
                gear_in:            iconConfig('gear_in.png'),
                gear_in_green:      iconConfig('gear_in_green.png'),
                gear_in_red:        iconConfig('gear_in_red.png'),
                page_copy:          iconConfig('page_copy.png'),
                exclamation:        iconConfig('exclamation.png'),
                folder_database:    iconConfig('folder_database.png'),
                folder_go:          iconConfig('folder_go.png'),
                star:               iconConfig('star.png'),
                tick:               iconConfig('tick.png'),
                warning:            '@error',

                // common
                arrow_refresh:      'arrow_refresh.png', // use silk version here w/o any variants to keep refresh consistent across the app
                refresh:            '@arrow_refresh',
                selectionMulti:     'selection-select.png', // this is from fugue icon set
                selectionEmpty:     '@warning',

                // repository types
                repositoryOpened:           '@database_edit',
                repositoryClosed:           '@database',
                repositoryPromoted:         '@database_go',
                repositoryGroup:            '@folder_database',
                repositoryGroupPromoted:    '@folder_go',
                repositoryGroupReleased:    '@database_link',
                repositoryReleased:         '@database_link',

                // repository actions
                closeRepository:            '@database_save',
                dropRepository:             '@database_delete',
                releaseRepository:          '@database_link',
                promoteRepository:          '@database_go',

                transitioning:              '@cog',
                transitioningBadge:         'gear-small.png',

                // activities
                activity_open:              '@database_edit',
                activity_close:             '@database_save',
                activity_promote:           '@database_go',
                activity_group:             '@folder_database',
                activity_release:           '@database_link',
                activity_drop:              '@database_delete',
                activity_demote:            '@database_back',

                // events
                eventSuccess:                   '@accept',
                eventFailure:                   '@exclamation',
                event_error:                    '@error',
                event_repositoryCreated:        '@eventSuccess',
                event_repositoryGroupCreated:   '@eventSuccess',
                event_repositoryClosed:         '@eventSuccess',
                event_repositoryCloseFailed:    '@eventFailure',
                event_repositoryPromoted:       '@eventSuccess',
                event_repositoryPromoteFailed:  '@eventFailure',
                event_repositoryDemoted:        '@eventSuccess',
                event_repositoryDemoteFailed:   '@eventFailure',
                event_repositoryReleased:       '@eventSuccess',
                event_repositoryReleaseFailed:  '@eventFailure',

                // rules
                event_rulesEvaluate:            '@gear_in',
                event_rulesPassed:              '@gear_in_green',
                event_rulesFailed:              '@gear_in_red',
                event_ruleEvaluate:             '@cog',
                event_rulePassed:               '@cog_green',
                event_ruleFailed:               '@cog_red',

                // email
                event_email:                    '@email',

                // copy items
                event_copyItems:                '@page_copy',

                // notifications
                notification_0:     '@accept',
                notification_1:     'notification-counter.png',
                notification_2:     'notification-counter-02.png',
                notification_3:     'notification-counter-03.png',
                notification_4:     'notification-counter-04.png',
                notification_5:     'notification-counter-05.png',
                notification_6:     'notification-counter-06.png',
                notification_7:     'notification-counter-07.png',
                notification_8:     'notification-counter-08.png',
                notification_9:     'notification-counter-09.png',
                notification_10:    'notification-counter-10.png',
                notification_11:    'notification-counter-11.png',
                notification_12:    'notification-counter-12.png',
                notification_13:    'notification-counter-13.png',
                notification_14:    'notification-counter-14.png',
                notification_15:    'notification-counter-15.png',
                notification_16:    'notification-counter-16.png',
                notification_17:    'notification-counter-17.png',
                notification_18:    'notification-counter-18.png',
                notification_19:    'notification-counter-19.png',
                notification_20:    'notification-counter-20.png',
                notification_more:  'notification-counter-20-plus.png'
            }
        });
    },

    //
    // FIXME: Should add this feature to the IconContainer, so that instead of exceptions we'd instead
    // FIXME: get some default icon which might help show problems visually but without causing javascript errors.
    //
    // FIXME: Since get() still throws an exception and its used all over, even use of invalidIcon() here
    // FIXME: could still cause javascript errors.
    //

    /**
     * Return icon to use when otherwise would be invalid.
     *
     * @private
     */
    invalidIcon: function(message) {
        this.logWarn(message);
        return this.get('bug');
    },

    /**
     * Determine icon for a given repository record data.
     *
     * @return {*} Icon; never null/undefined.
     */
    forRepository: function(repo) {
        var self = this;

        // promotion groups have a bit more work to figure out which icon to use
        if (repo.profileType === 'group') {
            if (repo.type === 'promoted') {
                return self.get('repositoryGroupPromoted');
            }
            else if (repo.type === 'released') {
                return self.get('repositoryGroupReleased');
            }
            else {
                return self.get('repositoryGroup');
            }
        }

        // else staging repository, has simple state -> icon
        if (repo.type === 'open') {
            return self.get('repositoryOpened');
        }
        else if (repo.type === 'closed') {
            return self.get('repositoryClosed');
        }
        else if (repo.type === 'promoted') {
            return self.get('repositoryPromoted');
        }
        else if (repo.type === 'released') {
            return self.get('repositoryReleased');
        }

        return self.invalidIcon('Unable to determine icon for: ' + repo);
    },

    /**
     * Determine icon for notification count.
     *
     * @return {*} Icon; never null/undefined.
     */
    forNotificationCount: function (count) {
        NX.assert(Ext.isNumber(count) && count >= 0);

        if (count > 20) {
            return this.get('notification_more');
        }
        else {
            return this.get('notification_' + count);
        }
    },

    // TODO: Probably should look into helpers to simplify making/using badges

    /**
     * Determine HTML for a repository icon with an optional badge.
     *
     * @return {String} Icon + optional badge HTML.
     */
    renderRepositoryWithBadge: function(repo) {
        var iconImg = this.forRepository(repo).img,
            badgeCls;

        if (repo.transitioning) {
            badgeCls = this.get('transitioningBadge').cls;
            return '<div><span class="' + badgeCls + ' nx-staging-icons-transitioning-badge"></span>' + iconImg + '</div>';
        }
        else if (repo.notifications > 0) {
            badgeCls = this.forNotificationCount(repo.notifications).cls;
            return '<div><span class="' + badgeCls + ' nx-staging-icons-notification-badge"></span>' + iconImg + '</div>';
        }
        else {
            return iconImg;
        }
    },

    /**
     * Determine icon for a given activity name.
     *
     * @param name  Activity name.
     * @return {*}  Icon; never null/undefined.
     */
    forActivityName: function(name) {
        return this.get('activity_' + name);
    },

    /**
     * Determine icon for a given event name.
     *
     *  @return {*} Icon; never null/undefined.
     */
    forEventName: function(name) {
        return this.get('event_' + name);
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging profile data store.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.profile.ProfileStore', {
    extend: 'Ext.data.JsonStore',

    /**
     * Flag if group profiles have been detected.
     *
     * @type {boolean}
     */
    hasGroups: false,

    constructor: function (config) {
        var self = this,
            config = config || {};

        // customize store configuration
        Ext.apply(config, {
            url: Sonatype.config.servicePath + '/staging/profiles',
            root: 'data',
            id: 'id',

            fields: [
                { name: 'resourceURI' },
                { name: 'id' },
                { name: 'name' },
                { name: 'promotionTargetRepository' },
                { name: 'mode' },
                { name: 'repositoryType' }
            ],

            sortInfo: {
                field: 'name',
                direction: 'ASC'
            }
        });

        // initialize parent class with customized configuration
        self.constructor.superclass.constructor.call(self, config);

        // Detect if there are any group profiles on load
        self.on('load', function(store, records) {
            var i;
            self.hasGroups = false;
            for (i = 0; i < records.length; i = i + 1) {
                if (records[i].get('mode') === 'GROUP') {
                    self.hasGroups = true;
                    break;
                }
            }
        });

        // Reset if a failure occurs
        self.on('loadexception', function() {
            self.hasGroups = false;
        })
    },

    /**
     * @public
     */
    nameForId: function(id) {
        var record = this.getById(id);
        if (record) {
            return record.get('name');
        }
        else {
            this.logDebug('Missing profile for ID:', id);
            // use id as name
            return id;
        }
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/

//
// FIXME: This is now only needed by staging upload UI, but really that UI needs to be updated anyways to handle async + activities for this,
// FIXME: so likely this will completely go away once that is updated.
//

define('Nexus/staging/Rules', function() {

    Ext.namespace('Nexus.staging');
    (function() {

      /**
       * Collection of methods that defines behavior.
       * @param methods An object holding methods that will be applied to an enhanced objects' prototype.
       * @constructor
       */
      var Trait = function(methods) {
        /**
         * @static
         * @param obj The object to enhance with this Trait's method collection.
         */
        this.treat = function(obj) {
          if (obj.prototype) {
            obj = obj.prototype;
          }
          Ext.apply(obj, methods);
        };
      };

      Nexus.staging.Rules = new Trait({
        generateHtmlFromRulesetFailure : function(responseText) {
          var
                json = Ext.decode(responseText),
                html = "<p><h3>Staging ruleset evaluation on repository '" + json.repositoryName + "' has failed.</h3>";

          Ext.each(json.failures, function(item) {
            html += "<br/><p><h3>" + item.ruleName + "</h3></p>";
            Ext.each(item.messages, function(msg) {
              html += "&nbsp;&nbsp;&nbsp;&nbsp;-" + msg.replace(/^([^:]*):/, '<b>$1:</b>') + "<br/>";
            });
          });

          html += "</p>";

          return html;
        },
        showRulesetEvaluationReport : function(responseText) {
          var reportWindow = new Ext.Window({
            id : 'ruleset-evaluation-report',
            title : 'Ruleset Evaluation Report',
            layout : 'border',
            width : 600,
            height : 300,
            buttonAlign : 'center',
            items : [
              {
                xtype : 'panel',
                autoScroll : true,
                html : this.generateHtmlFromRulesetFailure(responseText),
                region : 'center'
              }
            ],
            buttons : [
              {
                text : 'OK',
                scope : this,
                handler : function() {
                  Ext.getCmp('ruleset-evaluation-report').close();
                }
              }
            ]
          });
          reportWindow.show();
        }
      });
    })();

})


/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryDropFailed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryDropFailed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Drop failed';
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'rulePassed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_rulePassed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        var id = this.getProperties().typeId;
        return 'Passed: ' + this.getRuleTypeName(id);
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryCloseFailed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryCloseFailed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Close failed';
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryReleased' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryReleased', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        if (!this.getProperties().target) {
            // groups do not have a target
            return 'Repository released';
        }
        else {
            return 'Repository released to: ' + this.getProperties().target;
        }
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryPromoted' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryPromoted', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Repository promoted to group: ' + this.getPropertyValues('group');
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryPromoteFailed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryPromoteFailed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Promotion failed';
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * HACK: Exists only to prevent exceptions when {@link #loadComplete()} is invoked after node already destroyed.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.AsyncNodeSupport', {
    extend: 'Ext.tree.AsyncTreeNode',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    /**
     * @constructor
     */
    constructor: function (config) {
        var self = this,
            config = config || {};

        self.constructor.superclass.constructor.call(self, config);
    },

    /**
     * @override
     * @private
     */
    loadComplete: function (deep, anim, callback, scope) {
        // HACK: Only attempt to call loadComplete if we haven't been destroyed
        if (!this.ui) {
            this.logWarn('Aborting loadComplete callback; node destroyed:', this);
            return;
        }
        //this.constructor.superclass.loadComplete.apply(this, arguments);
        Nexus.staging.repository.activity.AsyncNodeSupport.superclass.loadComplete.apply(this, arguments);
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * ???
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.DetailSupport', {
    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.prefs.DateFormat',
        'Nexus.staging.Icons'
    ],

    statics: {
        /**
         * Helper to render a date in a full timestamp.
         *
         * @protected
         */
        timestamp: function (date) {
            return date.format(Nexus.prefs.DateFormat.forName('datetime')['long']);
        },

        /**
         * Message icon, title and timestamp.
         *
         * @protected
         */
        headerTpl: NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-ActivityView-messagePanel-head">',
                '{icon}',
                '<div class="nx-staging-repository-ActivityView-messagePanel-title">{title}</div>',
                '<div class="nx-staging-repository-ActivityView-messagePanel-timestamp">{timestamp}</div>',
            '</div>',
            {
                compiled: true
            }
        ),

        /**
         * Message body, no additional detail provided.
         *
         * @protected
         */
        noDetailTpl: NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-ActivityView-messagePanel-body-nodetail">',
                'No additional detail provided for event.',
            '</div>',
            {
                compiled: true
            }
        ),

        /**
         * Renders properties as key -> value data.
         *
         * @protected
         */
        propertiesTpl: NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-ActivityView-messagePanel-properties">',
                '<tpl if="values.properties">',
                    '<table>',
                        '<tpl for="properties">',
                            '<tr class="{[ this.rowclass(xindex) ]}">',
                                '<td class="nx-staging-repository-ActivityView-messagePanel-properties-name">{name}</td>',
                                '<td class="nx-staging-repository-ActivityView-messagePanel-properties-value">{value}</td>',
                            '</tr>',
                        '</tpl>',
                    '</table>',
                '</tpl>',
                '<tpl if="!values.properties">',
                    '{[ Nexus.staging.repository.activity.DetailSupport.noDetailTpl.apply() ]}',
                '</tpl>',
            '</div>',
            {
                compiled: true,

                rowclass: function(xindex) {
                    var base = 'nx-staging-repository-ActivityView-messagePanel-properties-entry-';
                    if (xindex % 2 === 0) {
                        return base + 'even';
                    }
                    else {
                        return base + 'odd';
                    }
                }
            }
        )
    },

    // TODO: Add node and message panel icon/text representations, they might be slightly different.

    /**
     * @public
     */
    getIcon: function () {
        throw 'abstract';
    },

    /**
     * @public
     */
    getTitle: function () {
        throw 'abstract';
    },

    /**
     * @public
     */
    getTemplate: function () {
        throw 'abstract';
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryReleaseFailed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryReleaseFailed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Released failed';
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'rulesEvaluate' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_rulesEvaluate', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        var id = this.getProperties().id;

        return 'Evaluating rules: ' + this.getRuleSetName(id);
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Support to render activity details into the message panel.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.ActivityDetail', {
    extend: 'Nexus.staging.repository.activity.DetailSupport',

    statics: {
        /**
         * @protected
         */
        activityTpl: NX.create('Ext.XTemplate',
            '<div>',
                '{[ this.header(values) ]}',
                '<div class="nx-staging-repository-ActivityView-messagePanel-body">',
                    '{[ this.details(values) ]}',
                '</div>',
            '</div>',
            {
                compiled: true,

                header: function (values) {
                    var detail = values.detail;

                    return Nexus.staging.repository.activity.DetailSupport.headerTpl.apply({
                        title: 'Activity: ' + detail.getTitle(),
                        icon: detail.getIcon().variant('x32').img, // use larger 32x32 icons if we have them
                        timestamp: this.renderDuration(values)
                    });
                },

                renderDuration: function (values) {
                    var activity = values.data,
                        elapsed;

                    // FIXME: Format something more reasonable than 'ms'
                    if (!activity.stopped) {
                        elapsed = activity.started.getElapsed(/*now*/);
                        return '(Running) ' + elapsed + 'ms';
                    }

                    elapsed = activity.started.getElapsed(activity.stopped);
                    return '' + elapsed + 'ms';
                },

                details: function (values) {
                    var activity = values.data,
                        properties = [];

                    properties.push(
                        { name: 'started', value: Nexus.staging.repository.activity.DetailSupport.timestamp(activity.started) }
                    );

                    if (activity.startedByUserId) {
                        properties.push(
                            { name: 'userId', value: activity.startedByUserId },
                            { name: 'ipAddress', value: activity.startedByIpAddress }
                        );
                    }

                    if (activity.stopped) {
                        properties.push(
                            { name: 'stopped', value: Nexus.staging.repository.activity.DetailSupport.timestamp(activity.stopped) }
                        );
                    }

                    properties.push(
                        { name: 'events', value: activity.events ? activity.events.length : 0 }
                    );

                    return Nexus.staging.repository.activity.DetailSupport.propertiesTpl.apply({
                        properties: properties
                    });
                }
            }
        )
    },

    /**
     * @property
     */
    activity: undefined,

    /**
     * @constructor
     */
    constructor: function (activity) {
        NX.assert(activity, 'Missing activity');
        this.activity = activity;
    },

    /**
     * @override
     */
    getIcon: function () {
        try {
            return Nexus.staging.Icons.forActivityName(this.activity.name);
        }
        catch (e) {
            this.logError('Missing icon for activity: ' + this.activity.name);
            return Nexus.staging.Icons.get('bug');
        }
    },

    /**
     * @override
     */
    getTitle: function () {
        return this.activity.name;
    },

    /**
     * @override
     */
    getTemplate: function () {
        return Nexus.staging.repository.activity.ActivityDetail.activityTpl;
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryDemoteFailed' detail.
 *
 * @since 2.7
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryDemoteFailed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Demotion failed';
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryCreated' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryCreated', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Repository created';
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'rulesFailed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_rulesFailed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        var id = this.getProperties().id,
            failures = this.getProperties().failureCount;

        failures = parseInt(failures);

        return Ext.util.Format.plural(failures, 'rule') + ' failed: ' + this.getRuleSetName(id);
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Manages activity/event detail mapping.
 *
 * @since staging 2.3
 *
 * @singleton
 */
NX.define('Nexus.staging.repository.activity.DetailManager', {
    singleton: true,

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.repository.activity.ActivityDetail',
        'Nexus.staging.repository.activity.EventDetail',

        // FIXME: Duplication here w/ *details is painful, how can we not require this?

        // repository*
        'Nexus.staging.repository.activity.EventDetail_repositoryCreated',
        'Nexus.staging.repository.activity.EventDetail_repositoryClosed',
        'Nexus.staging.repository.activity.EventDetail_repositoryCloseFailed',
        'Nexus.staging.repository.activity.EventDetail_repositoryPromoted',
        'Nexus.staging.repository.activity.EventDetail_repositoryPromoteFailed',
        'Nexus.staging.repository.activity.EventDetail_repositoryDemoted',
        'Nexus.staging.repository.activity.EventDetail_repositoryDemoteFailed',
        'Nexus.staging.repository.activity.EventDetail_repositoryReleased',
        'Nexus.staging.repository.activity.EventDetail_repositoryReleaseFailed',
        'Nexus.staging.repository.activity.EventDetail_repositoryDropFailed',

        // group*
        'Nexus.staging.repository.activity.EventDetail_repositoryGroupCreated',

        // rules*
        'Nexus.staging.repository.activity.EventDetail_rulesEvaluate',
        'Nexus.staging.repository.activity.EventDetail_rulesPassed',
        'Nexus.staging.repository.activity.EventDetail_rulesFailed',

        // rule*
        'Nexus.staging.repository.activity.EventDetail_ruleEvaluate',
        'Nexus.staging.repository.activity.EventDetail_rulePassed',
        'Nexus.staging.repository.activity.EventDetail_ruleFailed',

        // email
        'Nexus.staging.repository.activity.EventDetail_email',

        // copy items
        'Nexus.staging.repository.activity.EventDetail_copyItems'
    ],

    /**
     * Event name to 'activity' detail class (either class-name or class-instance (after cached).
     *
     * @private
     */
    activityDetails: {
        // for now, all activities will use the basic ActivityDetail
    },

    /**
     * Event name to 'event' detail class (either class-name or class-instance.
     *
     * @private
     */
    eventDetails: {
        // FIXME: Must keep in sync with 'requires' above

        // repository*
        'repositoryCreated':        'Nexus.staging.repository.activity.EventDetail_repositoryCreated',
        'repositoryGroupCreated':   'Nexus.staging.repository.activity.EventDetail_repositoryGroupCreated',
        'repositoryClosed':         'Nexus.staging.repository.activity.EventDetail_repositoryClosed',
        'repositoryCloseFailed':    'Nexus.staging.repository.activity.EventDetail_repositoryCloseFailed',
        'repositoryPromoted':       'Nexus.staging.repository.activity.EventDetail_repositoryPromoted',
        'repositoryPromoteFailed':  'Nexus.staging.repository.activity.EventDetail_repositoryPromoteFailed',
        'repositoryDemoted':        'Nexus.staging.repository.activity.EventDetail_repositoryDemoted',
        'repositoryDemoteFailed':   'Nexus.staging.repository.activity.EventDetail_repositoryDemoteFailed',
        'repositoryReleased':       'Nexus.staging.repository.activity.EventDetail_repositoryReleased',
        'repositoryReleaseFailed':  'Nexus.staging.repository.activity.EventDetail_repositoryReleaseFailed',
        'repositoryDropFailed':     'Nexus.staging.repository.activity.EventDetail_repositoryDropFailed',

        // rules*
        'rulesEvaluate':        'Nexus.staging.repository.activity.EventDetail_rulesEvaluate',
        'rulesPassed':          'Nexus.staging.repository.activity.EventDetail_rulesPassed',
        'rulesFailed':          'Nexus.staging.repository.activity.EventDetail_rulesFailed',

        // rule*
        'ruleEvaluate':         'Nexus.staging.repository.activity.EventDetail_ruleEvaluate',
        'rulePassed':           'Nexus.staging.repository.activity.EventDetail_rulePassed',
        'ruleFailed':           'Nexus.staging.repository.activity.EventDetail_ruleFailed',

        // email
        'email':                'Nexus.staging.repository.activity.EventDetail_email',

        // copy items
        'copyItems':            'Nexus.staging.repository.activity.EventDetail_copyItems'
    },

    /**
     * @private
     */
    getDetailType: function (type, name) {
        var self = this,
            details,
            detailType;

        self.logDebug('Looking up ' + type + ' detail: ' + name);

        switch (type) {
            case 'activity':
                details = self.activityDetails;
                break;
            case 'event':
                details = self.eventDetails;
                break;
            default:
                throw new Error('Invalid type: ' + type);
        }

        detailType = details[name];

        // select a default detail type
        if (detailType === undefined) {
            self.logDebug('No detail class configured for ' + type + ': ' + name + '; using default');

            switch (type) {
                case 'activity':
                    detailType = 'Nexus.staging.repository.activity.ActivityDetail';
                    break;
                case 'event':
                    detailType = 'Nexus.staging.repository.activity.EventDetail';
                    break;
                // default already covered
            }
        }

        // Reference detail type class and cache it
        if (Ext.isString(detailType)) {
            var className = detailType;
            detailType = NX.obj(className);
            details[name] = detailType;
            self.logDebug('Cached ' + type + ' detail type: ' + className);
        }

        return detailType;
    },

    /**
     * Return activity detail class.
     */
    forActivity: function (activity) {
        var detailType = this.getDetailType('activity', activity.name);
        return new detailType(activity);
    },

    /**
     * Return event detail class.
     */
    forEvent: function (event) {
        var detailType = this.getDetailType('event', event.name);
        return new detailType(event);
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Shows the current activity for a staging repository.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.ActivityView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.Icons',
        'Nexus.staging.repository.activity.ActivityTreeLoader',
        'Nexus.staging.repository.activity.DetailManager'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.treeLoader = NX.create('Nexus.staging.repository.activity.ActivityTreeLoader');

        self.treePanel = NX.create('Ext.tree.TreePanel', {
            cls: 'nx-staging-repository-ActivityView-treePanel',
            border: false,
            region: 'center',
            minWidth: 100,
            autoScroll: true,

            loader: self.treeLoader,
            root: self.treeLoader.createRootNode(),
            rootVisible: false,

            listeners: {
                click: function(node) {
                    self.nodeSelected(node);
                }
            }
        });
        
        self.messagePanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-ActivityView-messagePanel',
            border: false,
            region: 'east',
            minWidth: 100,
            width: '50%',
            split: true,
            collapsed: true,
            collapseMode: 'mini',
            autoScroll: true
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-ActivityView',
            title: 'Activity',
            border: false,
            layout: 'border',
            items: [
                self.treePanel,
                self.messagePanel
            ]
        });

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * @private
     */
    hideMessagePanel: function () {
        var self = this;
        self.messagePanel.collapse(false);
        self.messagePanel.hide();

        // HACK: Avoid ugly think black bar where collapse tool would have been rendered
        self.doLayout();
    },

    /**
     * @private
     */
    showMessagePanel: function () {
        var self = this;
        self.messagePanel.show();
        self.messagePanel.expand(false);
    },

    /**
     * Node selected, update message panel details.
     *
     * @private
     */
    nodeSelected: function(node) {
        var self = this,
            data = node.attributes.data,
            dataType = node.attributes.dataType,
            detail;

        self.logDebug('Selected:', dataType, node.id, node.text);

        switch (dataType) {
            case 'activity':
                detail = Nexus.staging.repository.activity.DetailManager.forActivity(data);
                break;

            case 'event':
                detail = Nexus.staging.repository.activity.DetailManager.forEvent(data);
                break;

            case 'root':
            case 'repository':
            case 'empty':
                // ignore these types and hide the message panel
                self.hideMessagePanel();
                return;

            default:
                self.logWarn('Unsupported dataType:', dataType);
        }

        // render template
        detail.getTemplate().overwrite(self.messagePanel.body, {
            detail: detail,
            data: data
        });

        self.showMessagePanel();
    },

    /**
     * Update view from record.
     *
     * @param repo
     */
    updateRecord: function (repo) {
        var self = this,
            root,
            node;

        node = self.treeLoader.createRepositoryNode(repo);
        root = self.treePanel.getRootNode();
        root.removeAll(true);
        root.appendChild(node);

        self.hideMessagePanel();
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'ruleFailed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_ruleFailed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        var id = this.getProperties().typeId;
        return 'Failed: ' + this.getRuleTypeName(id);
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Support to render event details into the message panel.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail', {
    extend: 'Nexus.staging.repository.activity.DetailSupport',

    requires: [
        'Nexus.staging.repository.RepositoriesMediator'
    ],

    statics: {
        /**
         * @protected
         */
        headerTpl: NX.create('Ext.XTemplate',
            '{[ this.header(values) ]}',
            {
                compiled: true,

                header: function (values) {
                    var event = values.data,
                        detail = values.detail;

                    // delegate to default detail header template
                    return Nexus.staging.repository.activity.DetailSupport.headerTpl.apply({
                        title: 'Event: ' + detail.getTitle(),
                        icon: detail.getIcon().variant('x32').img,
                        timestamp: Nexus.staging.repository.activity.DetailSupport.timestamp(event.timestamp)
                    });
                }
            }
        ),

        /**
         * Default body template.
         *
         * @protected
         */
        bodyTmpl: NX.create('Ext.XTemplate',
            '{[ this.properties(values) ]}',
            {
                compiled: true,

                properties: function (values) {
                    var event = values.data;

                    return Nexus.staging.repository.activity.DetailSupport.propertiesTpl.apply({
                        properties: event.properties
                    })
                }
            }
        ),

        /**
         * @private
         */
        bodyTplFactories: {
            // HACK: This is here only for testing
            'ruleEvaluate': [
                function(event, detail) {
                    switch (detail.getPropertyValue('typeId')) {
                        case 'testing.always-pass':
                        case 'testing.always-fail':
                            return NX.create('Ext.XTemplate',
                                '<div align="center">',
                                    '<b>OINK?</b>',
                                '</div>',
                                {
                                    compiled: true
                                }
                            );

                        default:
                            return undefined;
                    }
                }
            ],

            // HACK: This is here only for testing
            'rulePassed': [
                function(event, detail) {
                    switch (detail.getPropertyValue('typeId')) {
                        case 'testing.always-pass':
                            return NX.create('Ext.XTemplate',
                                '<div align="center">',
                                    '<b>OINK!</b>',
                                    '<br/><br/>',
                                    '<img src="', Sonatype.config.resourcePath, '/static/images/pig.png">',
                                '</div>',
                                {
                                    compiled: true
                                }
                            );

                        default:
                            return undefined;
                    }
                }
            ],

            // HACK: This is here only for testing
            'ruleFailed': [
                function(event, detail) {
                    switch (detail.getPropertyValue('typeId')) {
                        case 'testing.always-fail':
                            return NX.create('Ext.XTemplate',
                                '<div align="center">',
                                    '<b>DINNER!</b>',
                                    '<br/><br/>',
                                    '<img src="', Sonatype.config.resourcePath, '/static/images/ham.png">',
                                '</div>',
                                {
                                    compiled: true
                                }
                            );

                        default:
                            return undefined;
                    }
                }
            ]
        },

        /**
         * Register a body template factory for an event.
         *
         * @public
         *
         * @param {string} eventName
         * @param {function} factoryFn
         */
        registerBodyTemplateFactory: function(eventName, factoryFn) {
            var factories = Nexus.staging.repository.activity.EventDetail.bodyTplFactories[eventName];
            if (factories === undefined) {
                factories = [];
            }
            factories.push(factoryFn);
        },

        /**
         * @private
         */
        getBodyTemplateFactories: function(event) {
            var factories = Nexus.staging.repository.activity.EventDetail.bodyTplFactories[event.name];
            if (Ext.isArray(factories)) {
                return factories;
            }
            else if (Ext.isFunction(factories)) {
                return [ factories ];
            }
            else {
                NX.assert(factories === undefined, 'Invalid bodyTplFactories entry for event name:', event.name, factories);
            }
            return undefined;
        }
    },

    /**
     * @property
     */
    event: undefined,

    /**
     * @constructor
     */
    constructor: function (event) {
        NX.assert(event, 'Missing event');
        this.event = event;
    },

    /**
     * @override
     */
    getIcon: function () {
        try {
            return Nexus.staging.Icons.forEventName(this.event.name);
        }
        catch (e) {
            this.logError('Missing icon for event: ' + this.event.name);
            return Nexus.staging.Icons.get('bug');
        }
    },

    /**
     * @override
     */
    getTitle: function () {
        return this.event.name;
    },

    /**
     * @private
     */
    selectBodyTemplate: function() {
        var self = this,
            tpl;

        var factories = Nexus.staging.repository.activity.EventDetail.getBodyTemplateFactories(self.event);

        // if there are factories, then ask them to create the template, first one wins
        if (factories) {
            var i;
            for (i = 0; i<factories.length; i++) {
                tpl = factories[i](self.event, self);

                // if the factory created a template, then use it
                if (tpl) {
                    break;
                }
            }
        }

        // use the default if not configured yet
        if (!tpl) {
            tpl = Nexus.staging.repository.activity.EventDetail.bodyTmpl;
        }

        self.logDebug('Selected body template:', tpl);

        return tpl;
    },

    /**
     * @override
     */
    getTemplate: function () {
        var self = this,
            bodyTpl = self.selectBodyTemplate();

        // no need to cache here, we get a new instance each time we render the view
        return NX.create('Ext.XTemplate',
            '<div>',
                '{[ this.header(values) ]}',
                '<div class="nx-staging-repository-ActivityView-messagePanel-body">',
                    '{[ this.body(values) ]}',
                '</div>',
            '</div>',
            {
                //compiled: true,

                header: function (values) {
                    return Nexus.staging.repository.activity.EventDetail.headerTpl.apply(values);
                },

                body: function (values) {
                    return bodyTpl.apply(values);
                }
            }
        );
    },

    // TODO: Split out the properties bits here into a class, this impl is getting messy

    /**
     * @private
     */
    cachedProperties: undefined,

    /**
     * Returns an object representing event properties.
     *
     * @return {*}
     *
     * @protected
     */
    getProperties: function() {
        var self = this,
            result = {},
            value;

        if (self.cachedProperties !== undefined) {
            return self.cachedProperties;
        }

        if (Ext.isArray(self.event.properties)) {
            Ext.each(self.event.properties, function(property) {
                value = self.readPropertyValues(property.name);

                // If there is only one value, then strip off the array
                if (Ext.isArray(value) && value.length === 1) {
                    value = value[0];
                }

                result[property.name] = value;
            });
        }
        else {
            self.logWarn('Event has no properties');
        }

        self.cachedProperties = result;
        //self.logDebug('Cached properties');

        return result;
    },

    /**
     * Read property values directly out of the event.
     *
     * @private
     */
    readPropertyValues: function(name) {
        var self = this,
            result = [];

        if (Ext.isArray(self.event.properties)) {
            Ext.each(self.event.properties, function(property) {
                if (name === property.name) {
                    result.push(property.value);
                }
            });
        }
        else {
            self.logWarn('Event has no properties');
        }

        //if (result.length === 0) {
        //    self.logWarn('Event has no property named:', name);
        //}

        return result;
    },

    /**
     * Check if the even has a named property or not.
     *
     * @protected
     */
    hasProperty: function(name) {
        var self = this,
            value = self.getProperties()[name];

        return value !== undefined && value.length !== 0;
    },

    /**
     * Get named property values.
     *
     * @return {Array} Never undefined.
     *
     * @protected
     */
    getPropertyValues: function(name) {
        var self = this,
            value = self.getProperties()[name];

        if (value === undefined) {
            //self.logWarn('No such property:', name);
            return [];
        }

        // translate to Array if needed
        if (!Ext.isArray(value)) {
            return [ value ];
        }
        return value;
    },

    /**
     * Get a single named property value.
     *
     * @return {String} Never undefined.
     *
     * @protected
     */
    getPropertyValue: function(name) {
        return this.getPropertyValues(name)[0];
    },

    /**
     * Get the name for a rule-set id.
     *
     * @protected
     */
    getRuleSetName: function(ruleSetId) {
        // HACK: special handling for this special rule-set which is always run, its not included in the store
        if (ruleSetId === 'nx-internal-ruleset') {
            return 'Default Always Run';
        }

        return Nexus.staging.repository.RepositoriesMediator.ruleSetStore.nameForId(ruleSetId);
    },

    /**
     * Get the name for a rule-type id.
     *
     * @protected
     */
    getRuleTypeName: function(ruleTypeId) {
        // HACK: special handling for rule included by default, it is not included in the store
        if (ruleTypeId === 'RepositoryWritePolicy') {
            return 'Repository Writable';
        }
        return Nexus.staging.repository.RepositoriesMediator.ruleTypeStore.nameForId(ruleTypeId);
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryGroupCreated' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryGroupCreated', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        var count = this.getPropertyValues('member').length;
        return 'Group created with ' + Ext.util.Format.plural(count, 'member', 'members');
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository activity TreeLoader.
 *
 * @since staging 2.3
 *
 * @see java:com.sonatype.nexus.staging.api.dto.StagingActivityDTO
 */
NX.define('Nexus.staging.repository.activity.ActivityTreeLoader', {
    extend: 'Nexus.util.TreeLoaderSupport',

    requires: [
        'Nexus.staging.repository.activity.DetailManager',
        'Nexus.staging.repository.activity.AsyncNodeSupport'
    ],

    /**
     * Template for dataUrl value.  Rendered with node.attribute.data.
     *
     * @property
     */
    URL_TEMPLATE: NX.create('Ext.Template',
        Sonatype.config.servicePath + '/staging/repository/{repositoryId}/activity',
        {
            compiled: true,
            disableFormats: true
        }),

    //
    // FIXME: We are passing &node=<node.id> parameter which is pointless, should sort out how to remove that.
    // FIXME: and/or figure out how to use it so that its functional.
    //

    /**
     * @constructor
     */
    constructor: function (config) {
        var self = this,
            config = config || {};

        Ext.apply(config, {
            requestMethod: 'GET',
            suppressStatus: [ 404 ],
            dataUrl: 'dynamic'
        });

        self.constructor.superclass.constructor.call(self, config);

        // trigger loading data, url will be updated when root node changes
        self.on('beforeload', function(loader, node, callback) {
            self.updateDataUrl(node);
        });

        // on 404 append the empty node
        self.on('loadexception', function(loader, node, response) {
            // HACK: ignore if node already destroyed
            if (!node.ui) {
                self.logWarn('Aborting (empty) node creation; node destroyed:', node);
                return;
            }

            if (response.status === 404) {
                node.appendChild(self.createEmptyNode());
            }
        });
    },

    /**
     * Change the dataUrl for the given node, which is expected to be the root node only.
     *
     * @private
     */
    updateDataUrl: function (node) {
        var self = this,
            data = node.attributes.data;

        NX.assert(data, 'Node missing attribute: data');
        self.dataUrl = self.URL_TEMPLATE.apply(data);
        self.logDebug('Data URL:', self.dataUrl);
    },

    /**
     * @protected
     * @override
     */
    createNodes: function(node, data) {
        var self = this;

        self.logGroup('Creating nodes; parent:', node.text, node);

        for (var i = 0, len = data.length; i < len; i++) {
            // HACK: Abort if the node was destroyed already
            if (!node.ui) {
                self.logWarn('Aborting node creation; parent node destroyed:', node);
                break;
            }

            // check if this is the last node or not
            var last = ((i + 1) === len);

            var child = self.createActivityNode(data[i], last);
            if (child) {
                node.appendChild(child);
            }
        }

        self.logGroupEnd();
    },

    /**
     * Translate an object field (in ISO 8601 format) to a date.
     *
     * @private
     */
    translateDate: function(object, fieldName) {
        var value = object[fieldName];
        if (value) {
            object[fieldName] = Date.parseDate(value, 'c');
        }
    },

    /**
     * Create an activity node.
     *
     * @param activity
     * @param last      True if this is last/most-recent activity.
     *
     * @private
     */
    createActivityNode: function (activity, last) {
        var self = this,
            node,
            hasEvents,
            i,
            child,
            detailProvider;

        hasEvents = Ext.isArray(activity.events) && activity.events.length > 0;

        // transform string dates into objects
        self.translateDate(activity, 'started');
        self.translateDate(activity, 'stopped');

        detailProvider = Nexus.staging.repository.activity.DetailManager.forActivity(activity);

        node = self.createNode({
            text: detailProvider.getTitle(),
            iconCls: detailProvider.getIcon().cls,
            nodeType: 'node',
            leaf: !hasEvents,
            expanded: last,
            dataType: 'activity',
            data: activity
        });

        if (hasEvents) {
            for (i = 0; i < activity.events.length; i++) {
                child = self.createEventNode(activity.events[i]);
                node.appendChild(child);
            }
        }

        return node;
    },

    /**
     * Create an event node.
     *
     * @param event
     *
     * @private
     */
    createEventNode: function (event) {
        var self = this,
            detailProvider;

        // transform string dates into objects
        self.translateDate(event, 'timestamp');

        detailProvider = Nexus.staging.repository.activity.DetailManager.forEvent(event);

        return self.createNode({
            text: detailProvider.getTitle(),
            iconCls: detailProvider.getIcon().cls,
            nodeType: 'node',
            leaf: true,
            dataType: 'event',
            data: event
        });
    },

    /**
     * Create the root node.
     *
     * @return {*}
     */
    createRootNode: function () {
        return NX.create('Ext.tree.TreeNode', {
            text: 'root',
            expanded: true,
            dataType: 'root',
            data: undefined
        });
    },

    /**
     * Create a node for a repository.
     *
     * @param repo
     * @return {*}
     */
    createRepositoryNode: function (repo) {
        var text = 'Activities';

        // show the repository id when debug is enabled
        if (window.location.search === '?debug') {
            text = text + ': ' + repo.repositoryId;
        }

        // HACK: Using custom async node impl so we can cope with aborting load better when node is destroyed
        // HACK: This happens if user changes repository selection w/ Activity panel active before activity
        // HACK: ... nodes have finished loading for the previously selected repository

        var node = NX.create('Nexus.staging.repository.activity.AsyncNodeSupport', {
            text: text,
            expanded: true,
            dataType: 'repository',
            data: repo
        });

        // disallow node collapse
        node.on('beforecollapse', function() { return false; })

        return node;
    },

    /**
     * Create "empty" node which is displayed as only child of repository node when there is no activity data.
     *
     * @return {*}
     */
    createEmptyNode: function() {
        return NX.create('Ext.tree.TreeNode', {
            text: 'No activity recorded',
            iconCls: Nexus.staging.Icons.get('warning').cls,
            nodeType: 'node',
            leaf: true,
            expanded: true,
            dataType: 'empty',
            data: undefined
        });
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'email' detail.
 *
 * @since 2.6.1
 */
NX.define('Nexus.staging.repository.activity.EventDetail_email', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Sending e-mail notifications';
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'ruleEvaluate' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_ruleEvaluate', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        var id = this.getProperties().typeId;
        return 'Evaluating rule: ' + this.getRuleTypeName(id);
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryDemoted' detail.
 *
 * @since 2.7
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryDemoted', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Repository demoted from group: ' + this.getPropertyValues('group');
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'repositoryClosed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_repositoryClosed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        return 'Repository closed';
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'rulesPassed' detail.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail_rulesPassed', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        var id = this.getProperties().id;

        return 'All rules passed: ' + this.getRuleSetName(id);
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Event 'copyItems' detail.
 *
 * @since 2.6.1
 */
NX.define('Nexus.staging.repository.activity.EventDetail_copyItems', {
    extend: 'Nexus.staging.repository.activity.EventDetail',

    getTitle: function () {
        var source = this.getPropertyValues('source');
        var target = this.getPropertyValues('target');
        return 'Copying items from ' + source + ' to ' + target;
    }

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Window presented to user to confirm an operation on a staging repository.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.ConfirmWindow', {
    extend: 'Ext.Window',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.Icons'
    ],

    statics: {
        /**
         * @private
         */
        messageTpl: NX.create('Ext.Template',
            '<div class="nx-staging-repository-ConfirmWindow-messagePanel-box">',
                '<div class="nx-staging-repository-ConfirmWindow-messagePanel-icon">{icon}</div>',
                '<div class="nx-staging-repository-ConfirmWindow-messagePanel-text">{message}</div>',
            '</div>',
            {
                compiled: true,
                disableFormats: true
            })
    },

    /**
     * URL to submit to on confirmation.
     *
     * @type {String}
     */
    url: undefined,

    /**
     * Activity name
     *
     * @type {String}
     */
    activity: undefined,

    /**
     * Confirmation message.
     *
     * @type {String}
     */
    message: undefined,

    /**
     * @override
     */
    initComponent: function () {
        var self = this,
            icons = Nexus.staging.Icons,
            title;

        NX.assert(self.url !== undefined, 'Missing: url');
        NX.assert(self.activity !== undefined, 'Missing: activity');
        NX.assert(self.message !== undefined, 'Missing: message');

        title = Ext.util.Format.capitalize(self.activity) + ' Confirmation';

        /**
         * @private
         */
        self.messagePanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-ConfirmWindow-messagePanel',
            border: false, // custom border handling in css

            html: Nexus.staging.repository.ConfirmWindow.messageTpl.apply({
                icon: icons.forActivityName(self.activity).variant('x32').img, // use larger 32x32 icons if we have them
                message: self.message
            })
        });

        /**
         * @private
         */
        self.formPanel = NX.create('Ext.form.FormPanel', {
            cls: 'nx-staging-repository-ConfirmWindow-formPanel',
            border: false, // custom border handling in css
            monitorValid: true,
            labelWidth: 120, // increase width to avoid wrapping promote/release option labels

            items: self.buildFormItems(),

            buttons: [
                {
                    text: 'Confirm',
                    formBind: true,
                    scope: self,
                    handler: self.handleConfirm
                },
                {
                    xtype: 'link-button',
                    text: 'Cancel',
                    formBind: false,
                    scope: self,
                    handler: self.handleCancel
                }
            ],

            keys: [
                {
                    // Close dialog on ESC
                    key: Ext.EventObject.ESC,
                    scope: self,
                    fn: self.handleCancel
                }
            ]
        });

        self.addEvents(
            /**
             * @event
             */
            'confirm',

            /**
             * @event
             */
            'cancel',

            /**
             * @event
             */
            'success',

            /**
             * @event
             */
            'failure'
        );

        Ext.apply(self, {
            cls: 'nx-staging-repository-ConfirmWindow',
            title: title,

            // FIXME: icon isn't properly aligned ATM, disable it for now its distracting
            //iconCls: icons.get('warning').cls,

            autoHeight: true,
            width: 400,
            border: false,

            modal: true,
            constrain: true,
            closable: false,
            resizable: false,

            items: [
                self.messagePanel,
                self.formPanel
            ],

            listeners: {
                show: function (window) {
                    // auto-focus comment/description if we are showing it
                    self.formPanel.find('name', 'description')[0].focus(false, 100);
                }
            }
        });

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * @private
     *
     * @return {Array}
     */
    buildFormItems: function () {
        var self = this,
            items = [];

        self.customizeFormItems(items);

        items.push({
            xtype: 'fieldset',
            title: 'Description',
            items: [
                {
                    name: 'description',
                    xtype: 'textarea',
                    height: 50,
                    anchor: '100%',
                    hideLabel: true

                    // Disable for now, as we auto-focus this field and its distracting to see this flicker off once the focus is changed
                    //emptyText: 'Optional description or comment'
                }
            ]
        });

        return items;
    },

    /**
     * Customize form items.
     *
     * @protected
     *
     * @param {Array} items
     */
    customizeFormItems: function (items) {
        // override to customize
    },

    /**
     * @private
     */
    handleConfirm: function () {
        var self = this,
            form,
            data,
            mask;

        self.fireEvent('confirm', self);

        form = self.formPanel.getForm();
        data = form.getValues();
        data = self.customizeFormData(form, data)

        mask = NX.create('Ext.LoadMask', self.body, {
            msg: 'Please wait...'
        });
        mask.show();

        Ext.Ajax.request({
            method: 'POST',
            url: self.url,

            options: {
                dontForceLogout: true
            },

            jsonData: data,

            scope: self,
            success: function (response, options) {
                self.handleSuccess(response, options);
                mask.hide();
            },
            failure: function (response, options) {
                self.handleFailure(response, options);
                mask.hide();
            }
        });
    },

    /**
     * Customize form data to submit.
     *
     * @protected
     *
     * @param {Ext.form.BasicForm} form     Form which collected details.
     * @param {*} values                    The form's current values.
     * @return {*}                          Form data to submit.
     */
    customizeFormData: function(form, values) {
        // override to customize
        return values;
    },

    /**
     * @private
     */
    handleCancel: function () {
        var self = this;
        self.fireEvent('cancel', self);
        self.close();
    },

    /**
     * @private
     */
    handleSuccess: function(response, options) {
        var self = this;
        self.fireEvent('success', self, response, options);
        self.close();
    },

    /**
     * @private
     */
    handleFailure: function(response, options) {
        var self = this;
        self.fireEvent('failure', self, response, options);
        self.close();
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Staging repositories master grid.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.RepositoriesGrid', {
    extend: 'Ext.grid.GridPanel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.prefs.DateFormat',
        'Nexus.staging.repository.RepositoriesMediator',
        'Nexus.ext.GridFilterBox'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this,
            mediator = Nexus.staging.repository.RepositoriesMediator,
            icons = Nexus.staging.Icons;

        self.ds = mediator.repositoryStore;
        mediator.repositoryStore.on('beforeload', self.rememberSelection, self);
        mediator.repositoryStore.on('load', self.recallSelection, self);

        self.loadMask = {
            msg: 'Loading...',
            msgCls: 'loading-indicator'
        };

        self.viewConfig = {
          emptyText: 'No staging repositories available',
          emptyTextWhileFiltering: 'No staging repositories matched criteria: {criteria}'
        };

        self.stripeRows = true;

        self.sm = NX.create('Ext.grid.CheckboxSelectionModel', {
            listeners: {
                selectionchange: {
                    fn: self.selectionChanged,
                    scope: self
                }
            }
        });

        var columns = [
            // checkbox
            self.sm,

            // icon
            {
                width: 30, // some padding here for badges
                resizable: false,
                sortable: false,
                fixed: true,
                hideable: false,
                menuDisabled: true,
                renderer: function (value, metaData, record) {
                    return icons.renderRepositoryWithBadge(record.data);
                }
            },
            {
                id: 'repositoryId',
                width: 250,
                header: 'Repository',
                dataIndex: 'repositoryId',
                sortable: true
            },
            {
                id: 'profileName',
                width: 100,
                header: 'Profile',
                dataIndex: 'profileName',
                sortable: true
            },
            {
                id: 'type',
                width: 60,
                header: 'Status',
                dataIndex: 'type',
                sortable: true
            },
            {
                id: 'releaseRepositoryId',
                width: 110,
                header: 'Release To',
                dataIndex: 'releaseRepositoryId',
                sortable: true,
                hidden: true
            },
            {
                id: 'parentGroupId',
                width: 100,
                header: 'Promoted To',
                dataIndex: 'parentGroupId',
                sortable: true,
                hidden: true
            },
            {
                id: 'userId',
                header: 'Owner', // Using owner here for clarity
                dataIndex: 'userId',
                sortable: true,
                hidden: true
            },
            {
                id: 'created',
                width: 150,
                header: 'Created',
                dataIndex: 'created',
                sortable: true,
                xtype: 'datecolumn',
                format: Nexus.prefs.DateFormat.forName('datetime')['short'],
                hidden: true
            },
            {
                id: 'updated',
                width: 150,
                header: 'Updated',
                dataIndex: 'updated',
                sortable: true,
                xtype: 'datecolumn',
                format: Nexus.prefs.DateFormat.forName('datetime')['short']
            },
            {
                id: 'userAgent',
                width: 100,
                header: 'User Agent',
                dataIndex: 'userAgent',
                sortable: true,
                hidden: true
            },

            {
                id: 'description',
                width: 100,
                header: 'Description',
                dataIndex: 'description',
                sortable: true
            }
        ];
        // Allow contributors to add to the view.
        Nexus.staging.repository.RepositoriesMediator.fireEvent('staging.repositoriesGrid.configureColumns', columns);

        self.cm = NX.create('Ext.grid.ColumnModel', {
            columns: columns
        });
        self.autoExpandColumn = 'description';

        // FIXME: Toolbar, and actions for them should really be handled by another class
        // FIXME: Should try to avoid using component ids here, a field ref would be preferable, or itemId instead.

        self.tbar = [
            {
                text: 'Refresh',
                tooltip: 'Refresh staging repositories',
                iconCls: icons.get('refresh').cls,
                handler: function () {
                    mediator.refreshHandler();
                }
                // always enabled
            },
            {
                text: 'Close',
                itemId: 'close',
                iconCls: icons.get('closeRepository').cls,
                tooltip: 'Close staging repositories',
                handler: function () {
                    mediator.closeHandler(self.getSelectionModel().getSelections());
                },
                disabled: true
            },
            {
                text: 'Promote',
                itemId: 'promote',
                iconCls: icons.get('promoteRepository').cls,
                tooltip: 'Promote staging repositories',
                handler: function () {
                    mediator.promoteHandler(self.getSelectionModel().getSelections());
                },
                disabled: true
            },
            {
                text: 'Release',
                itemId: 'release',
                iconCls: icons.get('releaseRepository').cls,
                tooltip: 'Release staging repositories',
                handler: function () {
                    mediator.releaseHandler(self.getSelectionModel().getSelections());
                },
                disabled: true
            },
            {
                text: 'Drop',
                itemId: 'drop',
                iconCls: icons.get('dropRepository').cls,
                tooltip: 'Drop staging repositories',
                handler: function () {
                    mediator.dropHandler(self.getSelectionModel().getSelections());
                },
                disabled: true
            },

            '->',
            NX.create('Nexus.ext.GridFilterBox', {
              filteredGrid: self
            })
        ];

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * @private
     */
    selectionChanged: function (sm) {
        var self = this,
            records,
            provider,
            i,
            mediator = Nexus.staging.repository.RepositoriesMediator,
            sp = Sonatype.lib.Permissions,
            closable,
            promotable,
            relesable,
            droppable;

        function disableAll() {
            closable = false;
            promotable = false;
            relesable = false;
            droppable = false;
        }

        // start out disabled
        disableAll();

        // if there is a selection then we have to work a little to figure out what actions are available
        if (sm.getCount() !== 0) {
            // we have a selection, enable all actions which the user has permission to use
            closable = sp.checkPermission('nexus:stagingfinish', sp.CREATE);
            promotable = sp.checkPermission('nexus:stagingpromote', sp.CREATE);
            relesable = sp.checkPermission('nexus:stagingpromote', sp.CREATE);
            droppable = sp.checkPermission('nexus:stagingdrop', sp.CREATE);

            records = sm.getSelections();
            provider = null;

            // scan through all selections and flip off any actions which are not valid for the selection
            for (i = 0; i < records.length; i = i + 1) {
                // if any record is transitioning disable all toolbar actions and top
                if (records[i].get('transitioning') === true) {
                    disableAll();
                    break;
                }

                // NOTE: Not really sure why this check is done ATM, since IIUC there is only 1 provider 'maven2'

                // the first items provider is used to compare with all other selected repositories
                if (provider === null) {
                    provider = records[i].get('provider');
                }
                // if there is a mismatch, then disable all actions
                else if (provider !== records[i].get('provider')) {
                    disableAll();
                    break;
                }

                if (records[i].get('type') === 'open') {
                    relesable = false;
                    promotable = false;
                }
                else if (records[i].get('type') === 'closed') {
                    closable = false;
                }
                else if (records[i].get('type') === 'promoted') {
                    disableAll();
                    break;
                }
                else if (records[i].get('type') === 'released') {
                    closable = false;
                    relesable = false;
                    promotable = false;
                    break;
                }
            }

            // if there are no promotion profiles, then disable promote action
            if (!mediator.profileStore.hasGroups) {
                promotable = false;
            }
        }

        // toggle enabled state of buttons by itemId
        function toggleEnabled(itemId, enable) {
            var button = self.getTopToolbar().getComponent(itemId);
            NX.assert(button);

            //self.logDebug('Toggling toolbar action enabled:', itemId, enable);

            if (enable) {
                button.enable();
            }
            else {
                button.disable();
            }
        }

        // toggle toolbar buttons
        toggleEnabled('close', closable);
        toggleEnabled('promote', promotable);
        toggleEnabled('release', relesable);
        toggleEnabled('drop', droppable);
    },

    /**
     * @private
     */
    selectedRecords: undefined,

    /**
     * Remember what records are selected.
     *
     * @private
     */
    rememberSelection: function () {
        this.selectedRecords = this.getSelectionModel().getSelections();
    },

    /**
     * Recall selection from previous remembered selection.
     *
     * @private
     */
    recallSelection: function () {
        var self = this,
            toSelect = [];

        if (self.selectedRecords === undefined || self.selectedRecords.length === 0) {
            return;
        }

        Ext.each(self.selectedRecords, function (record) {
            record = self.getStore().getById(record.id);
            if (!Ext.isEmpty(record)) {
                toSelect.push(record);
            }
        });

        self.getSelectionModel().selectRecords(toSelect);
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repositories master/detail view.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.RepositoriesView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.Icons',
        'Nexus.staging.repository.RepositoriesGrid',
        'Nexus.staging.repository.RepositoryView',
        'Nexus.staging.repository.RepositoriesMediator'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this,
            icons = Nexus.staging.Icons;

        // force all datastores to refresh when the main staging repositories view loads
        Nexus.staging.repository.RepositoriesMediator.refreshHandler();

        self.masterPanel = NX.create('Nexus.staging.repository.RepositoriesGrid', {
            region: 'center'
        });

        self.emptySelectionPanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-RepositoriesView-messagePanel',
            title: 'Empty Selection',
            iconCls: icons.get('selectionEmpty').cls,
            html: '<span class="nx-staging-repository-RepositoriesView-messagePanel-text">Please select a staging repository</span>'
        });

        self.multiSelectionPanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-RepositoriesView-messagePanel',
            title: 'Multiple Selection',
            iconCls: icons.get('selectionMulti').cls,
            html: '<span class="nx-staging-repository-RepositoriesView-messagePanel-text">Multiple staging repositories are selected</span>'
        });

        self.repoView = NX.create('Nexus.staging.repository.RepositoryView');

        self.detailPanel = NX.create('Ext.Panel', {
            region: 'south',
            minHeight: 25, // height of panel title
            split: true,
            autoDestroy: false,

            layout: 'card',
            defaults: {
                border: false
            },
            items: [
                self.emptySelectionPanel, // 0
                self.multiSelectionPanel, // 1
                self.repoView             // 2
            ],
            activeItem: 0,

            listeners: {
                // resize detail panel height to 50% of parent on first render
                afterrender: {
                    single: true,
                    fn: function() {
                        self.detailPanel.setHeight(self.getHeight() / 2);
                    }
                }
            }
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-RepositoriesView',
            layout: 'border',
            items: [
                self.masterPanel,
                self.detailPanel
            ]
        });

        self.masterPanel.getSelectionModel().on('selectionchange', self.selectionChanged, self);

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * Update detail panel when grid selection changes.
     *
     * @param sm    grid selection model
     *
     * @private
     */
    selectionChanged: function(sm) {
        var self = this,
            cardLayout = self.detailPanel.getLayout(),
            selections = sm.getSelections();

        self.logDebug('Selection changed:', selections.length);

        if (selections.length === 0) {
            cardLayout.setActiveItem(0); // empty-selection
        }
        else if (selections.length === 1) {
            cardLayout.setActiveItem(2); // single-selection

            // update after layout changed, so that tabs can properly [un]hide
            self.repoView.updateRecord(selections[0].data);
        }
        else {
            cardLayout.setActiveItem(1); // multi-selection
        }
    }

}, function () {
    var type = this,
        sp = Sonatype.lib.Permissions;

    NX.log.debug('Adding global view: ' + type.$className);

    // install panel into main NX navigation
    Sonatype.Events.on('nexusNavigationInit', function (panel) {
        panel.add({
            enabled: sp.checkPermission('nexus:stagingprofilerepos', sp.READ),
            sectionId: 'st-enterprise-staging',
            title: 'Staging Repositories',
            tabId: 'stagingRepositories',
            tabCode: type
        });

        NX.log.debug('Registered global view: ' + type.$className);
    });
})
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository data store.
 *
 * @since staging 2.3
 *
 * @see java:com.sonatype.nexus.staging.api.dto.stagingProfileRepository
 */
NX.define('Nexus.staging.repository.RepositoryStore', {
    extend: 'Ext.data.JsonStore',

    constructor: function (config) {
        var self = this,
            config = config || {},
            ST = Ext.data.SortTypes;

        Ext.apply(config, {
            url: Sonatype.config.servicePath + '/staging/profile_repositories',
            root: 'data',
            id: 'repositoryId',

            // NOTE: There is more data returned from the REST resource, but it is not consumed by the UI

            fields: [
                { name: 'repositoryId' },
                { name: 'repositoryURI' },

                { name: 'profileId' },
                { name: 'profileName' },
                { name: 'profileType' }, // 'group' or 'repository'

                { name: 'policy' }, // always 'release' ???
                { name: 'provider' }, // maven1, maven2, etc.
                { name: 'description' },
                { name: 'type' }, // status

                { name: 'created', type: 'date', dateFormat: 'c' },
                { name: 'updated', type: 'date', dateFormat: 'c' },

                { name: 'releaseRepositoryId' },
                { name: 'parentGroupId' },

                // exposed conditions (ie. properties)
                { name: 'userId', type: 'string' }, // this is 'owner'
                { name: 'ipAddress', type: 'string' },
                { name: 'userAgent', type: 'string' },

                { name: 'notifications', type: 'int' },
                { name: 'transitioning', type: 'boolean' },

                // optional list of additional properties
                {name: 'properties'}
            ],

            sortInfo: {
                field: 'repositoryId',
                direction: 'ASC'
            }
        });

        self.constructor.superclass.constructor.call(self, config);
    },

    /**
     * @public
     */
    nameForId: function(id) {
        var record = this.getById(id);
        if (record) {
            return record.get('name');
        }
        else {
            this.logDebug('Missing repository for ID:', id);
            // use id as name
            return id;
        }
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Staging repositories mediator.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.RepositoriesMediator', {
    extend: 'Ext.util.Observable',
    singleton: true,

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.profile.ProfileStore',
        'Nexus.staging.repository.RepositoryStore',
        'Nexus.staging.rule.RuleSetStore',
        'Nexus.staging.rule.RuleTypeStore',
        'Nexus.staging.repository.ConfirmWindow'
    ],

    /**
     * @constructor
     */
    constructor: function () {
        this.profileStore = NX.create('Nexus.staging.profile.ProfileStore');
        this.repositoryStore = NX.create('Nexus.staging.repository.RepositoryStore');
        this.ruleSetStore = NX.create('Nexus.staging.rule.RuleSetStore');
        this.ruleTypeStore = NX.create('Nexus.staging.rule.RuleTypeStore');

        // provided centralized event management allowing for dynamic enhancement of UI's depending on this class
        this.addEvents(
            /**
             * Fires when repositoriesGrid columns are being defined.
             *
             * @event staging.repositoriesGrid.configureColumns
             * @param {Array} Column configuration objects for Ext.grid.ColumnModel.
             */
            'staging.repositoriesGrid.configureColumns',

            /**
             * Fires when SummaryView properties are being defined.
             *
             * @event staging.repositorySummary.configureProperties
             * @param {Array} properties Objects with name/value properties; push new Object to extend the view.
             * @param {Array} values Repository data for inspection.
             */
            'staging.repositorySummary.configureProperties',

            // NOTE: These are not really in the right place, these are profile specific and this mediator is repository specific

            /**
             * Fires when StagingProfile is submitted.
             *
             * @event staging.profile.submitted
             * @param {Array} properties The StagingProfile properties; create an entry with the KEY for a Field created by staging.profile.loaded event to
             * include it in the submitted form.
             */
            'staging.profile.submitted',

            /**
             * Fires when StagingProfile is loaded.
             *
             * @event staging.profile.loaded
             * @param {MixedCollection} items The Ext.form.Fields definitions for the StagingProfile form; push new Field to extend the view.
             * @param {Array} properties The StagingProfile properties for inspection.
             * @param {String} dataRoot The prefix to be used for naming new Fields. Added fields should define {name: dataRoot + '.' + KEY}
             */
            'staging.profile.loaded'
        );
    },

    refreshHandler: function () {
        this.logDebug('Refreshing');

        this.profileStore.reload();
        this.repositoryStore.reload();
        this.ruleSetStore.reload();
        this.ruleTypeStore.reload();
    },

    closeHandler: function (selected) {
        this.displayConfirmWindow('close', selected);
    },

    dropHandler: function (selected) {
        this.displayConfirmWindow('drop', selected);
    },

    releaseHandler: function (selected) {
        this.displayConfirmWindow('release', selected);
    },

    promoteHandler: function (selected) {
        this.displayConfirmWindow('promote', selected);
    },

    /**
     * @private
     */
    displayConfirmWindow: function (actionName, selected) {
        var self = this,
            action = self.getActionWords(actionName),
            selectedIds = self.repositoryIdsForSelection(selected),
            dialog;

        dialog = NX.create('Nexus.staging.repository.ConfirmWindow', {
            activity: actionName,
            url: Sonatype.config.servicePath + '/staging/bulk/' + action.rest,
            message: '<p>You are about to <b>' + action.verb + '</b> ' +
                Ext.util.Format.plural(selectedIds.length, 'repository', 'repositories') + '.</p>' +
                '<p>This may take some time to complete.</p>',

            // HACK: Test adding a custom field item
            customizeFormItems: function (items) {
                // release needs to include a checkbox to allow automatic dropping
                if (actionName === 'release') {
                    items.push({
                        xtype: 'fieldset',
                        title: 'Options',
                        labelSeparator: '',
                        items: [
                            {
                                xtype: 'checkbox',
                                name: 'autoDropAfterRelease',
                                fieldLabel: 'Automatically Drop',
                                helpText: 'After the release operation is successful, automatically drop released repositories.  Automatic drop was the default behavior of Nexus 2.3 and earlier.',
                                checked: true
                            }
                        ]
                    });
                }
                else if (actionName === 'promote') {
                    // promotion needs to include a promotion profile selection
                    items.push({
                        xtype: 'fieldset',
                        title: 'Options',
                        labelSeparator: '',
                        items: [
                            {
                                xtype: 'combo',
                                name: 'stagingProfileGroup',
                                fieldLabel: 'Promotion Profile',
                                emptyText: 'Select a profile',

                                mode: 'local',
                                store: self.createPromotionProfileStore(selected),
                                displayField: 'name',
                                valueField: 'id', // FIXME: This is not working, value is always from 'name' field :-(
                                triggerAction: 'all',

                                forceSelection: true,
                                editable: false,
                                selectOnFocus: true,
                                allowBlank: false,
                                required: true
                            }
                        ]});
                }
            },

            customizeFormData: function (form, values) {
                values.stagedRepositoryIds = selectedIds;

                // HACK: Work around with checkbox setting form value to "on" instead of true (boolean value)
                if (actionName === 'release') {
                    values.autoDropAfterRelease = form.findField('autoDropAfterRelease').getValue();
                }

                // HACK: Work around issue with combobox not returning valueField
                if (actionName === 'promote') {
                    values.stagingProfileGroup = form.findField('stagingProfileGroup').getValue();
                }

                return {
                    data: values
                }
            },

            listeners: {
                success: function (window, response, options) {
                    self.displaySuccess(action.Noun + ' ' + Ext.util.Format.plural(selectedIds.length, 'repository', 'repositories'));
                },

                failure: function (window, response, options) {
                    self.displayFailure(action.Verb + ' failed', response);
                }
            }
        });

        dialog.show();
    },

    /**
     * @private
     */
    repositoryIdsForSelection: function (selected) {
        var result = [];

        Ext.each(selected, function (record) {
            result.push(record.data.repositoryId);
        });

        return result;
    },

    /**
     * @private
     */
    actionWords: {
        close: {
            noun: 'closing',
            verb: 'close'
        },

        drop: {
            noun: 'dropping',
            verb: 'drop'
        },

        promote: {
            noun: 'promoting',
            verb: 'promote'
        },

        release: {
            noun: 'releasing',
            verb: 'release',
            rest: 'promote'
        }
    },

    /**
     * @private
     */
    getActionWords: function (actionName) {
        var action = this.actionWords[actionName];

        // auto capitalize
        if (!action.capitalized) {
            action.Noun = Ext.util.Format.capitalize(action.noun);
            action.Verb = Ext.util.Format.capitalize(action.verb);
            action.capitalized = true;
        }

        // auto sort rest shit, the names used here don't line up anymore so we fux around it
        if (!action.rest) {
            action.rest = action.verb;
        }

        return action;
    },

    /**
     * Build the promotion profile store, which is displayed to the user in the promote confirmation dialog.
     *
     * @private
     *
     * @return {@Ext.data.Store}
     */
    createPromotionProfileStore: function (records) {
        var self = this,
            data = [],
            provider,
            profileId,
            profileName;

        // FIXME: convoluted way to filter only profiles which repositories can be promoted into

        provider = records[0].get('provider');
        profileId = records[0].get('profileId');

        // TODO: Translate into name with profile store
        profileName = profileId;

        self.profileStore.each(function (rec) {
            // can only promote to groups
            if (rec.get('mode') !== 'GROUP') {
                return;
            }

            // group must match repository type
            if (rec.get('repositoryType') !== provider) {
                return;
            }

            // do not allow the user to promote to a group, if what they are
            // promoting is in the same group profile, and is only thing selected
            if (records.length === 1 && profileName === rec.get('name')) {
                return;
            }

            data.push([
                rec.get('id'),
                rec.get('name')
            ]);
        });

        return NX.create('Ext.data.ArrayStore', {
            id: 0,

            fields: [
                'id',
                'name'
            ],

            data: data
        });
    },

    /**
     * @private
     */
    displaySuccess: function (message) {
        Nexus.messages.show('Staging', message);

        this.repositoryStore.reload();
    },

    /**
     * @private
     */
    displayFailure: function (message, response) {
        Sonatype.utils.connectionError(response, message, false);

        this.repositoryStore.reload();
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository content browser.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.ContentView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Sonatype.repoServer.ArtifactContainer'

        // FIXME: not a real module
        //'Sonatype.repoServer.RepositoryBrowsePanel'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.artifactPanel = new Sonatype.repoServer.ArtifactContainer({
            region: 'east',
            split: true,
            width: '50%',
            collapsible: true,
            collapsed: true,

            // hide the collapse tool so we don't have that ugly blank title bar
            hideCollapseTool: true,

            // Change the events which are fired (this is pretty shitty api); this transforms the container into a file container
            // (instead of an artifact container) which alters which set of tabs will show up.
            // This is required or tabs will show up which are not meant to be used with staging content (like custom metadata).
            initEventName: 'fileContainerInit',
            updateEventName: 'fileContainerUpdate'
        });

        // HACK: override tools, (AutoTabPanel via ArtifactContainer) wants to put a hidden refresh tool here which is an extra <div> with padding we don't want
        // HACK: doesn't seem to have affect when used in ctor cfg, so we force undefine all tools here
        self.artifactPanel.tools = undefined;

        // FIXME: Kill refresh button here, move "Path Lookup" to south?

        self.browserPanel = new Sonatype.repoServer.RepositoryBrowsePanel({
            payload: null,
            browseIndex: false,
            region: 'center',

            // Using a simple callback function instead of complex events
            nodeClickHandler: function (node) {
                self.selectArtifact(node);
            }
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-ContentView',
            title: 'Content',
            layout: 'border',
            hideMode: 'offsets',
            items: [
                self.browserPanel,
                self.artifactPanel
            ]
        });

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * An artifact has been selected, update the artifact panel.
     *
     * @param node
     *
     * @private
     */
    selectArtifact: function(node) {
        var self = this,
            panel = self.artifactPanel,
            attrs = node.attributes;

        if (panel.items.getCount() > 0) {
            if (node && node.isLeaf()) {
                self.logDebug('Selected artifact:', node);

                panel.updateArtifact({
                    text: attrs.text,
                    leaf: attrs.leaf,
                    resourceURI: attrs.resourceURI
                });
            }
            else {
                panel.collapsePanel();
            }
        }
    },

    /**
     * Update the repository record.
     *
     * @param repo
     */
    updateRecord: function (repo) {
        var self = this,
            sc = Sonatype.config,
            u = Sonatype.config.repos.urls;

        self.artifactPanel.collapsePanel();

        // updatePayload() wants a Ext.data.Record-like object
        self.browserPanel.updatePayload({
            data: {
                resourceURI: sc.host + u.repositories + '/' + repo.repositoryId,
                name: repo.repositoryId + " (u:" + repo.userId + ", a:" + repo.ipAddress + ")",
                repositoryURI: sc.host + u.repositories + '/' + repo.repositoryId,
                repoType: 'hosted',
                exposed: false,
                contentResourceURI: repo.repositoryURI,
                showCtx: true
            },

            // here to make more Ext.data.Record-like
            get: function (field) {
                return this.data[field];
            }
        });
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository summary details.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.SummaryView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.prefs.DateFormat',
        'Nexus.staging.Icons',
        'Nexus.staging.repository.RepositoriesMediator'
    ],

    requirejs: [
        'Nexus/messages',

        // FIXME: Use requires 'Nexus.ext.LinkButton' once that class is in properly named file/path
        'Nexus/ext/linkbutton'
    ],

    /**
     * The currently selected repository data.
     *
     * @private
     */
    current: undefined,

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.templatePanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-SummaryView-template',
            border: false
        });

        self.descriptionPanel = NX.create('Ext.form.FormPanel', {
            cls: 'nx-staging-repository-SummaryView-description',
            border: false,
            hideLabels: true,

            items: [
                {
                    name: 'description',
                    xtype: 'textarea',
                    height: 50,

                    // If 100% his gets too big and helpText icon rendered below
                    //anchor: '90%',

                    // Right edge can corrupt when using anchor and browser window resizes (textarea box grows, but does not shrink)
                    // so use a specific size, this is a little bigger than what is shown in the confirm dialog
                    width: 400,

                    helpText: 'An optional comment to describe the contents of the staging repository.'

                    // NOTE: Disabling this, I don't really care for how it looks ATM
                    //emptyText: 'Optional description or comment'
                }
            ],

            buttonAlign: 'left',
            buttons: [
                {
                    text: 'Save',
                    formBind: true,
                    scope: self,
                    handler: self.updateDescription
                },
                {
                    xtype: 'link-button',
                    text: 'Discard',
                    formBind: false,
                    scope: self,
                    handler: function() {
                        // reset to last known data
                        self.descriptionPanel.getForm().setValues(self.current);
                    }
                }
            ]
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-SummaryView',
            title: 'Summary',
            border: false,
            autoScroll: true,

            items: [
                self.templatePanel,
                {
                    xtype: 'fieldset',
                    title: 'Description',

                    items: [
                        self.descriptionPanel
                    ]
                }
            ]
        });

        self.initTemplate();

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * @private
     */
    initTemplate: function() {
        var self = this,
            icons = Nexus.staging.Icons;
        self.mainTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-body">',
                '{[ this.status(values) ]}',
                '{[ this.properties(values) ]}',
                '{[ this.message(values) ]}',
            '</div>',
            {
                compiled: true,

                link: function(url) {
                    return '<a href="' + url + '" target="_blank">' + url + '</a>';
                },

                timestamp: function(date) {
                    if (date) {
                        // Full date and time
                        return date.format(Nexus.prefs.DateFormat.forName('datetime')['long']);
                    }
                    return undefined;
                },

                status: function(values) {
                    return self.statusTpl.apply(values);
                },

                repository: function (values) {
                    return values.repositoryId + ' (' + values.profileName + ')';
                },

                owner: function (values) {
                    if (values.userId) {
                        if (values.ipAddress) {
                            return values.userId + ' (' + values.ipAddress + ')';
                        }
                        else {
                            return values.userId;
                        }
                    }
                    else {
                        return 'unknown';
                    }
                },

                activity: function(values) {
                    return self.activityTpl.apply(values);
                },

                properties: function(values) {
                    var properties;

                    properties = [
                        { name: 'Repository', value: this.repository(values) },
                        { name: 'Created', value: this.timestamp(values.created) },
                        { name: 'Updated', value: this.timestamp(values.updated) }
                    ];

                    // open and released repos hide the URL
                    if (values.type !== 'open' && values.type !== 'released') {
                        properties.push(
                            // only show URL when the repository has been closed (or its a 404)
                            { name: 'URL', value: this.link(values.repositoryURI) }
                        );
                    }

                    properties.push(
                        { name : 'Activity', value : this.activity(values) },
                        { name : 'Owner', value : this.owner(values) }
                    );

                    if (values.userAgent) {
                        properties.push(
                            { name: 'User-Agent', value: values.userAgent }
                        )
                    }

                    //fire event allowing dynamic customization of properties.
                    Nexus.staging.repository.RepositoriesMediator.fireEvent('staging.repositorySummary.configureProperties', properties, values);

                    return self.propertiesTpl.apply({
                        properties: properties
                    })
                },

                message: function(values) {
                    if (values.type === 'released') {
                        // whos your fuzzy wusszy, ya does that make you feel all nice and comfy inside, yaaaa
                        return self.messageTpl.apply({
                            icon: icons.get('star').img,
                            html: 'Staging repository has been released and can be <b>safely dropped</b>.'
                        });
                    }
                    return '';
                }
            });

        self.activityTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-activity">',
                '<span class="nx-staging-repository-SummaryView-activity-icon">',
                    '{[ this.icon(values) ]}',
                '</span>',
                '<span class="nx-staging-repository-SummaryView-activity-message">',
                    '{[ this.message(values) ]}',
                '</span>',
            '</div>',
            {
                compiled: true,

                icon: function(values) {
                    if (values.transitioning) {
                        return icons.get('transitioning').img;
                    }
                    else {
                        return icons.forNotificationCount(values.notifications).img;
                    }
                },

                message: function(values) {
                    if (values.transitioning) {
                        return 'Operation in progress';
                    }
                    else {
                        if (values.notifications === 0) {
                          return 'Last operation completed successfully';
                        }
                        else {
                          return 'Last operation failed with ' +
                              Ext.util.Format.plural(values.notifications, 'notification', 'notifications');
                        }
                    }
                }
            });

        self.statusTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-status">',
                '<div class="nx-staging-repository-SummaryView-status-icon">',
                    '{[ this.statusIcon(values) ]}',
                '</div>',
                '<div class="nx-staging-repository-SummaryView-status-label">',
                    '{[ this.statusLabel(values) ]}',
                '</div>',
            '</div>',
            {
                compiled: true,

                statusIcon: function(values) {
                    return icons.forRepository(values).variant('x32').img;
                },

                statusLabel: function(values) {
                    return values.type;
                }
            });

        self.propertiesTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-properties">',
                '<table>',
                    '<tpl for="properties">',
                        '<tr class="nx-staging-repository-SummaryView-properties-entry">',
                            '<td class="nx-staging-repository-SummaryView-properties-name">{name}</td>',
                            '<td class="nx-staging-repository-SummaryView-properties-value">{value}</td>',
                        '</tr>',
                    '</tpl>',
                '</table>',
            '</div>',
            {
                compiled: true
            });

        self.messageTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-message">',
                '<span>{icon}{html}</span>',
            '</div>',
            {
                compiled: true
            });
    },

    // TDOO: Move updateDescription to Mediator component

    /**
     * Update the description of the selected staging repository.
     *
     * @private
     */
    updateDescription: function() {
        var self = this,
            mediator = Nexus.staging.repository.RepositoriesMediator,
            repo = self.current;

        repo.description = self.descriptionPanel.getForm().findField('description').getValue();

        self.logDebug('Updating repository:', repo.repositoryId, 'description:', repo.description);

        Ext.Ajax.request({
            method: 'PUT',
            url: Sonatype.config.servicePath + '/staging/repository/' + repo.repositoryId,

            jsonData: {
                // for now only send the data back which is relevant
                repositoryId: repo.repositoryId,
                description: repo.description
            },

            scope: self,
            success: function (response, options) {
                mediator.repositoryStore.reload();

                Nexus.messages.show('Staging', 'Description saved');
            },

            failure: function (response, options) {
                self.logError('Failed to save description', response);
            }
        });
    },

    /**
     * Update details.
     *
     * @param repo
     */
    updateRecord: function (repo) {
        var self = this;

        self.current = repo;

        self.mainTpl.overwrite(self.templatePanel.body, repo);

        self.descriptionPanel.getForm().setValues(repo);

        // disable the description panel when when the repository is in transition we don't want to allow changes to it while this is happening
        // also disable the description panel when released, its pretty immutable now
        var disable = repo.transitioning === true || repo.type === 'released';

        // When disabling the entire panel it looks really horrible, so only flip off the textarea and buttons
        //self.descriptionPanel.setDisabled(disable);

        // items is a MixedCollection, use its each()
        self.descriptionPanel.items.each(function(item) {
            item.setDisabled(disable);
        });

        // buttons is an array use Ext.each()
        Ext.each(self.descriptionPanel.buttons, function(item) {
            // hide buttons don't disable
            item.setVisible(!disable);
        });
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository detail view.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.RepositoryView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.Icons',
        'Nexus.staging.repository.SummaryView',
        'Nexus.staging.repository.activity.ActivityView',
        'Nexus.staging.repository.ContentView',
        'Nexus.staging.repository.MembersView'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.summaryView = NX.create('Nexus.staging.repository.SummaryView');
        self.activityView = NX.create('Nexus.staging.repository.activity.ActivityView');
        self.contentView = NX.create('Nexus.staging.repository.ContentView');
        self.membersView = NX.create('Nexus.staging.repository.MembersView');

        self.allViews = [
            self.summaryView,
            self.activityView,
            self.contentView,
            self.membersView
        ];

        self.tabPanel = NX.create('Ext.TabPanel', {
            items: self.allViews,
            activeTab: 0,
            layoutOnTabChange: true
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-RepositoryView',
            header: true,
            border: false,
            layout: 'fit',

            items: self.tabPanel
        });

        self.constructor.superclass.initComponent.apply(self, arguments);

        // propagate 'hide' and 'deactivate' events to child views, 'show' and 'activate' handled when [re-]enabling views
        self.on('deactivate', function(panel) {
            //self.logDebug('Deactivated');
            Ext.each(self.allViews, function(view) {
                view.fireEvent('deactivate', view);
            });
        });
        self.on('hide', function(panel) {
            //self.logDebug('Hidden');
            Ext.each(self.allViews, function(view) {
                view.fireEvent('hide', view);
            });
        });

        //self.on('activate', function(panel) {
        //    self.logDebug('Activated');
        //});
        //self.on('show', function(panel) {
        //    self.logDebug('Shown');
        //});
    },

    /**
     * Enable specific views and disable all others.
     *
     * @param {Array} views   An array of view components to enable.
     *
     * @private
     */
    enableViews: function (views) {
        var self = this,
            lastActive = self.tabPanel.getActiveTab();

        if (views.length === 0) {
            throw 'At least one view must be enabled';
        }

        // Show all active views, hide others
        Ext.each(self.allViews, function (view) {
            if (views.indexOf(view) !== -1) {
                self.tabPanel.unhideTabStripItem(view);
            }
            else {
                self.tabPanel.hideTabStripItem(view);
            }
        });

        // If the active views does not contain the last active view, then reset back to first
        if (views.indexOf(lastActive) === -1) {
            self.tabPanel.setActiveTab(0);
        }
        else {
            // else inform the panel its showing and active
            lastActive.fireEvent('show', lastActive);
            lastActive.fireEvent('activate', lastActive);
        }
    },

    /**
     * Update detail display for repository record.
     *
     * @param repo
     */
    updateRecord: function (repo) {
        var self = this,
            iconCls;

        self.logGroup('Updating record');

        // summary and activity views are always enabled
        var views = [
            self.summaryView,
            self.activityView
        ];

        // extra views if repository is not transitioning
        if (!repo.transitioning) {
            // Unreleased groups need members view
            if (repo.profileType === 'group' && repo.type !== 'released') {
                views.push(self.membersView);
            }

            // Unreleased repositories need content view
            if (repo.type !== 'released' && !repo.transitioning) {
                views.push(self.contentView);
            }
        }

        iconCls = Nexus.staging.Icons.forRepository(repo).cls;
        self.setTitle(repo.repositoryId, iconCls);

        // Update enabled tabs with the currently selected data
        Ext.each(views, function (view) {
            view.updateRecord(repo);
        });

        // Change the view set
        self.enableViews(views);

        self.logGroupEnd();
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository group members.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.MembersView', {
    extend: 'Ext.tree.TreePanel',

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.store = NX.create('Ext.data.SimpleStore', {
            fields: [
                { name: 'id' },
                { name: 'name' },
                { name: 'description' }
            ]
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-MembersView',
            title: 'Members',
            layout: 'fit',
            animate: false,

            root: NX.create('Ext.tree.TreeNode', {
                text: ''
            }),
            rootVisible: false,
            enableDD: false,

            loader: NX.create('Ext.tree.SonatypeTreeLoader', {
                url: '',
                baseAttrs: {
                    expanded: true
                },
                listeners: {
                    scope: self,
                    loadexception: self.treeLoadExceptionHandler
                }
            })
        });

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * Update the repository record.
     */
    updateRecord: function (repo) {
        var self = this;

        // FIXME: Sort out how to put proper icons on to tree elements

        //noinspection JSValidateTypes
        self.setRootNode(new Ext.tree.AsyncTreeNode({
            text: repo.repositoryName,
            id: Sonatype.config.servicePath + '/staging/profile_repositories/tree/' + repo.repositoryId,
            expanded: true
        }));
    },

    /**
     * @private
     */
    treeLoadExceptionHandler: function (treeLoader, node, response) {
        var self = this,
            code = response.status,
            mb = Sonatype.MessageBox;

        if (code === 503) {
            if (mb.isVisible()) {
                mb.hide();
            }
            node.setText(node.text + ' (Out of Service)');
        }
        else if (code === 404 || code === 400) {
            if (mb.isVisible()) {
                mb.hide();
            }
            node.setText(node.text + (node.isRoot ? ' (Not Available)' : ' (Not Found)'));
        }
        else if (code === 401 || code === 403) {
            if (mb.isVisible()) {
                mb.hide();
            }
            node.setText(node.text + ' (Access Denied)');
        }
    }
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

// TODO: Move to NX core, once we are ready to use this other places, maybe tree handling still blows so this might be premature to move

/**
 * Support for {@link Ext.tree.TreeLoader} implementations.
 *
 * Not sure this is the best way to use TreeLoader, hopefully we can move on to ExtJS 4 soon,
 * which has *much more sane* data-model for trees (same as grids, etc).
 *
 * @since staging 2.3
 */
NX.define('Nexus.util.TreeLoaderSupport', {
    extend: 'Ext.tree.TreeLoader',
    
    mixins: [
        'Nexus.LogAwareMixin'
    ],

    /**
     * This mostly duplicated from super-impl so we can pass in this#suppressStatus :-(
     *
     * @override
     * @private
     */
    requestData: function (node, callback, scope) {
        if (this.fireEvent("beforeload", this, node, callback) !== false) {
            if (this.directFn) {
                var args = this.getParams(node);
                args.push(this.processDirectResponse.createDelegate(this, [
                    {callback: callback, node: node, scope: scope}
                ], true));
                this.directFn.apply(window, args);
            }
            else {
                this.transId = Ext.Ajax.request({
                    method: this.requestMethod,
                    url: this.dataUrl || this.url,
                    success: this.handleResponse,
                    failure: this.handleFailure,
                    scope: this,
                    argument: {callback: callback, node: node, scope: scope},
                    params: this.getParams(node),

                    // HACK: Pass our custom status suppression configuration
                    suppressStatus: this.suppressStatus
                });
            }
        }
        else {
            // if the load is cancelled, make sure we notify the node that we are done
            this.runCallback(callback, scope || node, []);
        }
    },

    /**
     * This mostly duplicated from super-impl so we can call #createNodes() to process node
     * creation response data which could be in any format.
     *
     * @override
     * @private
     */
    processResponse: function (response, node, callback, scope) {
        var json = response.responseText;

        try {
            var o = response.responseData || Ext.decode(json);
            node.beginUpdate();

            // HACK: Delegate to createNodes()
            this.createNodes(node, o);

            node.endUpdate();
            this.runCallback(callback, scope || node, [node]);
        }
        catch (e) {
            this.logError('Node creation failed', e);
            this.handleFailure(response);

            // If we do not throw here failures when loading the detail will be lost!
            throw e;
        }
    },

    /**
     * Create nodes from the given response data.
     *
     * @protected
     *
     * @param node
     * @param data
     */
    createNodes: function(node, data) {
        throw new Error('abstract');
    }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/

// FIXME: Rename this module/class

define('Sonatype/repoServer/StagingPanel', function() {

    Sonatype.repoServer.StagingPanel = function(cfg) {
      var
            i, sp = Sonatype.lib.Permissions, toolbar = [],
            updateGridView,
            _this = this,
            config = cfg || {},
            defaultConfig = {
              title : 'Staging Profiles'
            };
      Ext.apply(this, config, defaultConfig);

      this.targetStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'resourceURI',
            fields : [{
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }, {
                  name : 'contentClass'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.repoTargets,
            autoLoad : false,
            listeners : {
              load : {
                fn : function(store, recs, opts) {
                  for (i = recs.length - 1; i >= 0; i--)
                  {
                    var rec = recs[i];
                    if (rec.get('contentClass') != 'maven1' && rec.get('contentClass') != 'maven2')
                    {
                      store.remove(rec);
                    }
                  }
                },
                scope : this
              }
            }
          });

      this.groupStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [{
                  name : 'resourceURI'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }, {
                  name : 'format'
                }, {
                  name : 'userManaged'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.groups,
            autoLoad : false,
            listeners : {
              load : {
                fn : function(store, recs, opts) {
                  for (var i = recs.length - 1; i >= 0; i--)
                  {
                    var rec = recs[i];
                    if (!rec.get('userManaged'))
                    {
                      store.remove(rec);
                    }
                  }
                },
                scope : this
              }
            }
          });

      this.repoStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [{
                  name : 'resourceURI'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }, {
                  name : 'format'
                }, {
                  name : 'repoType'
                }, {
                  name : 'repoPolicy'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.repositories,
            autoLoad : false
          });

      this.modeStore = new Ext.data.SimpleStore({
            fields : ['value', 'display'],
            data : [['BOTH', 'Deploy and UI Upload'], ['DEPLOY', 'Deploy'], ['UPLOAD', 'UI Upload']]
          });

      this.fullModeStore = new Ext.data.SimpleStore({
            fields : ['value', 'display'],
            data : [['BOTH', 'Deploy and UI Upload'], ['DEPLOY', 'Deploy'], ['UPLOAD', 'UI Upload'], ['GROUP', 'Group']]
          });

      this.joinGroups = function(value, parent) {
        var s = "";
        for (var i = 0; i < value.length; i++)
        {
          if (i > 0)
          {
            s += ', ';
          }
          if (value[i])
          {
            var rec = this.groupStore.getAt(this.groupStore.find('id', value[i]));
            if (rec)
            {
              s += rec.get('name');
            }
          }
        }
        return s;
      };

      this.getRepositoryTarget = function(val, parent) {
        if (val)
        {
          var rec = this.targetStore.getAt(this.targetStore.find('id', val));
          if (rec)
          {
            return rec.get('name');
          }
        }
        return '';
      };

      this.getRepoName = function(value, parent) {
        if (value)
        {
          var rec = this.repoStore.getAt(this.repoStore.find('id', new RegExp('^' + value + '$')));
          if (rec)
          {
            return rec.get('name');
          }
        }
        return '';
      }

      this.getModeName = function(value, parent) {
        if (value)
        {
          var rec = this.fullModeStore.getAt(this.fullModeStore.find('value', value));
          if (rec)
          {
            return rec.get('display');
          }
        }
        return '';
      }

      if (sp.checkPermission('nexus:stagingprofileorder', sp.CREATE))
      {
        toolbar.push({
              id : 'profile-move-up-button',
              text : 'Move Up',
              icon : Sonatype.config.resourcePath + '/static/images/move-up.gif',
              cls : 'x-btn-text-icon',
              tooltip : {
                title : 'Profile Ordering',
                text : 'Move the profile up in the list priority.'
              },
              handler : this.moveUpResourceHandler,
              scope : this
            }, {
              id : 'profile-move-down-button',
              text : 'Move Down',
              icon : Sonatype.config.resourcePath + '/static/images/move-down.gif',
              cls : 'x-btn-text-icon',
              tooltip : {
                title : 'Profile Ordering',
                text : 'Move the profile down in the list priority.'
              },
              handler : this.moveDownResourceHandler,
              scope : this
            }, {
              id : 'profile-save-order-button',
              text : 'Save Order',
              icon : Sonatype.config.extPath + '/resources/images/default/dd/drop-yes.gif',
              cls : 'x-btn-text-icon',
              tooltip : {
                title : 'Profile Ordering',
                text : 'Save the current profile priority order.'
              },
              handler : this.saveOrderResourceHandler,
              scope : this
            });
      }

      /* NXCM-4478 begin
       * Prevent loading of main dataStore before dependent stores are loaded
       *
       * FIXME this should rather be implemented in the Sonatype.panels.GridViewer itself (superclass)
       */
      var mask;
      var latch = 0;
      var dataStoreLoader = function() {
        // only load if all dependent stores are already loaded
        latch = latch + 1;
        if ( latch >= _this.dataStores.length ) {
          _this.dataStore.load();
        }
      };

      var dataStores = [this.targetStore, this.groupStore, this.repoStore];
      for (var j = 0; j < dataStores.length; j++) {
        var store = dataStores[j];
        store.on('load', dataStoreLoader, _this, {single:true});
      }
      /* NXCM-4478 end*/

      Sonatype.repoServer.StagingPanel.superclass.constructor.call(this, {
            dataStores : dataStores,
            addMenuInitEvent : 'profileAddMenuInit',
            deleteButton : sp.checkPermission('nexus:stagingprofiles', sp.DELETE),
            rowClickEvent : 'profileViewInit',
            url : Sonatype.config.servicePath + '/staging/profiles',
            dataAutoLoad : false,
            tabbedChildren : true,
            tbar : toolbar,
            dataSortInfo : {
              field : 'order',
              direction : 'asc'
            },
            columns : [{
                  name : 'resourceURI'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  header : 'Name',
                  width : 100,
                  renderer : function(value, metadata, record, rowIndex, colIndex, store) {
                    return record.get('mode') == 'GROUP' ? ('<b>' + value + '</b>') : value;
                  }
                }, {
                  name : 'mode'
                }, {
                  name : 'sMode',
                  header : 'Mode',
                  width : 150,
                  mapping : 'mode',
                  convert : this.getModeName.createDelegate(this)
                }, {
                  name : 'repositoryTargetId'
                }, {
                  name : 'sTarget',
                  header : 'Repo Target',
                  width : 100,
                  mapping : 'repositoryTargetId',
                  convert : this.getRepositoryTarget.createDelegate(this)
                }, {
                  name : 'currentRepositoryIds'
                },
                {
                  name : 'order',
                  sortType : Ext.data.SortTypes.asInt
                }, {
                  name : 'finishNotifyRoles'
                }, {
                  name : 'promotionNotifyRoles'
                }, {
                  name : 'promotionTargetRepository'
                }, {
                  name : 'deployURI'
                }, {
                  name : 'sPromotionTargetRepository',
                  header : 'Release Repository',
                  width : 200,
                  mapping : 'promotionTargetRepository',
                  convert : this.getRepoName.createDelegate(this)
                }, {
                  name : 'targetGroups'
                }, {
                  name : 'sTargetGroups',
                  header : 'Target Groups',
                  width : 150,
                  mapping : 'targetGroups',
                  convert : this.joinGroups.createDelegate(this),
                  autoExpand : true
                }
              , {
                  name: 'properties'
                }
            ]
          });

      /* NXCM-4478 begin
       * Prevent loading of main dataStore before dependent stores are loaded
       */
      // delay this listener to have the panel completely rendered before applying the loadmask
      // (ExtJS bug, event 'afterrender' not working)
      this.gridPanel.on('render', function() {
        if (!this.dataStore.isLoaded) {
          mask = new Ext.LoadMask(_this.gridPanel.el);
          mask.show();
        }
      }, _this, {single:true, delay : 100});


    //  _this.gridPanel.loadMask.show();
      this.dataStore.on('load', function() {
        if(mask) {
          mask.hide();
        }
      });
      /* NXCM-4478 end*/
    };

    Ext.extend(Sonatype.repoServer.StagingPanel, Sonatype.panels.GridViewer, {
          emptyText: 'No profiles defined',
          emptyTextWhileFiltering: 'No profiles matched criteria: {criteria}',

          moveUpResourceHandler : function() {
            if (this.gridPanel.getSelectionModel().hasSelection())
            {
              var rec = this.gridPanel.getSelectionModel().getSelected();

              if (rec)
              {
                var i = this.gridPanel.getStore().indexOf(rec);

                if (i > 0)
                {
                  this.gridPanel.getStore().remove(rec);
                  this.gridPanel.getStore().insert(i - 1, rec);
                  this.gridPanel.getSelectionModel().selectRecords([rec], false);
                }
              }
            }
          },
          moveDownResourceHandler : function() {
            if (this.gridPanel.getSelectionModel().hasSelection())
            {
              var rec = this.gridPanel.getSelectionModel().getSelected();

              if (rec)
              {
                var i = this.gridPanel.getStore().indexOf(rec);

                if (i < (this.gridPanel.getStore().getCount() - 1))
                {
                  this.gridPanel.getStore().remove(rec);
                  this.gridPanel.getStore().insert(i + 1, rec);
                  this.gridPanel.getSelectionModel().selectRecords([rec], false);
                }
              }
            }
          },
          saveOrderResourceHandler : function() {
            var idList = [];
            for (var i = 0; i < this.gridPanel.getStore().getCount(); i++)
            {
              var rec = this.gridPanel.getStore().getAt(i);

              if (rec.data.resourceURI != 'new')
              {
                idList[idList.length] = rec.data.id;
              }
            }

            Sonatype.utils.defaultToNo();
            Sonatype.MessageBox.show({
                  animEl : this.gridPanel.getEl(),
                  title : 'Save Profile Order',
                  msg : 'Save the current order of the Profiles?',
                  buttons : Sonatype.MessageBox.YESNO,
                  scope : this,
                  icon : Sonatype.MessageBox.QUESTION,
                  fn : function(btnName) {
                    if (btnName == 'yes' || btnName == 'ok')
                    {
                      Ext.Ajax.request({
                            callback : this.saveOrderCallback,
                            scope : this,
                            method : 'POST',
                            url : Sonatype.config.servicePath + '/staging/profile_order',
                            jsonData : {
                              data : idList
                            }
                          });
                    }
                  }
                });
          },
          saveOrderCallback : function(options, isSuccess, response) {
            if (!isSuccess)
            {
              Sonatype.utils.connectionError(response, 'The server did not save the order of the profiles.');
            }
          }
        });

    Sonatype.Events.addListener('nexusNavigationPostInit', function(nexusPanel) {

          nexusPanel.insert(2,{
                title : 'Build Promotion',
                id : 'st-enterprise-staging'
              });

          nexusPanel.add({
                enabled : Sonatype.lib.Permissions.checkPermission('nexus:stagingprofiles', Sonatype.lib.Permissions.READ),
                sectionId : 'st-enterprise-staging',
                title : 'Staging Profiles',
                tabId : 'stagingProfiles',
                tabCode : Sonatype.repoServer.StagingPanel
              });
        });

    Sonatype.Events.addListener('profileAddMenuInit', function(menu) {
          var sp = Sonatype.lib.Permissions;

          if (sp.checkPermission('nexus:stagingprofiles', sp.CREATE))
          {
            var createProfileFunc = function(container, rec, item, e) {
              rec.beginEdit();
              if (item.groupMode)
              {
                rec.set('mode', 'GROUP');
              }
              rec.set('resourceURI', 'new');
              rec.set('targetGroups', []);
              rec.set('finishNotifyRoles', []);
              rec.set('promotionNotifyRoles', []);
              rec.set('properties', {});
              rec.commit();
              rec.endEdit();
            };

            menu.add(['-', {
                  text : 'Staging Profile',
                  autoCreateNewRecord : true,
                  handler : createProfileFunc
                }]);

            menu.add(['-', {
                  text : 'Build Promotion Profile',
                  autoCreateNewRecord : true,
                  handler : createProfileFunc,
                  groupMode : true
                }]);
          }
        });

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/

define('Sonatype/repoServer/ProfileEditor', function() {

    Sonatype.repoServer.ProfileEditor = function(cfg) {
      var config = cfg || {},
        defaultConfig = {
        uri : Sonatype.config.servicePath + '/staging/profiles',
        resetButton : true,
        defaultTimeoutValues : {},
        dataModifiers : {
          load : {
            finishNotifyRoles : function(arr, srcObj, fpanel) {
              fpanel.find('name', 'finishRoleManager')[0].setSelectedRoleIds(arr, true);
              return arr;
            },
            promotionNotifyRoles : function(arr, srcObj, fpanel) {
              fpanel.find('name', 'promotionRoleManager')[0].setSelectedRoleIds(arr, true);
              return arr;
            },
            dropNotifyRoles : function(arr, srcObj, fpanel) {
              fpanel.find('name', 'dropRoleManager')[0].setSelectedRoleIds(arr, true);
              return arr;
            },
            targetGroups : this.importGroupsHelper.createDelegate(this),
            closeRuleSets : this.importCloseRuleSetsHelper.createDelegate(this),
            promoteRuleSets : this.importPromoteRuleSetsHelper.createDelegate(this)
          },
          submit : {
            finishNotifyRoles : function(value, fpanel) {
              return fpanel.find('name', 'finishRoleManager')[0].getSelectedRoleIds();
            },
            promotionNotifyRoles : function(value, fpanel) {
              return fpanel.find('name', 'promotionRoleManager')[0].getSelectedRoleIds();
            },
            dropNotifyRoles : function(value, fpanel) {
              return fpanel.find('name', 'dropRoleManager')[0].getSelectedRoleIds();
            },
            targetGroups : this.exportGroupsHelper.createDelegate(this),
            closeRuleSets : this.exportCloseRuleSetsHelper.createDelegate(this),
            promoteRuleSets : this.exportPromoteRuleSetsHelper.createDelegate(this),
            autoStagingDisabled : function(value, fpanel) {
              if ( value === true || value === false ) {
                return value;
              } else {
                return 'true' === value;
              }
            },
            repositoriesSearchable : function(value, fpanel) {
              if ( value === true || value === false ) {
                return value;
              } else {
                return 'false' === value;
              }
            }
          }
        },
        validationModifiers : {
          targetGroups: function(error, fpanel) {
            fpanel.find('name', 'targetGroups')[0].markTreeInvalid(null, error.msg);
          }
        },
        referenceData : {
          id : "",
          name : "",
          repositoryTemplateId : "",
          repositoryType : "",
          repositoryTargetId : "",
          targetGroups : [],
          finishNotifyRoles : [],
          promotionNotifyRoles : [],
          dropNotifyRoles : [],
          finishNotifyEmails : "",
          promotionNotifyEmails : "",
          dropNotifyEmails : "",
          closeRuleSets : [],
          promoteRuleSets : [],
          promotionTargetRepository : "",
          dropNotifyCreator : true,
          finishNotifyCreator : true,
          promotionNotifyCreator : true,
          autoStagingDisabled : false,
          repositoriesSearchable : true,
          mode : "",
          properties: {}
        }
      };
      Ext.apply(this, config, defaultConfig);

      this.currentTemplate = undefined;

      this.selectionStrategyStore = new Ext.data.SimpleStore({
        fields : ['value', 'display'],
        data : [[false, 'Explicit or Implicit - selected by ID or Nexus will calculate match'], [true, 'Explicit Only - only selected by ID']]
      });

      this.templateStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [{
                  name : 'resourceURI'
                }, {
                  name : 'repoType'
                }, {
                  name : 'repoPolicy'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString,
                  convert : this.calculateTemplateName
                }, {
                  name : 'format'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.repoTemplates,
            autoLoad : false,
            listeners : {
              load : {
                fn : function(store, records, options) {
                  Sonatype.repoServer.ProfileEditor.prototype.repositoryTemplateFilter(store);

                  var
                        combo = this.find('name', 'repositoryTemplateId')[0],
                        nameField = this.find('name', 'name')[0];

                  combo.doQuery(combo.allQuery, true);
                  nameField.focus();
                },
                scope : this
              }
            }
          });

      this.rulesetStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [{
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.servicePath + '/staging/rule_sets',
            autoLoad : false
          });

      this.repoTypeStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'roleHint',
            fields : [{
                  name : 'description'
                }, {
                  name : 'roleHint',
                  sortType : Ext.data.SortTypes.asUCString
                }],
            sortInfo : {
              field : 'description',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.repoTypes,
            autoLoad : false
          });

      this.checkPayload();

      this.COMBO_WIDTH = 376;

      this.items = [];
      if (!this.isNew)
      {
        if (!this.groupProfile)
        {
          this.items.push({
              xtype : 'panel',
              layout : 'column',
              items : [{
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : 'Profile ID',
                    height : 21,
                    width : 205
                  }, {
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : this.payload.get('id')
                  }]
          });
          this.items.push({
              xtype : 'panel',
              layout : 'column',
              items : [{
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : 'Deploy URL',
                    height : 21,
                    width : 205
                  }, {
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : this.payload.get('deployURI')
                  }]
          });
        }
        else
        {
          this.items.push({
              xtype : 'panel',
              layout : 'column',
              items : [{
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : 'Profile ID',
                    height : 21,
                    width : 205
                  }, {
                    xtype : 'panel',
                    cls: 'x-form-item',
                    html : this.payload.get('id')
                  }]
          });
        }
      }

      this.items.push({
            xtype : 'textfield',
            fieldLabel : 'Profile Name',
            itemCls : 'required-field',
            helpText : 'Staging profile name.',
            name : 'name',
            width : this.COMBO_WIDTH,
            allowBlank : false,
            disabled: !this.isNew
          });

      if (this.groupProfile)
      {
        this.items.push({
              xtype : 'textfield',
              fieldLabel : 'Staging Mode',
              itemCls : 'required-field',
              helpText : 'The source(s) this profile will accept artifacts from, either from the staging deploy URL, or staging upload via the UI, or both.',
              name : 'modeInput',
              width : this.COMBO_WIDTH,
              allowBlank : false,
              disabled : true
            }, {
              xtype : 'hidden',
              name : 'mode'
            }, {
              xtype : 'hidden',
              value : false,
              name : 'autoStagingDisabled'
            }, {
              xtype : 'hidden',
              value : true,
              name : 'repositoriesSearchable'
            });
      }
      else
      {
        this.items.push({
          xtype : 'combo',
          fieldLabel : 'Profile Selection Strategy',
          itemCls : 'required-field',
          helpText : '\'Explicit or Implicit\' will include this profile as a possible match using user agent, IP address, username and repository target if the profile ID is not specified explicitly. \'Explicit Only\' means this profile will only be selected when identified by profile ID.',
          name : 'autoStagingDisabledComboName',
          hiddenName : 'autoStagingDisabled',
          store : this.selectionStrategyStore,
          displayField : 'display',
          valueField : 'value',
          width : this.COMBO_WIDTH,
          listWidth : this.COMBO_WIDTH,
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          allowBlank : false
        });

        this.items.push({
          xtype : 'checkbox',
          checked : true,
          fieldLabel : 'Searchable Repositories',
          itemCls : 'required-field',
          helpText : 'When enabled, repositories created using this profile are indexed and searchable. Setting applies only to new repositories.',
          name : 'repositoriesSearchable',
          width : this.COMBO_WIDTH
        });

        this.items.push({
              xtype : 'combo',
              fieldLabel : 'Staging Mode',
              itemCls : 'required-field',
              helpText : 'The source(s) this profile will accept artifacts from, either from the staging deploy URL, or staging upload via the UI, or both.',
              name : 'mode',
              store : this.modeStore,
              displayField : 'display',
              valueField : 'value',
              width : this.COMBO_WIDTH,
              listWidth : this.COMBO_WIDTH,
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false
            });
      }

      this.items.push({
            xtype : 'combo',
            fieldLabel : 'Template',
            itemCls : 'required-field',
            helpText : 'Template used to create staging repositories.',
            name : 'repositoryTemplateId',
            width : this.COMBO_WIDTH,
            listWidth : this.COMBO_WIDTH,
            store : this.templateStore,
            displayField : 'name',
            valueField : 'id',
            editable : false,
            forceSelection : true,
            mode : 'local',
            triggerAction : 'all',
            emptyText : 'Select...',
            selectOnFocus : true,
            allowBlank : false,
            listeners : {
              select : {
                fn : this.templateSelectHandler,
                scope : this
              }
            }
          });

      if (!this.groupProfile)
      {
        this.items.push({
              xtype : 'combo',
              fieldLabel : 'Repository Target',
              itemCls : 'required-field',
              helpText : 'Repository target that will cause this profile to be selected.',
              name : 'repositoryTargetId',
              store : this.targetStore,
              displayField : 'name',
              valueField : 'id',
              width : this.COMBO_WIDTH,
              listWidth : this.COMBO_WIDTH,
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              lastQuery : ''
            });
        this.items.push({
              xtype : 'combo',
              fieldLabel : 'Release Repository',
              helpText : 'A Repository that will be used to release all staged repositories to.',
              name : 'promotionTargetRepository',
              width : this.COMBO_WIDTH,
              itemCls : 'required-field',
              listWidth : this.COMBO_WIDTH,
              store : this.filteredRepoStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : false,
              mode : 'local',
              required : true,
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              lastQuery : '',
              clearFilterOnReset: false
            });
      }

      // Allow contributors to add to the form. They will also be responsible for ensuring that this additional data gets serialized into the form appropriately.
      Nexus.staging.repository.RepositoriesMediator.fireEvent('staging.profile.loaded', this.items, this.payload.get('properties'), 'properties');

      this.items.push({
            xtype : 'textfield',
            fieldLabel : 'Content Type',
            itemCls : 'required-field',
            helpText : 'The type of the generated staging repositories (e.g. "maven1", "maven2").',
            name : 'repositoryType',
            width : this.COMBO_WIDTH,
            allowBlank : false,
            disabled : true
          }, {
            xtype : 'twinpanelchooser',
            titleLeft : 'Target Groups',
            titleRight : 'Available Groups',
            name : 'targetGroups',
            valueField : 'id',
            store : this.groupStore,
            halfSize : true,
            required : true,
            validateLeftItems: true,
            validateLeftItemsText : "Invalid target group selected."
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Close Repository Notification Settings',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'textfield',
                  fieldLabel : 'Email Addresses',
                  helpText : 'Comma seperated list of email addresses to send notifications to.',
                  name : 'finishNotifyEmails',
                  width : this.COMBO_WIDTH
                }, {
                  xtype : 'rolemanager',
                  name : 'finishRoleManager',
                  height : 200,
                  width : 581,
                  usePrivileges : false,
                  doValidation : false,
                  style : 'margin-top: 10px;border: 1px solid #B5B8C8;'
                }, {
                  xtype : 'checkbox',
                  checked : true,
                  fieldLabel : 'Notify creator',
                  helpText : 'Notify the repository creator (the user who uploaded the repository content) if the repository gets closed.',
                  name : 'finishNotifyCreator',
                  width : this.COMBO_WIDTH
                }]
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Promote Repository Notification Settings',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'textfield',
                  fieldLabel : 'Email Addresses',
                  helpText : 'Comma seperated list of email addresses to send notifications to.',
                  name : 'promotionNotifyEmails',
                  width : this.COMBO_WIDTH
                }, {
                  xtype : 'rolemanager',
                  name : 'promotionRoleManager',
                  height : 200,
                  width : 581,
                  usePrivileges : false,
                  doValidation : false,
                  style : 'margin-top: 10px;border: 1px solid #B5B8C8;'
                }, {
                  xtype : 'checkbox',
                  checked : true,
                  fieldLabel : 'Notify creator',
                  helpText : 'Notify the repository creator (the user who uploaded the repository content) if the repository gets promoted.',
                  name : 'promotionNotifyCreator',
                  width : this.COMBO_WIDTH
                }]
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Drop Repository Notification Settings',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'textfield',
                  fieldLabel : 'Email Addresses',
                  helpText : 'Comma seperated list of email addresses to send notifications to.',
                  name : 'dropNotifyEmails',
                  width : this.COMBO_WIDTH
                }, {
                  xtype : 'rolemanager',
                  name : 'dropRoleManager',
                  height : 200,
                  width : 581,
                  usePrivileges : false,
                  doValidation : false,
                  style : 'margin-top: 10px;border: 1px solid #B5B8C8;'
                }, {
                  xtype : 'checkbox',
                  checked : true,
                  fieldLabel : 'Notify creator',
                  helpText : 'Notify the repository creator (the user who uploaded the repository content) if the repository gets dropped.',
                  name : 'dropNotifyCreator',
                  width : this.COMBO_WIDTH
                }]
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Close Repository Staging Rulesets',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'twinpanelchooser',
                  titleLeft : 'Rulesets',
                  titleRight : 'Available Rulesets',
                  name : 'closeRuleSets',
                  valueField : 'id',
                  store : this.rulesetStore,
                  halfSize : true
                }]
          }, {
            xtype : 'fieldset',
            checkboxToggle : false,
            title : 'Promote Repository Staging Rulesets',
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : true,
            autoHeight : true,
            layoutConfig : {
              labelSeparator : ''
            },
            items : [{
                  xtype : 'twinpanelchooser',
                  titleLeft : 'Rulesets',
                  titleRight : 'Available Rulesets',
                  name : 'promoteRuleSets',
                  valueField : 'id',
                  store : this.rulesetStore,
                  halfSize : true
                }]
          }, {
            xtype : 'hidden',
            name : 'id'
          });

      Sonatype.repoServer.ProfileEditor.superclass.constructor.call(this, {
            dataStores : [this.templateStore, this.rulesetStore, this.repoTypeStore],
            listeners : {
              submit : {
                fn : this.submitHandler,
                scope : this
              },
              load : {
                fn : this.loadHandler,
                scope : this
              }
            }
          });
    };

    Ext.extend(Sonatype.repoServer.ProfileEditor, Sonatype.ext.FormPanel, {
      saveHandler : function( button, event ){
        // Allow contributors to enhance the data before serialization.
        Nexus.staging.repository.RepositoriesMediator.fireEvent('staging.profile.submitted', this.referenceData.properties);

        if (this.isValid())
        {
          this.form.doAction('sonatypeSubmit', {
            method : this.getSaveMethod(),
            url : this.getActionURL(),
            waitMsg : this.isNew ? 'Creating a new record...' : 'Updating records...',
            fpanel : this,
            validationModifiers : this.validationModifiers,
            dataModifiers : this.dataModifiers ? this.dataModifiers.submit : {},
            serviceDataObj : this.referenceData,
            isNew : this.isNew
          });
        }
      },

          loadData : function() {
            if (this.isNew)
            {
              this.form.reset();
              this.fireEvent('load', this.form);
            }
            else
            {
              this.form.doAction('sonatypeLoad', {
                    url : this.getActionURL(),
                    method : 'GET',
                    fpanel : this,
                    dataModifiers : this.dataModifiers.load,
                    scope : this
                  });
            }
          },
          isValid : function() {
            return this.form.isValid() && this.find('name', 'targetGroups')[0].validate();
          },
          loadHandler : function(form, action, receivedData) {
            if (this.groupProfile)
            {
              this.find('name', 'modeInput')[0].setRawValue('Group');
              this.find('name', 'mode')[0].setRawValue('GROUP');
              this.templateStore.filterBy(function(rec, id) {
                    return rec.get('repoType') === 'group';
                  });
            }
            else
            {
              this.templateStore.filterBy(function(rec, id) {
                    return rec.get('repoType') === 'hosted' && (rec.get('format') === 'maven1' || rec.get('format') === 'maven2');
                  });

              // If there is data received (ie, not a newly added record, then do some filtering ( https://issues.sonatype.org/browse/NXCM-3852 )
              if (receivedData) {
                  var templateRec = this.templateStore.getById(receivedData.repositoryTemplateId);
                  this.filterTemplateSelectHandler(templateRec);
              }
            }

          },
          submitHandler : function(form, action, receivedData) {
            if (this.isNew)
            {
              receivedData.sTargetGroups = this.joinGroups(receivedData.targetGroups);
              receivedData.sTarget = this.getRepositoryTarget(receivedData.repositoryTargetId);
              receivedData.sPromotionTargetRepository = this.getRepoName(receivedData.promotionTargetRepository);
              receivedData.sMode = this.getModeName(receivedData.mode);
              return;
            }

            var rec = this.payload;
            rec.beginEdit();
            rec.set('name', receivedData.name);
            rec.set('repositoryTemplateId', receivedData.repositoryTemplateId);
            rec.set('repositoryType', receivedData.repositoryType);
            rec.set('targetGroups', receivedData.targetGroups);
            rec.set('sTargetGroups', this.joinGroups(receivedData.targetGroups));
            rec.set('currentRepositoryIds', receivedData.currentRepositoryIds);
            rec.set('repositoryTargetId', receivedData.repositoryTargetId);
            rec.set('sTarget', this.getRepositoryTarget(receivedData.repositoryTargetId));
            rec.set('finishNotifyRoles', receivedData.finishNotifyRoles);
            rec.set('promotionNotifyRoles', receivedData.promotionNotifyRoles);
            rec.set('dropNotifyRoles', receivedData.dropNotifyRoles);
            rec.set('closeRuleSets', receivedData.closeRuleSets);
            rec.set('promoteRuleSets', receivedData.promoteRuleSets);
            rec.set('promotionTargetRepository', receivedData.promotionTargetRepository);
            rec.set('sPromotionTargetRepository', this.getRepoName(receivedData.promotionTargetRepository));
            rec.set('mode', receivedData.mode);
            rec.set('sMode', this.getModeName(receivedData.mode));
            rec.set('deployURI', receivedData.deployURI);
            rec.set('properties', receivedData.properties)
            rec.commit();
            rec.endEdit();
          },
          exportGroupsHelper : function(value, formPanel) {
            return formPanel.find('name', 'targetGroups')[0].getValue();
          },
          importGroupsHelper : function(value, parent, formPanel) {
            formPanel.find('name', 'targetGroups')[0].setValue(value);
            return value;
          },
          importCloseRuleSetsHelper : function(value, parent, formPanel) {
            formPanel.find('name', 'closeRuleSets')[0].setValue(value);
            return value;
          },
          exportCloseRuleSetsHelper : function(value, formPanel) {
            return formPanel.find('name', 'closeRuleSets')[0].getValue();
          },
          importPromoteRuleSetsHelper : function(value, parent, formPanel) {
            formPanel.find('name', 'promoteRuleSets')[0].setValue(value);
            return value;
          },
          exportPromoteRuleSetsHelper : function(value, formPanel) {
            return formPanel.find('name', 'promoteRuleSets')[0].getValue();
          },
          filterTemplateSelectHandler : function(record) {
            this.currentTemplate = record.get('id');

            var format = record.get('format');
            if (this.payload.get('mode') !== 'GROUP')
            {
              // filter the targets
              this.targetStore.filterBy(function(rec, id) {
                    return rec.get('contentClass') === format;
                  });

              // filter the promotion repos
              this.filteredRepoStore.filterBy(function(rec, id) {
                    return rec.get('format') === format;
                  });

              // filter the target groups
              this.groupStore.filterBy(function(rec, id) {
                    return rec.get('format') === format;
                  });
            }
          },
          templateSelectHandler : function(combo, record, index) {
            var
                  reset = this.currentTemplate !== record.get('id'),
                  groupsField = this.find('name', 'targetGroups')[0],
                  format = record.get('format');

            this.filterTemplateSelectHandler(record);

            if (reset && this.payload.get('mode') !== 'GROUP')
            {
              this.find('name', 'repositoryTargetId')[0].reset();
              this.find('name', 'promotionTargetRepository')[0].reset();
            }

            if(reset) {
              groupsField.setValue([]);
              groupsField.loadStore();
              groupsField.clearInvalid();
            }

            // set the content type
            this.find('name', 'repositoryType')[0].setRawValue(format);
          },
          // Reverted NXCM-2691 (Add support for staging snapshots)
          repositoryTemplateFilter : function(store) {
            var rec, policy, i, records;
            records = store.getRange();
            for (i = records.length - 1; i >= 0; i = i - 1)
            {
              rec = records[i];
              policy = rec.get('repoPolicy');
              // NXCM-3548 only accept maven release repositories
              if (!(rec.get('format') === 'maven2' && rec.get('repoType') !== 'proxy' && (policy === 'RELEASE' || policy === ''))) {
                store.remove(rec);
              }
            }
          }
        });

    Sonatype.Events.addListener('profileViewInit', function(cardPanel, rec, gridPanel) {
          var
                sp = Sonatype.lib.Permissions,
                data = [],
                repoData,filteredRepoStore,
                i, repoRec;

          if (sp.checkPermission('nexus:stagingprofiles', sp.READ) && (sp.checkPermission('nexus:stagingprofiles', sp.CREATE) || sp.checkPermission('nexus:stagingprofiles', sp.DELETE) || sp.checkPermission('nexus:stagingprofiles', sp.EDIT)))
          {
            for (i = 0; i < gridPanel.repoStore.getCount(); i = i + 1)
            {
              repoRec = gridPanel.repoStore.getAt(i);

              // Reverted NXCM-2691 (Add support for staging snapshots)
              if (repoRec.get('repoType') === 'hosted' && repoRec.get('repoPolicy') !== 'SNAPSHOT')
              {
                repoData = [repoRec.get('id'), repoRec.get('name'), repoRec.get('format')];
                data.push(repoData);
              }
            }

            filteredRepoStore = new Ext.data.SimpleStore({
                  fields : [{
                        name : 'id'
                      }, {
                        name : 'name'
                      }, {
                        name : 'format'
                      }, {
                        name : 'repoPolicy'
                      }],
                  data : data
                });

            cardPanel.add(new Sonatype.repoServer.ProfileEditor({
                  name : 'configuration',
                  groupStore : gridPanel.groupStore,
                  targetStore : gridPanel.targetStore,
                  repoStore : gridPanel.repoStore,
                  filteredRepoStore : filteredRepoStore,
                  joinGroups : gridPanel.joinGroups,
                  modeStore : gridPanel.modeStore,
                  fullModeStore : gridPanel.fullModeStore,
                  getRepositoryTarget : gridPanel.getRepositoryTarget,
                  getRepoName : gridPanel.getRepoName,
                  getModeName : gridPanel.getModeName,
                  tabTitle : 'Configuration',
                  payload : rec,
                  groupProfile : rec.get('mode') === 'GROUP'
                }));
          }
        });

});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus, define*/

define('Sonatype/repoServer/StagingUploadPanel', ['Nexus/staging/Rules'], function() {

    Sonatype.repoServer.StagingUploadPanel = function(config) {
      var config = config || {};

      var availableModes =[];
      if(Sonatype.lib.Permissions.checkPermission('nexus:stagingupload', Sonatype.lib.Permissions.CREATE) )
      {
        availableModes.push(['pom', 'Artifact(s) with a POM']);
        availableModes.push(['gav', 'Artifact(s) with GAV']);
      }

      if (Sonatype.lib.Permissions.checkPermission('nexus:stagingbundleupload', Sonatype.lib.Permissions.CREATE))
      {
        availableModes.push(['bundle', 'Artifact Bundle']);
      }

      var defaultConfig = {
        id : 'staging-upload',
        uploadModeTitle : 'Select Staging Upload Mode',
        uploadModeLabel : 'Upload Mode',
        uploadModeStore : new Ext.data.SimpleStore({
              fields : ['value', 'display'],
              data : availableModes
            }),
        uploadModeHt : 'The mode of staging upload, which depends on how you want to deploy your artifacts.',
        uploadModeExplanation : '<b>Artifact(s) with a POM:</b> GAV will be defined from a POM file.<br/>' + '<b>Artifact(s) with GAV:</b> GAV needs to be manually defined.<br/>'
            + '<b>Artifact Bundle:</b> A bundle file produced by <a href="http://maven.apache.org/plugins/maven-repository-plugin/">Maven Repository Plugin</a>, which should contain the POM.',
        extraItems : {
          xtype : 'textarea',
          fieldLabel : 'Description',
          helpText : 'A detailed description of the staged repository',
          anchor : Sonatype.view.FIELD_OFFSET,
          name : 'description',
          allowBlank : false,
          itemCls : 'required-field'
        }
      };
      Ext.apply(this, config, defaultConfig);

      // need to set invalid repo id, otherwise parent panel will fail
      // also setting to release policy, so validation will work properly
      this.payload = {
        id : 'invalid-repo-id',
        data : {
          repoPolicy : 'release'
        }
      };

      Sonatype.repoServer.StagingUploadPanel.superclass.constructor.call(this, {});

      this.on('beforerender', this.beforeRenderHandler, this);
    };

    Ext.extend(Sonatype.repoServer.StagingUploadPanel, Sonatype.repoServer.ArtifactUploadPanel, {
          createUploadForm : function(treePanel, fileInput, classifier, extension, lastItem) {
            var pomMode = this.find('name', 'gavDefinition')[0].getValue() == 'pom', tmpFormChildren = [];

            if (this.gavResponse)
            {
              pomMode = false;
              this.form.findField('g').setValue(this.gavResponse.groupId);
              this.form.findField('a').setValue(this.gavResponse.artifactId);
              this.form.findField('v').setValue(this.gavResponse.version);
              this.form.findField('p').setValue(this.gavResponse.packaging);

              tmpFormChildren.push({
                tag : 'input',
                type : 'hidden',
                name : 'r',
                value : this.gavResponse.repositoryId
              });
            }

            if (pomMode) {
              tmpFormChildren = tmpFormChildren.concat([
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'hasPom',
                  value : 'true'
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'c',
                  value : classifier
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'e',
                  value : extension
                }
              ]);
            } else {
              tmpFormChildren = tmpFormChildren.concat([
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'g',
                  value : this.form.findField('g').getValue()
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'a',
                  value : this.form.findField('a').getValue()
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'v',
                  value : this.form.findField('v').getValue()
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'p',
                  value : this.form.findField('p').getValue()
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'c',
                  value : classifier
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'e',
                  value : extension
                }
              ]);
            }

            tmpFormChildren.push({
              tag : 'input',
              type : 'hidden',
              name : 'desc',
              value : this.form.findField('description').getValue()
            });

            var tmpForm = Ext.getBody().createChild({
                  tag : 'form',
                  cls : 'x-hidden',
                  id : Ext.id(),
                  children : tmpFormChildren
                });

            if (pomMode)
            {
              this.pomInput.appendTo(tmpForm);
            }
            if (fileInput)
            {
              fileInput.appendTo(tmpForm);
            }

            Ext.Ajax.request({
                  url : Sonatype.config.servicePath + '/staging/upload',
                  form : tmpForm,
                  isUpload : true,
                  cbPassThru : {
                    description : this.form.findField('description').getValue(),
                    treePanel : treePanel
                  },
                  callback : function(options, success, response) {
                    tmpForm.remove();

                    // This is a hack to get around the fact that upload submit
                    // always returns
                    // success = true
                    if (response.responseText.toLowerCase().indexOf('<error>') == -1)
                    {
                      this.gavResponse = Ext.decode(response.responseText);
                      if (lastItem)
                      {
                        this.finishRepository(options.cbPassThru.description);
                        this.gavResponse = null;
                      }
                      else
                      {
                        this.currentChildNode++;
                        var treePanel = options.cbPassThru.treePanel;
                        if (this.currentChildNode < treePanel.root.childNodes.length)
                        {
                          this.createUploadForm(treePanel, treePanel.root.childNodes[this.currentChildNode].attributes.payload.fileInput, treePanel.root.childNodes[this.currentChildNode].attributes.payload.classifier,
                              treePanel.root.childNodes[this.currentChildNode].attributes.payload.extension, this.currentChildNode == treePanel.root.childNodes.length - 1);
                        }
                      }
                    }
                    else
                    {
                      var s = 'Artifact upload failed.<br />';
                      var r = response.responseText;
                      var n1 = r.toLowerCase().indexOf('<error>') + 7;
                      var n2 = r.toLowerCase().indexOf('</error>');
                      if (n2 > n1)
                      {
                        s += r.substring(n1, n2);
                      }
                      else
                      {
                        s += 'Check Nexus logs for more information.';
                      }
                      Sonatype.MessageBox.show({
                            title : 'Upload Failed',
                            msg : s,
                            buttons : Sonatype.MessageBox.OK,
                            icon : Sonatype.MessageBox.ERROR
                          });
                    }
                  },
                  scope : this
                });
          },
          getJsonFromHtmlResponse : function(response) {
            var r = response.responseText;
            var n1 = r.toLowerCase().indexOf('<pre>') + 5;
            var n2 = r.toLowerCase().indexOf('</pre>');
            var json = '';
            if (n2 > n1)
            {
              json = r.substring(n1, n2);
            }
            else
            {
              json = r;
            }

            return Ext.util.JSON.decode(json);
          },
          finishRepository : function(description) {
            var profileId = this.gavResponse.profileId;
            var repositoryId = this.gavResponse.repositoryId;

            Ext.Ajax.request({
                  scope : this,
                  method : 'POST',
                  options : {
                    dontForceLogout : true,
                    profileId : profileId,
                    repositoryId : repositoryId
                  },
                  callback : function(options, isSuccess, response) {
                    if (!isSuccess)
                    {
                      // async
                      this.dropRepository(options.options.profileId, options.options.repositoryId);

                      if (response.responseText.indexOf('failures') !== -1)
                      {
                        Sonatype.MessageBox.hide();
                        this.showRulesetEvaluationReport(response.responseText);
                      }
                      else
                      {
                        Sonatype.utils.connectionError(response, 'The server did not close the staging repository, it will be dropped.', false, options.options);
                      }
                    }
                    else
                    {
                      Sonatype.MessageBox.show({
                            title : 'Upload Complete',
                            msg : 'Artifact upload finished successfully',
                            buttons : Sonatype.MessageBox.OK,
                            icon : Sonatype.MessageBox.INFO
                          });
                      this.resetFields();
                    }
                  },
                  jsonData : {
                    data : {
                      stagedRepositoryId : repositoryId,
                      description : description
                    }
                  },
                  url : Sonatype.config.servicePath + '/staging/profiles/' + profileId + '/finish'
                });
          },
          dropRepository : function(profileId, repositoryId) {
            Ext.Ajax.request({
                  scope : this,
                  method : 'POST',
                  options : {
                    dontForceLogout : true
                  },
                  url : Sonatype.config.servicePath + '/staging/profiles/' + profileId + '/drop',
                  jsonData : {
                    data : {
                      stagedRepositoryId : repositoryId
                    }
                  },
                  callback : function(options, isSuccess, response) {
                    if (!isSuccess)
                    {
                      Sonatype.utils.connectionError(response, 'The server did not close the staging repository, and failed to drop it.', false, options.options);
                    }
                  }
                });
          },
          gavDefinitionSelectHandler : function(combo, record, index) {
            var gavDefinitionPanel = this.findById('gav-definition-card-panel');
            var artifactsUploadFieldset = this.findById('artifacts-upload-fieldset');
            // First disable all the items currently on screen, so they wont be
            // validated/submitted etc
            gavDefinitionPanel.getLayout().activeItem.items.each(function(item) {
                  if (item.xtype != 'browsebutton')
                  {
                    item.disable();
                  }
                });
            this.clearInvalid();
            // Then find the proper card to activate (based upon the selected
            // schedule type)
            if (record.data.value == 'pom')
            {
              gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(1));
              artifactsUploadFieldset.setVisible(true);
            }
            else if (record.data.value == 'gav')
            {
              gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(2));
              artifactsUploadFieldset.setVisible(true);
            }
            else if (record.data.value == 'bundle')
            {
              gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(3));
              artifactsUploadFieldset.setVisible(false);
            }
            else
            {
              gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(0));
            }
            // Then enable the fields in that card
            gavDefinitionPanel.getLayout().activeItem.items.each(function(item) {
                  if (item.xtype != 'browsebutton')
                  {
                    item.enable();
                    if (item.readOnly)
                    {
                      item.getEl().dom.readOnly = true;
                    }
                  }
                  else
                  {
                    item.setClipSize();
                  }
                });
            this.doLayout();
          },
          beforeRenderHandler : function(formPanel) {
            var bundleUploadEnabled = Sonatype.lib.Permissions.checkPermission('nexus:stagingbundleupload', Sonatype.lib.Permissions.CREATE);
            var gavDefinitionPanel = this.findById('gav-definition-card-panel');
            gavDefinitionPanel.add({
                  xtype : 'fieldset',
                  border : false,
                  checkboxToggle : false,
                  collapsible : false,
                  style : 'margin-right:7px;',
                  autoHeight : true,
                  buttonAlign : 'center',
                  items : [{
                        hideLabel : true,
                        name : 'upload-bundle-button',
                        xtype : 'browsebutton',
                        text : 'Select Bundle to Upload...',
                        style : 'margin-bottom: 5px;',
                        disabled : !bundleUploadEnabled,
                        uploadPanel : this,
                        handler : function(b) {
                          b.uploadPanel.bundleInput = b.detachInputFile();
                          var bundleName = b.uploadPanel.bundleInput.getValue();
                          b.uploadPanel.updateBundleFileName(b.uploadPanel, bundleName);
                        }
                      }, {
                        xtype : 'textfield',
                        fieldLabel : 'Bundle Filename',
                        name : 'bundle-name-field',
                        anchor : Sonatype.view.FIELD_OFFSET,
                        readOnly : true,
                        allowBlank : false,
                        itemCls : 'required-field',
                        disabled : true
                      }],
                  buttons : [{
                        xtype : 'button',
                        id : 'upload-bundle-button',
                        text : 'Upload Bundle',
                        handler : this.uploadBundleHandler,
                        disabled : !bundleUploadEnabled,
                        scope : this
                      }, {
                        xtype : 'button',
                        id : 'reset-bundle-button',
                        text : 'Reset',
                        handler : this.resetBundleHandler,
                        disabled : !bundleUploadEnabled,
                        scope : this
                      }]
                });
          },
          updateBundleFileName : function(uploadPanel, bundleName) {
            var bundleNameField = uploadPanel.find('name', 'bundle-name-field')[0];
            if (bundleName)
            {
              bundleNameField.setValue(bundleName);
            }
          },
          resetBundleHandler : function() {
            var bundleNameField = this.find('name', 'bundle-name-field')[0];
            bundleNameField.reset();
            this.bundleInput = null;
          },
          uploadBundleHandler : function() {
            if (!this.bundleInput)
            {
              Sonatype.MessageBox.show({
                    title : 'No Bundle Selected',
                    msg : 'You must select a bundle produced by the Maven Repository Plugin.',
                    buttons : Sonatype.MessageBox.OK,
                    icon : Sonatype.MessageBox.ERROR
                  });
            }
            else
            {
              Sonatype.MessageBox.wait('Uploading ...');
              var bundleForm = Ext.getBody().createChild({
                    tag : 'form',
                    cls : 'x-hidden',
                    id : Ext.id(),
                    children : []
                  });
              this.bundleInput.appendTo(bundleForm);
              Ext.Ajax.request({
                    url : Sonatype.config.servicePath + '/staging/bundle_upload',
                    form : bundleForm,
                    isUpload : true,
                    scope : this,
                    callback : function(options, success, response) {
                      // file uploads have to be forced to always succeed for IE, so we have
                      // to parse the response
                      if (response.responseText.toLowerCase().indexOf('<error>') == -1)
                      {
                        if (Ext.isEmpty(response.responseText))
                        {
                          Sonatype.MessageBox.show({
                                title : 'Upload Failed',
                                msg : 'Artifact bundle upload failed.',
                                buttons : Sonatype.MessageBox.OK,
                                icon : Sonatype.MessageBox.ERROR
                              });
                        }
                        else
                        {
                          var jsonResponse = Ext.decode(response.responseText);

                          var msg = 'Artifact Bundle upload finished successfully<br>';

                          if (jsonResponse.repositoryUris.length == 1)
                          {
                            msg += 'The following staging repository has been created<br><br>';
                          }
                          else if (jsonResponse.repositoryUris.length > 1)
                          {
                            msg += 'The following staging repositories have been created<br><br>';
                          }
                          for (var i = 0; i < jsonResponse.repositoryUris.length; i++)
                          {
                            msg += '<a href="' + jsonResponse.repositoryUris[i] + '" target="_new">' + jsonResponse.repositoryUris[i] + '</a>'
                          }

                          Sonatype.MessageBox.show({
                                title : 'Upload Complete',
                                msg : msg,
                                buttons : Sonatype.MessageBox.OK,
                                icon : Sonatype.MessageBox.INFO
                              });
                        }
                      }
                      else
                      {
                        var s = 'Artifact upload failed.<br />';
                        var r = response.responseText;
                        var n1 = r.toLowerCase().indexOf('<error>') + 7;
                        var n2 = r.toLowerCase().indexOf('</error>');
                        if (n2 > n1)
                        {

                          if (r.indexOf('failures') !== -1)
                          {
                            Sonatype.MessageBox.hide();
                            // there is markup like <b> in the rule messages that is 'double encoded' because of
                            // weirdo upload error handling
                            this.showRulesetEvaluationReport(Nexus.util.Format.htmlDecode(r.substring(n1, n2)));
                            s = null;
                          } else {
                            s += r.substring(n1, n2);
                          }
                        }
                        else
                        {
                          s += 'Check Nexus logs for more information.';
                        }

                        if (s)
                        {
                          Sonatype.MessageBox.show({
                                title : 'Upload Failed',
                                msg : s,
                                buttons : Sonatype.MessageBox.OK,
                                icon : Sonatype.MessageBox.ERROR
                              });
                        }
                      }
                    }
                  });
            }
          }
        });
    Nexus.staging.Rules.treat(Sonatype.repoServer.StagingUploadPanel);

    Sonatype.Events.addListener('nexusNavigationInit', function(nexusPanel) {
          var enable = Sonatype.lib.Permissions.checkPermission('nexus:stagingupload', Sonatype.lib.Permissions.CREATE) ||
                 Sonatype.lib.Permissions.checkPermission('nexus:stagingbundleupload', Sonatype.lib.Permissions.CREATE);
          nexusPanel.add({
                enabled : enable,
                sectionId : 'st-enterprise-staging',
                title : 'Staging Upload',
                tabId : 'staging-upload',
                tabCode : Sonatype.repoServer.StagingUploadPanel
              });
        });

});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype*/

define('Sonatype/repoServer/RulesetEditor', ['Nexus/staging/rule/RuleTypeStore'], function() {

    Sonatype.repoServer.RulesetEditor = function(cfg) {
      var
            config = cfg || {},
            defaultConfig = {
              region : 'center',
              width : '100%',
              height : '100%',
              autoScroll : true,
              border : false,
              frame : true,
              collapsible : false,
              collapsed : false,
              buttonAlign : 'center'
            },
            checkColumn = new Ext.grid.CheckColumn({
              header : 'Enabled',
              dataIndex : 'enabled',
              width : 60
            });

      Ext.apply(this, config, defaultConfig);

      this.isNew = this.payload.data.id ? false : true;

      // not using mediators store here since we want to filter it specially to hide some stuff
      this.ruleTypeDataStore = NX.create('Nexus.staging.rule.RuleTypeStore', {
          autoLoad: true,
          filterTesting: window.location.search !== '?debug' // filter out testing.* rules unless ?debug
      });

      this.ruleStore = new Ext.data.SimpleStore({
            id : 0,
            fields : [{
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }, {
                  name : 'enabled'
                }, {
                  name : 'typeId'
                }, {
                  name : 'typeName'
                }, {
                  name : 'properties'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            }
          });

      Sonatype.repoServer.RulesetEditor.superclass.constructor.call(this, {
            items : [{
                  xtype : 'hidden',
                  name : 'id'
                }, {
                  xtype : 'textfield',
                  fieldLabel : 'Name',
                  htmlDecode : true,
                  helpText : 'The name of the rule set',
                  itemCls : 'required-field',
                  name : 'name',
                  width : 250,
                  allowBlank : false
                }, {
                  xtype : 'textfield',
                  fieldLabel : 'Description',
                  htmlDecode : true,
                  helpText : 'The description of the rule set',
                  name : 'description',
                  width : 250
                }, {
                  xtype : 'fieldset',
                  title : 'Rules',
                  name : 'rulesFieldSet',
                  collapsible : false,
                  autoHeight : true,
                  items : [{
                        xtype : 'panel',
                        layout : 'table',
                        layougConfig : {
                          columns : 2
                        },
                        style : 'padding-bottom: 10px',
                        items : [{
                              xtype : 'combo',
                              fieldLabel : 'Rule Type',
                              helpText : 'The type of the rule',
                              name : 'ruleType',
                              width : 530,
                              store : this.ruleTypeDataStore,
                              clearFilterOnReset: false, // do not reset the store filter
                              displayField : 'name',
                              valueField : 'id',
                              editable : false,
                              emptyText : 'Select...',
                              selectOnFocus : true,
                              triggerAction : 'all',
                              lastQuery: '', // do not clear filter on first load
                              forceSelection : true,
                              itemCls : 'required-field',
                              mode : 'local'
                            }, {
                              xtype : 'button',
                              name : 'ruleAdd',
                              text : 'Add',
                              handler : this.ruleAddHandler,
                              scope : this,
                              style : 'padding-left: 30px'
                            }]
                      }, {
                        xtype : 'editorgrid',
                        name : 'rulesGrid',
                        title : 'Applied Rules',
                        autoExpandColumn : 'type-column',
                        width : 600,
                        height : 160,
                        frame : true,
                        store : this.ruleStore,
                        plugins : checkColumn,
                        colModel : new Ext.grid.ColumnModel({
                              defaults : {
                                sortable : true
                              },
                              columns : [checkColumn, {
                                    id : 'type-column',
                                    header : 'Type',
                                    dataIndex : 'typeName'
                                  }, {
                                    width : 30,
                                    renderer : function(value, meta, record, row, col, store) {
                                      return '<img src="' + Sonatype.config.extPath + '/resources/images/gray/tabs/tab-close.gif" height="16" width="16" alt="Delete" />';
                                    }
                                  }]
                            }),
                        selModel : new Ext.grid.RowSelectionModel({
                              singleSelect : true
                            }),
                        listeners : {
                          cellclick : {
                            fn : function(grid, rowIndex, columnIndex, event) {
                              if (columnIndex === 2)
                              {
                                var record = grid.getStore().getAt(rowIndex);
                                grid.getStore().remove(record);
                              }
                            },
                            scope : this
                          }
                        }
                      }]
                }],
            buttons : [{
                  name : 'rulesetSave',
                  text : 'Save',
                  handler : this.saveHandler,
                  scope : this
                }, {
                  name : 'rulesetCancel',
                  text : this.isNew ? 'Cancel' : 'Reset',
                  handler : this.isNew ? this.cancelHandler : this.resetHandler,
                  scope : this
                }]
          });

      this.on('afterlayout', this.initData, this);

      this.initialized = false;
    };

    Ext.extend(Sonatype.repoServer.RulesetEditor, Ext.form.FormPanel, {
          initData : function() {
            if (!this.initialized)
            {
              var
                    i, ruleRec,
                    idField = this.find('name', 'id')[0],
                    nameField = this.find('name', 'name')[0],
                    descriptionField = this.find('name', 'description')[0];

              this.ruleStore.removeAll();

              idField.setValue(this.payload.data.id);
              nameField.setValue(this.payload.data.name);
              descriptionField.setValue(this.payload.data.description);

              for (i = 0; i < this.payload.data.rules.length; i = i + 1)
              {
                ruleRec = new Ext.data.Record(this.payload.data.rules[i], this.payload.data.rules[i].name);
                this.ruleStore.addSorted(ruleRec);
              }

              if (this.isNew)
              {
                nameField.setRawValue('');
                descriptionField.setRawValue('');
              }

              this.initialized = true;
            }
          },

          ruleAddHandler : function() {
            if (!this.validateRuleFields())
            {
              return;
            }

            var
                  typeId = this.find('name', 'ruleType')[0].getValue(),
                  typeName = this.ruleTypeDataStore.getById(typeId).data.name,
                  rec = new Ext.data.Record({}, typeName);

            rec.beginEdit();
            rec.set('enabled', true);
            rec.set('name', typeName);
            rec.set('typeId', typeId);
            rec.set('typeName', typeName);
            rec.commit();
            rec.endEdit();
            this.ruleStore.addSorted(rec);

            this.resetRuleFields();

            this.clearRuleGridInvalid();
          },

          validateRuleFields : function() {
            var
                  records, i,
                  typeCombo = this.find('name', 'ruleType')[0];

            if (typeCombo.getValue() === '')
            {
              typeCombo.markInvalid('This field is required');
              return false;
            }

            records = this.ruleStore.getRange();

            for (i = 0; i < records.length; i = i+1)
            {
              if (records[i].get('typeId') === typeCombo.getValue())
              {
                typeCombo.markInvalid('This Rule is already in the ruleset');
                return false;
              }
            }

            return true;
          },

          resetRuleFields : function() {
            this.find('name', 'ruleType')[0].reset();
          },

          cancelHandler : function() {
            this.fireEvent('cancel', this);
          },
          resetHandler : function() {
            this.initialized = false;
            this.initData();
          },
          saveHandler : function() {
            if (!this.validateRuleSet())
            {
              return;
            }

            /*
              The payload is the record in the upper grid, updating it will trigger the rulesetViewInit event,
              which will in turn recreate the below ruleSetEditor. We could show the saved values before the server
              roundtrip this way, but the grid is updated in the ajax callback as well.

            this.payload.beginEdit();
            this.payload.set('name', this.find('name', 'name')[0].getValue());
            this.payload.set('description', this.find('name', 'description')[0].getValue());
            this.payload.set('rules', this.getRulesData());
            this.payload.commit();
            this.payload.endEdit();
            */

            var
                  submitUrl = this.isNew ? Sonatype.config.servicePath + '/staging/rule_sets' : Sonatype.config.servicePath + '/staging/rule_sets/' + this.payload.data.id,
                  submitData = { data : {} };

            if (this.isNew) {
              submitData.data.id = '';
            } else {
              submitData.data.id = this.payload.data.id;
            }
            submitData.data.name =  this.find('name', 'name')[0].getValue();
            submitData.data.description = this.find('name', 'description')[0].getValue();
            submitData.data.rules = this.getRulesData();

            Ext.Ajax.request({
                  scope : this,
                  method : this.isNew ? 'POST' : 'PUT',
                  url : submitUrl,
                  jsonData : Ext.util.JSON.encode(submitData),
                  callback : function(options, isSuccess, response) {
                    var errMsg, recData;

                    if (!isSuccess)
                    {
                      errMsg = this.isNew ? 'Unable to create the rule set.' : 'Unable to update the rule set.';
                      Sonatype.utils.connectionError(response, errMsg, false, options.options);
                    }
                    else
                    {
                      recData = Ext.util.JSON.decode(response.responseText).data;
                      this.updateGrid(recData);
                    }
                  }
                });

          },
          getRulesData : function() {
            var rulesData = [], i;
            for (i = 0; i < this.ruleStore.getCount(); i = i+1)
            {
              rulesData[i] = this.ruleStore.getAt(i).data;
            }
            return rulesData;
          },
          validateRuleSet : function() {
            var
                  nameField = this.find('name', 'name')[0],
                  currentIndex, existingIndex;

            if (!nameField.validate())
            {
              return false;
            }
            if (this.ruleStore.getCount() === 0)
            {
              this.markRuleGridInvalid('At least one rule should be specified!');
              return false;
            }

            currentIndex = this.payload.store.indexOf(this.payload);
            existingIndex = this.payload.store.find('name', new RegExp('^' + nameField.getValue() + '$'));
            if (this.isNew)
            {
              existingIndex = this.payload.store.find('name', new RegExp('^' + nameField.getValue() + '$'), 1);
            }
            if (existingIndex !== -1 && existingIndex !== currentIndex)
            {
              nameField.markInvalid('A rule set with the same name already exists.');
              return false;
            }
            return true;
          },

          clearRuleGridInvalid : function() {
            var
                  rulesFS = this.find('name', 'rulesFieldSet')[0],
                  rulesWarning = rulesFS.find('name', 'rulesWarning')[0];

            if (rulesWarning)
            {
              rulesFS.remove(rulesWarning);
            }
          },

          markRuleGridInvalid : function(msg) {
            this.clearRuleGridInvalid();
            var rulesFS = this.find('name', 'rulesFieldSet')[0];
            rulesFS.insert(1, {
                  xtype : 'panel',
                  name : 'rulesWarning',
                  layout : 'fit',
                  html : '<div style="padding-bottom:10px; padding-top:5px; color:red">' + 'WARNING: ' + msg + '</div>'
                });
            rulesFS.doLayout();
          },

          updateGrid : function(data) {
            var rec, pos, store = this.payload.store;

            store.remove(this.payload);
            rec = new Ext.data.Record(data, data.resourceURI);
            store.addSorted(rec);
            this.isNew = false;
            this.payload.autoCreateNewRecord = false;

            pos = store.indexOf(rec);
            this.grid.gridPanel.getSelectionModel().selectRow(pos);
          }
        });

    Sonatype.Events.addListener('rulesetViewInit', function(cardPanel, rec, gridPanel) {
          var sp = Sonatype.lib.Permissions;
          if (sp.checkPermission('nexus:stagingrulesets', sp.READ) && (sp.checkPermission('nexus:stagingrulesets', sp.CREATE) || sp.checkPermission('nexus:stagingrulesets', sp.DELETE) || sp.checkPermission('nexus:stagingrulesets', sp.EDIT)))
          {
            cardPanel.add(new Sonatype.repoServer.RulesetEditor({
                  name : 'configuration',
                  payload : rec,
                  grid : gridPanel
                }));
          }
        });
});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype*/

define('Sonatype/repoServer/StagingRuleSet', function() {

    Sonatype.repoServer.StagingRuleSet = function(cfg) {
      var
            config = cfg || {},
            defaultConfig = {
              title : 'Staging Ruleset'
            },
            sp = Sonatype.lib.Permissions,
            toolbar = [];

      Ext.apply(this, config, defaultConfig);

      Sonatype.repoServer.StagingRuleSet.superclass.constructor.call(this, {
            url : Sonatype.config.servicePath + '/staging/rule_sets',
            tbar : toolbar,
            dataAutoLoad : false,
            tabbedChildren : true,
            addMenuInitEvent : 'rulesetAddMenuInit',
            deleteButton : sp.checkPermission('nexus:stagingrulesets', sp.DELETE),
            rowClickEvent : 'rulesetViewInit',
            dataSortInfo : {
              field : 'name',
              direction : 'asc'
            },
            columns : [{
                  name : 'resourceURI'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  header : 'Name',
                  width : 100
                }, {
                  name : 'description',
                  header : 'Description',
                  width : 200
                }, {
                  name : 'rules'
                }]
          });

      this.dataStore.load();
    };

    Ext.extend(Sonatype.repoServer.StagingRuleSet, Sonatype.panels.GridViewer, {
      emptyText: 'No rulesets defined',
      emptyTextWhileFiltering: 'No rulesets matched criteria: {criteria}'
    });

    Sonatype.Events.addListener('nexusNavigationInit', function(nexusPanel) {
          nexusPanel.add({
                enabled : Sonatype.lib.Permissions.checkPermission('nexus:stagingrulesets', Sonatype.lib.Permissions.READ),
                sectionId : 'st-enterprise-staging',
                title : 'Staging Ruleset',
                tabId : 'staging-ruleset',
                tabCode : Sonatype.repoServer.StagingRuleSet
              });
        });

    Sonatype.Events.addListener('rulesetAddMenuInit', function(menu) {
      var sp = Sonatype.lib.Permissions;

      if (sp.checkPermission('nexus:stagingrulesets', sp.CREATE)) {
        menu.add(['-', {
          text : 'Ruleset',
          autoCreateNewRecord : true,
          handler : function(container, rec, item, e) {
            rec.beginEdit();
            rec.set('resourceURI', 'new');
            rec.set('rules', []);
            rec.commit();
            rec.endEdit();
          }
        }]);
      }
    });

});

